<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Traits\PartyTrait;
use App\Http\Traits\PriceTrait;
use App\Http\Traits\UserTrait;
use App\Models\AccountClose;
use App\Models\Party;
use App\Models\PartyBalance;
use App\Models\Vatav;
use App\Models\VatavBackUp;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Carbon\Carbon;
use Illuminate\Support\Str;
use Mpdf\Mpdf as PDF;

class AjaxController extends Controller
{
    use PartyTrait;
    use PriceTrait;
    use UserTrait;

    public function __construct()
    {
        $this->middleware('auth');
    }

    public function get_general_report_list(Request $request, $print): void
    {
        $reqData = $request->all();
        $date = $reqData['date'];
        $particular_date = date('Y-m-d', strtotime($date));
        $accessPartyDetail = $this->getAccessParty();
        $accessParty = explode(",", $accessPartyDetail);
        $role = session('role');

        $partyArr = array();
        $resultSet = array();
        $partyByIds = array();

        $partyData = Party::where('is_deleted', 0)->get();

        foreach ($partyData as $_party) {
            $partyByIds[$_party->id] = $_party;
        }

        $partyBalanc = "SELECT * FROM `party_balance` pb, (SELECT max(date) as dt, party_id from party_balance WHERE DATE(`date`) <= '$particular_date' GROUP by party_id) pbb where pb.date = pbb.dt and pb.party_id = pbb.party_id ORDER BY `pb`.`id` DESC";
        $partyBalanceData = DB::select($partyBalanc);


        foreach ($partyBalanceData as $_partyBalance) {
            $allAccount = AccountClose::where('party_id', $_partyBalance->party_id)
                ->where('vatav_id', $_partyBalance->vatav_id)
                ->first();
            if (in_array($_partyBalance->party_id, $partyArr)) {

            } else {
                if ($role === 'superadmin') {
                    if ($_partyBalance->party_id == $partyByIds[$_partyBalance->party_id]->id) {
                        if (isset($allAccount) && !empty($allAccount)) {
                            $_partyBalance->is_close = $allAccount->is_close;
                        } else {
                            $_partyBalance->is_close = 0;
                        }
                        $_partyBalance->name = $partyByIds[$_partyBalance->party_id]->name;
                        $_partyBalance->code = $partyByIds[$_partyBalance->party_id]->code;
                        $partyArr[] = $_partyBalance->party_id;
                        $resultSet[] = $_partyBalance;
                    }
                } else {
                    if (in_array($_partyBalance->party_id, $accessParty)) {
                        if ($_partyBalance->party_id == $partyByIds[$_partyBalance->party_id]->id) {
                            if (isset($allAccount) && !empty($allAccount)) {
                                $_partyBalance->is_close = $allAccount->is_close;
                            } else {
                                $_partyBalance->is_close = 0;
                            }
                            $_partyBalance->name = $partyByIds[$_partyBalance->party_id]->name;
                            $_partyBalance->code = $partyByIds[$_partyBalance->party_id]->code;
                            $partyArr[] = $_partyBalance->party_id;
                            $resultSet[] = $_partyBalance;
                        }
                    }
                }
            }
        }

        $column_key = 'code';
        $sortType = SORT_ASC;

        $volume = array_column($resultSet, $column_key);
        array_multisort($volume, $sortType, $resultSet);

        $negativeArr = array();
        $positiveArr = array();
        $length_positive_arr = 0;
        $length_negative_arr = 0;
        $arrayCount = 0;
        foreach ($resultSet as $_party) {
            if ($_party->closing_balance < 0 and $_party->closing_balance != '') {
                $negativeArr[] = $_party;
            } elseif ($_party->closing_balance > 0 and $_party->closing_balance != '') {
                $positiveArr[] = $_party;
            }
        }
        if (!empty($positiveArr)) {
            $length_positive_arr = count($positiveArr);
        }
        if (!empty($negativeArr)) {
            $length_negative_arr = count($negativeArr);
        }

        $arrayCount = max($length_positive_arr, $length_negative_arr);
        if ($print == 1) {
            $data = view('pages.party.general', compact('arrayCount', 'positiveArr', 'negativeArr'))->render();
            echo $data;
            exit;
        } else {
            $input_parmeter = array(
                'date' => $date,
                'downloaded_by' => session('user_name'),
            );
            $data['inputs'] = $input_parmeter;
            $data['arrayCount'] = $arrayCount;
            $data['positiveArr'] = $positiveArr;
            $data['negativeArr'] = $negativeArr;
            $html = view('pages.reportdownload.general', compact('data'))->render();
            $pdfFilePath = 'General_report_' . date('Y_m_d_H_i_s') . '.pdf';
            ob_end_clean();
            $document = new PDF([
                'mode' => 'utf-8',
                'format' => 'A4',
                'margin_header' => '3',
                'margin_top' => '20',
                'margin_bottom' => '20',
                'margin_footer' => '2',
            ]);

            // Set some header information for output
            $header = [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' => 'inline; filename="' . $pdfFilePath . '"'
            ];
            $document->WriteHTML($html);
            $document->Output($pdfFilePath, "D");
        }
    }

    public function get_party_list(Request $request, $print): void
    {
        $role = session('role');
        $party = $request->post('party');
        $date = $request->post('date_range');
        $date_range = explode(" to ", $date);

        $start_date = date('Y-m-d', strtotime($date_range[0]));
        $end_date = date('Y-m-d', strtotime($date_range[1]));

        $query = Vatav::where('ho_approve', 1)
            ->where('is_deleted', '=', 0)
            ->where(function ($q) use ($party) {
                $q->orWhere('sender_party_id', $party)
                    ->orWhere('receiver_party_id', $party);
            })
            ->select('*', 'receiver as opp');
        $query = $query->whereBetween(DB::raw("DATE(date)"), [$start_date, $end_date]);

        $query = $query->orderBy("date", 'ASC')
            ->orderBy("serial_no", 'ASC');

        $queryBackup = VatavBackUp::where('ho_approve', 1)
            ->where('is_deleted', 0)
            ->where(function ($q) use ($party) {
                $q->where('sender_party_id', $party)
                    ->orWhere('receiver_party_id', $party);
            })
            ->select('*', 'receiver as opp');

        $queryBackup = $queryBackup->whereBetween(DB::raw("DATE(date)"), [$start_date, $end_date]);

        $queryBackup = $queryBackup->orderBy("date", 'ASC')
            ->orderBy("serial_no", 'ASC');

        $partyList = $query->union($queryBackup)->get();

        $partyDetails = $this->getPartyDetailsFromId($party);

        $data = array();
        $ceCount = 0;
        $deCount = 0;
        $closing_balance = 0;

        $data['opening_balance'] = $this->getPartyOpeningBalance($party, $start_date);

        $current_balance_vat = $data['opening_balance'];
        $final_arr = array();
        foreach ($partyList as $_allParties) {
            $sender = explode(' (', $_allParties->sender);
            $receiver = explode(' (', $_allParties->opp);

            if ($_allParties->sender_party_id == $party) {
                $n_arr['opposite_party'] = $receiver[1];
                $n_arr['sender_party'] = $_allParties->sender;
                if ($_allParties->type == "namu") {
                    $n_arr['debit'] = $_allParties->sender_total_amount;
                    $n_arr['credit'] = '';
                    $current_balance_vat += $n_arr['debit'];
                    $deCount++;
                } else {
                    $n_arr['debit'] = '';
                    $n_arr['credit'] = $_allParties->sender_total_amount;
                    $current_balance_vat += $n_arr['credit'];
                    $ceCount++;
                }
                $allAccount = AccountClose::where('party_id', $_allParties->sender_party_id)
                    ->where('vatav_id', $_allParties->id)
                    ->first();

                if (!empty($allAccount)) {
                    $n_arr['is_close'] = $allAccount->is_close;
                } else {
                    $n_arr['is_close'] = 0;
                }
            }
            if ($_allParties->receiver_party_id == $party) {
                $n_arr['opposite_party'] = $sender[1];
                $n_arr['sender_party'] = $_allParties->opp;
                if ($_allParties->type == "namu") {
                    $n_arr['credit'] = $_allParties->receiver_total_amount;
                    $n_arr['debit'] = '';
                    $current_balance_vat += $n_arr['credit'];
                    $ceCount++;
                } else {
                    $n_arr['credit'] = '';
                    $n_arr['debit'] = $_allParties->receiver_total_amount;
                    $current_balance_vat += $n_arr['debit'];
                    $deCount++;
                }
                $allAccount = AccountClose::where('party_id', $_allParties->receiver_party_id)
                    ->where('vatav_id', $_allParties->id)
                    ->first();

                if (!empty($allAccount)) {
                    $n_arr['is_close'] = $allAccount->is_close;
                } else {
                    $n_arr['is_close'] = 0;
                }
            }

            $n_arr['entered_by'] = $_allParties->entered_by;
            $n_arr['remarks'] = $_allParties->remarks;
            $n_arr['date'] = $_allParties->date;
            $n_arr['serial_no'] = $_allParties->serial_no;
            $n_arr['current_balance'] = number_format($current_balance_vat, "2", ".", "");

            $n_arr['opening_balance'] = number_format($_allParties->opening_balance, "2", ".", "");
            $n_arr['id'] = $_allParties->id;

            $n_arr['party_id'] = $party;
            $n_arr['sender_name'] = $_allParties->sender_name;
            $n_arr['sender_contact'] = $_allParties->sender_contact;
            $n_arr['receiver_name'] = $_allParties->receiver_name;
            $n_arr['receiver_contact'] = $_allParties->receiver_contact;
            $n_arr['token'] = $_allParties->token;

            if ($role == 'superadmin') {
                $n_arr['is_delete'] = 1;
            } else {
                if (strtotime(date('Y-m-d', strtotime($_allParties->date))) == strtotime(date('Y-m-d'))) {
                    $n_arr['is_delete'] = 1;
                } else {
                    $n_arr['is_delete'] = 0;
                }
            }
            $closing_balance = $n_arr['current_balance'];
            $final_arr[] = $n_arr;
        }


        $data['allParties'] = $final_arr;
        $data['party_name'] = $partyDetails->code . " (" . $partyDetails->name . ")";
        $data['ceCount'] = $ceCount;
        $data['deCount'] = $deCount;
        $data['closing_balance'] = $closing_balance;

        if ($print == '1') {
            $data = view('pages.party.partyreport', compact("data"))->render();
            echo $data;
            exit;
        } else {
            $input_parmeter = array(
                'party_name' => $partyDetails->code . " (" . $partyDetails->name . ")",
                'date' => $date,
                'downloaded_by' => session('user_name')
            );
            $data_arr['inputs'] = $input_parmeter;
            $data_arr['opening_balance'] = $data['opening_balance'];
            $pdfFilePath = 'Party_report_' . date('Y_m_d_H_i_s') . '.pdf';
            ob_end_clean();
            $document = new PDF([
                'mode' => 'utf-8',
                'format' => 'A4',
                'margin_header' => '3',
                'margin_top' => '20',
                'margin_bottom' => '20',
                'margin_footer' => '2',
            ]);

            // Set some header information for output
            $header = [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' => 'inline; filename="' . $pdfFilePath . '"'
            ];
            $data_arr['allParties'] = $data['allParties'];
            $html = view('pages.reportdownload.partyreport', compact("data_arr"))->render();
            $html = mb_convert_encoding($html, 'UTF-8', 'UTF-8');
            $document->WriteHTML($html);
            $document->Output($pdfFilePath, "D");
        }

    }

    public function get_vatav_from_id(Request $request)
    {
        $party = $request->input('party');
        $vatav_id = $request->input('vatav_id');
        $vatavData = Vatav::find($vatav_id);
        $f_vatav = '';
        $remarks = '';
        if (!empty($vatavData)) {
            $remarks = $vatavData->remarks;
            if ($vatavData->sender == $party) {
                if ($vatavData->type == 'auto' or $vatavData->type == 'manual') {
                    $amount = $vatavData->amount;
                    $calculate_vatav_on = $this->generate_price_range($amount * 100);

                    $sender_vatav = $vatavData->s_vatav;
                    $f_vatav = $calculate_vatav_on - ($sender_vatav * 100);
                    $f_vatav = $f_vatav / 100;
                } else {
                    $f_vatav = 0;
                }
            } else {
                $f_vatav = $vatavData->r_vatav;
            }
        }
        $data['remarks'] = $remarks;
        $data['vatav'] = (float)$f_vatav;
        echo json_encode($data);

    }

    public function get_day_report(Request $request, $print)
    {
        $reqData = $request->all();
        $particular_date = $reqData['date'];
        $particular_date = date('Y-m-d', strtotime($particular_date));

        $accessParty = $this->getAccessParty();
        $accessPartyArray = explode(",", $accessParty);

        $query = Party::join('vatav', function ($join) {
            $join->on('vatav.sender_party_id', '=', 'party.id')
                ->orOn('vatav.receiver_party_id', 'party.id');
        })
            ->where('vatav.ho_approve', 1)
            ->where('vatav.is_deleted', '=', 0)
            ->where('party.is_deleted', '=', 0)
            ->whereIn('party.id', $accessPartyArray)
            ->select('party.id', 'party.code', 'party.name', 'vatav.date', 'vatav.sender_party_id', 'vatav.receiver_party_id', DB::raw('SUM(vatav.sender_total_amount) as sender_balance'), DB::raw('SUM(vatav.receiver_total_amount) as receiver_balance'));
        $query = $query->where('date', "LIKE", $particular_date . '%');

//        $query = $query->groupBy('party.id');
        $query = $query->groupBy('party.code');
//        $query = $query->groupBy('party.name');
//        $query = $query->groupBy('vatav.date');
//        $query = $query->groupBy('vatav.sender_party_id');
//        $query = $query->groupBy('vatav.receiver_party_id');

        $queryBackup = Party::join('vatavBackUp', function ($join) {
            $join->on('vatavBackUp.sender_party_id', '=', 'party.id')
                ->orOn('vatavBackUp.receiver_party_id', '=', 'party.id');
        })
            ->where('vatavBackUp.ho_approve', 1)
            ->where('vatavBackUp.is_deleted', 0)
            ->where('party.is_deleted', 0)
            ->whereIn('party.id', $accessPartyArray)
            ->select('party.id', 'party.code', 'party.name', 'vatavBackUp.date', 'vatavBackUp.sender_party_id', 'vatavBackUp.receiver_party_id', DB::raw('SUM(vatavBackUp.sender_total_amount) as sender_balance'), DB::raw('SUM(vatavBackUp.receiver_total_amount) as receiver_balance'));

        $queryBackup = $queryBackup->where('date', 'LIKE', $particular_date . '%');

        $queryBackup = $queryBackup->groupBy('party.id');
        $queryBackup = $queryBackup->groupBy('party.code');
        $queryBackup = $queryBackup->groupBy('party.name');
        $queryBackup = $queryBackup->groupBy('vatavBackUp.date');
        $queryBackup = $queryBackup->groupBy('vatavBackUp.sender_party_id');
        $queryBackup = $queryBackup->groupBy('vatavBackUp.receiver_party_id');

        // $daywiseList = $query->get();
        $daywiseList = $query->union($queryBackup)->get();

        $final_arr = array();
        foreach ($daywiseList as $value) {
            $n_arr['code'] = $value->code;
            $n_arr['name'] = $value->name;

            if ($value->sender_party_id == $value->id) {
                $n_arr['total_amount'] = number_format($value->sender_balance, '2', '.', '');
            }

            if ($value->receiver_party_id == $value->id) {
                $n_arr['total_amount'] = number_format($value->receiver_balance, '2', '.', '');
            }
            $final_arr[] = $n_arr;
        }

        $negativeArray = array();
        $positiveArray = array();
        $positiveCount = 0;
        $negativeCount = 0;
        $arrayCount = 0;

        foreach ($final_arr as $key => $row) {
            if ($row['total_amount'] < 0) {
                $negativeArray[] = $row;
                $negativeCount++;
            } else {
                $positiveArray[] = $row;
                $positiveCount++;
            }
        }

        $arrayCount = max($negativeCount, $positiveCount);
        if ($print == '1') {
            $data = view('pages.party.dayreport', compact('negativeArray', 'positiveArray', 'arrayCount'))->render();
            echo $data;
            exit;
        } else {
            $input_parmeter = array(
                'date' => $particular_date,
                'downloaded_by' => session('user_name')
            );
            $data['inputs'] = $input_parmeter;
            $data['negativeArray'] = $negativeArray;
            $data['positiveArray'] = $positiveArray;
            $data['arrayCount'] = $arrayCount;
            $html = view('pages.reportdownload.dayreport', compact('data'))->render();
            $pdfFilePath = 'Day_report_' . date('Y_m_d_H_i_s') . '.pdf';
            ob_end_clean();
            $document = new PDF([
                'mode' => 'utf-8',
                'format' => 'A4',
                'margin_header' => '3',
                'margin_top' => '20',
                'margin_bottom' => '20',
                'margin_footer' => '2',
            ]);

            // Set some header information for output
            $header = [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' => 'inline; filename="' . $pdfFilePath . '"'
            ];
            $document->WriteHTML($html);
            $document->Output($pdfFilePath, "D");
        }
    }

    public function get_day_entry_report(Request $request, $print)
    {
        $reqData = $request->all();

        $amount = $reqData['amount'];
        $is_deleted = $reqData['is_deleted'] ?? 0;
        $date = $reqData['date_range'];
        $date_range = explode(" to ", $date);

        $start_date = date('Y-m-d', strtotime($date_range[0]));
        $end_date = date('Y-m-d', strtotime($date_range[1]));

        $vatavData = array();
        $query = Vatav::where('ho_approve', 1)->orderBY('id', 'asc');

        if ($amount != '' && $amount != 0.00) {
            $query = $query->where('amount', $amount);
        }

        if ($is_deleted == 1) {
            $query = $query->where('is_deleted', 1);
            if ($start_date == $end_date) {
                $query = $query->where('deleted_time', "LIKE", $start_date . '%');
            } else {
                $query = $query->whereBetween(DB::raw("DATE(deleted_time)"), [$start_date, $end_date]);
            }
            $vatavData = $query->onlyTrashed()->get();
        } else {
            $query = $query->where('is_deleted', 0)->where('ho_approve', 1);
            if ($start_date == $end_date) {
                $query = $query->where("date", "LIKE", $start_date . '%');
            } else {
                $query = $query->whereBetween(DB::raw("DATE(date)"), [$start_date, $end_date]);
            }
            // $vatavData = $query->get();
        }

        $queryBackup = VatavBackUp::where('ho_approve', 1)->orderBY('id', 'asc');

        if ($amount != '' && $amount != 0.00) {
            $queryBackup = $queryBackup->where('amount', $amount);
        }

        if ($is_deleted == 1) {
            $queryBackup = $queryBackup->where('is_deleted', 1);
            if ($start_date == $end_date) {
                $queryBackup = $queryBackup->where('deleted_time', 'LIKE', $start_date . '%');
            } else {
                $queryBackup = $queryBackup->whereBetween(DB::raw("DATE(deleted_time)"), [$start_date, $end_date]);
            }
            $vatavBackupData = $queryBackup->onlyTrashed()->get();
        } else {
            $queryBackup = $queryBackup->where('is_deleted', 0)->where('ho_approve', 1);
            if ($start_date == $end_date) {
                $queryBackup = $queryBackup->where("date", "LIKE", $start_date . '%');
            } else {
                $queryBackup = $queryBackup->whereBetween(DB::raw("DATE(date)"), [$start_date, $end_date]);
            }
            // $vatavBackupData = $queryBackup->get();
        }
        // dd($vatavBackupData);
        $vatavData = $query->union($queryBackup)->get();
        // $combinedData = $vatavData->union($vatavBackupData);

        foreach ($vatavData as $row) {
            $sender = explode('(', $row->sender);
            $row->sender_code_name = rtrim($sender[1], ")");
            $row->sender_code = trim($sender[0]);

            $receiver = explode('(', $row->receiver);
            $row->receiver_code_name = rtrim($receiver[1], ")");
            $row->receiver_code = trim($receiver[0]);
        }

        if ($print == '1') {
            $data = view('pages.party.dayentryreport', compact('vatavData'))->render();
            echo $data;
            exit;
        } else {
            $input_parmeter = array(
                'amount' => $amount,
                'date' => $date,
                'is_deleted' => $is_deleted,
                'downloaded_by' => session('user_name')
            );
            $data['inputs'] = $input_parmeter;
            $data['vatavData'] = $vatavData;
            $html = view('pages.reportdownload.dayentryreport', compact('data'))->render();
            $pdfFilePath = 'Day_Entry_report_' . date('Y_m_d_H_i_s') . '.pdf';
            ob_end_clean();
            $document = new PDF([
                'mode' => 'utf-8',
                'format' => 'A4',
                'margin_header' => '3',
                'margin_top' => '20',
                'margin_bottom' => '20',
                'margin_footer' => '2',
            ]);

            // Set some header information for output
            $header = [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' => 'inline; filename="' . $pdfFilePath . '"'
            ];
            $document->WriteHTML($html);
            $document->Output($pdfFilePath, "D");
        }
    }

    public function get_city_report(Request $request, $print)
    {
        $party_by_ids = array();
        $partyList = Party::where('is_deleted', 0)->get();
        foreach ($partyList as $_party) {
            $party_by_ids[$_party->id] = $_party;
        }

        $citySql = "SELECT * FROM `party_balance` pb,(SELECT MAX(DATE_ADD(pba.date, INTERVAL  pba.id SECOND)) as dt, ipbb.party_id, pba.id FROM `party_balance` pba, (SELECT max(date) as idt, party_id from party_balance GROUP by party_id) as ipbb WHERE ipbb.idt = pba.date AND ipbb.party_id = pba.party_id GROUP by pba.party_id) pbb where DATE_ADD(pb.date, INTERVAL pb.id SECOND) = pbb.dt and pb.party_id = pbb.party_id and pb.party_id != 0  ORDER BY `pb`.`id` DESC";
        $allPartyBalance = DB::select($citySql);
        $partyCityRep = array();
        foreach ($allPartyBalance as $row) {
            if (!isset($partyCityRep[$party_by_ids[$row->party_id]->city])) {
                $partyCityRep[$party_by_ids[$row->party_id]->city] = number_format($row->closing_balance, 2, ".", "");
            } else {
                $partyCityRep[$party_by_ids[$row->party_id]->city] += number_format($row->closing_balance, 2, ".", "");
            }

        }
        // exit;
        ksort($partyCityRep);
        $allCitiesBalance = $partyCityRep;


        if ($print == 1) {
            $view = view('pages.party.cityreport', compact('allCitiesBalance'))->render();
            echo $view;
            exit;
        } else {

        }
    }

    public function get_city_wise_report(Request $request, $print): void
    {
        $cityString = $request->input('cityarr');
        $role = session('role');
        if (is_array($cityString)) {
            $cityArray = $cityString;
        } else {
            $cityArray = explode(",", $cityString);
        }
        sort($cityArray);
        $accessParty = $this->getAccessParty();
        $accessPartyArray = explode(",", $accessParty);
        $grouped_city = [];

        if (isset($cityArray) && !empty($cityArray)) {
            $partyData = Party::whereIn('city', $cityArray)->get();

            $partyByIds = [];
            foreach ($partyData as $row) {
                if ($role === 'superadmin' || in_array($row->id, $accessPartyArray)) {
                    $partyByIds[$row->id] = $row;
                }
            }

            $partyBalanceSQL = "SELECT * FROM `party_balance` pb,(SELECT MAX(DATE_ADD(pba.date, INTERVAL  pba.id SECOND)) as dt, ipbb.party_id, pba.id FROM `party_balance` pba, (SELECT max(date) as idt, party_id from party_balance GROUP by party_id) as ipbb WHERE ipbb.idt = pba.date AND ipbb.party_id = pba.party_id GROUP by pba.party_id) pbb where DATE_ADD(pb.date, INTERVAL pb.id SECOND) = pbb.dt and pb.party_id = pbb.party_id  ORDER BY `pb`.`id` DESC";
            $partyBalanceData = DB::select($partyBalanceSQL);
            $partyIdArray = [];

            foreach ($partyBalanceData as $row) {
                if (isset($partyIdArray[$row->party_id])) {
                    $tmpObj = $partyIdArray[$row->party_id];
                    if ($tmpObj->id < $row->id) {
                        $partyIdArray[$row->party_id] = $row;
                    }
                } else {
                    $partyIdArray[$row->party_id] = $row;
                }
            }

            foreach ($partyIdArray as $type) {
                if (isset($partyByIds[$type->party_id])) {
                    if ($type->closing_balance != 0) {
                        if ($type->closing_balance < 0) {
                            $grouped_city[$partyByIds[$type->party_id]->city]['negative_arr'][] = [
                                'name' => $partyByIds[$type->party_id]->name,
                                'code' => $partyByIds[$type->party_id]->code,
                                'current_balance' => $type->closing_balance
                            ];
                        } else {
                            $grouped_city[$partyByIds[$type->party_id]->city]['positive_arr'][] = [
                                'name' => $partyByIds[$type->party_id]->name,
                                'code' => $partyByIds[$type->party_id]->code,
                                'current_balance' => $type->closing_balance
                            ];
                        }
                    }
                }
            }

            foreach ($grouped_city as $city => $data) {
                if (isset($grouped_city[$city]['negative_arr'])) {
                    $code = array_column($grouped_city[$city]['negative_arr'], 'code');
                    array_multisort($code, SORT_ASC, $grouped_city[$city]['negative_arr']);
                }
                if (isset($grouped_city[$city]['positive_arr'])) {
                    $codewise = array_column($grouped_city[$city]['positive_arr'], 'code');
                    array_multisort($codewise, SORT_ASC, $grouped_city[$city]['positive_arr']);
                }
            }

            ksort($grouped_city);

            if ($print == 1) {
                $view = view('pages.party.citywisereport', compact('grouped_city'))->render();
                echo $view;
                exit;
            } else {
                $input_parmeter = [
                    'cities' => $cityString,
                    'downloaded_by' => session('user_name'),
                ];
                $data = [
                    'inputs' => $input_parmeter,
                    'grouped_city' => $grouped_city,
                ];
                $html = view('pages.reportdownload.citywisereport', compact('data'))->render();
                $pdfFilePath = "City_report_" . date("Y_m_d_H_i_s") . ".pdf";
                ob_end_clean();
                $document = new PDF([
                    'mode' => 'utf-8',
                    'format' => 'A4',
                    'margin_header' => '3',
                    'margin_top' => '20',
                    'margin_bottom' => '20',
                    'margin_footer' => '2',
                ]);

                $header = [
                    'Content-Type' => 'application/pdf',
                    'Content-Disposition' => 'inline; filename="' . $pdfFilePath . '"'
                ];
                $document->WriteHTML($html);
                $document->Output($pdfFilePath, "D");
            }
        }
    }

    public function get_city_wise_report_old($print = '0')
    {
        $user_id = auth()->id(); // Assuming you're using Laravel's built-in authentication
        $role = auth()->user()->role; // Assuming you have a role attribute in your user model
        $accessible_party = $this->getAccessParty($user_id);
        $accessible_party_arr = explode(",", $accessible_party);

        $cityString = request()->input('cityarr');
        if (!empty($cityString) && is_array($cityString)) {
            sort($cityString);
        }

        $grouped_city = [];
        if (isset($cityString) && $cityString != '') {
            if (is_array($cityString)) {
                $cities = "'" . implode("','", $cityString) . "'";
            } else {
                $city_Array = explode(',', $cityString);
                $cities = "'" . implode("','", $city_Array) . "'";
            }

            $cityList = PartyBalance::select('party.name', 'party.code', 'party_balance.closing_balance')
                ->join('party', 'party_balance.party_id', '=', 'party.id')
                ->whereIn('party.city', $cityString)
                ->orderByDesc('party_balance.id')
                ->get();
            dd($cityList);
            foreach ($cityList as $type) {
                $party = Party::find($type->party_id);
                if ($party) {
                    if ($type->closing_balance < 0 && $type->closing_balance != 0) {
                        $grouped_city[$party->city]['negative_arr'][] = [
                            'name' => $party->name,
                            'code' => $party->code,
                            'current_balance' => $type->closing_balance
                        ];
                    }
                    if ($type->closing_balance > 0 && $type->closing_balance != 0) {
                        $grouped_city[$party->city]['positive_arr'][] = [
                            'name' => $party->name,
                            'code' => $party->code,
                            'current_balance' => $type->closing_balance
                        ];
                    }
                }
            }
            foreach ($grouped_city as $city => $data) {
                if (isset($data['negative_arr'])) {
                    $code = array_column($data['negative_arr'], 'code');
                    array_multisort($code, SORT_ASC, $grouped_city[$city]['negative_arr']);
                }
                if (isset($data['positive_arr'])) {
                    $codewise = array_column($data['positive_arr'], 'code');
                    array_multisort($codewise, SORT_ASC, $grouped_city[$city]['positive_arr']);
                }
            }
            ksort($grouped_city);
        }
        // dd($grouped_city);
        if ($print == 1) {
            $view = view('pages.party.citywisereport', compact('grouped_city',))->render();
            echo $view;
            exit;
        } else {
            $cityString = request()->input('cityarr');
            $input_parmeter = [
                'cities' => $cityString,
                'downloaded_by' => auth()->user()->name // Assuming you have a name attribute in your user model
            ];
            $data['inputs'] = $input_parmeter;
            $html = view('reportdownload.citywisereport', $data)->render();
            $pdfFilePath = "City_report_" . date("Y-m-d H:i:s") . ".pdf";
            $mpdf = new \Mpdf\Mpdf();
            $mpdf->WriteHTML($html);
            $mpdf->Output($pdfFilePath, "D");
        }
    }

    public function get_label_count_report(Request $request, $print)
    {
        $particular_date = $request->input('date');
        $particular_date = date('Y-m-d', strtotime($particular_date));
        $role = session('role');
        $accessParty = $this->getAccessParty();

        $condition = "type != 'namu' AND date LIKE '$particular_date%'";
        $tableName = (strtotime(date('Y-m-d', strtotime($particular_date))) < strtotime(date('Y-m-d'))) ? 'vatavBackUp' : 'vatav';

        $dayWiseList = DB::table($tableName)
            ->select('entered_by', DB::raw('COUNT(entered_by) as usercount'))
            ->whereRaw($condition)
            ->groupBy('entered_by')
            ->get();

        $total_count = 0;
        foreach ($dayWiseList as $_dayWiseList) {
            $total_count += $_dayWiseList->usercount;
        }
        $typeWiseList = array();
        $senderBranchWiseList = array();
        $receiverBranchWiseList = array();
        $namaSenderBranchWiseList = array();
        $namaReceiverBranchWiseList = array();
        $finalCenterWiseArr = array();

        if ($role === 'superadmin') {
            $typeWiseList = DB::table($tableName)
                ->select('type', DB::raw('COUNT(id) as typecount'))
                ->whereRaw("date LIKE '$particular_date%' AND type != 'namu'")
                ->groupBy('type')
                ->get();

            $senderBranchWiseList = DB::table($tableName)
                ->select('sender', DB::raw('COUNT(id) as sendercount'))
                ->whereRaw("date LIKE '$particular_date%' AND type != 'namu'")
                ->groupBy('sender')
                ->get();

            $receiverBranchWiseList = DB::table($tableName)
                ->select('receiver', DB::raw('COUNT(id) as receivercount'))
                ->whereRaw("date LIKE '$particular_date%' AND type != 'namu'")
                ->groupBy('receiver')
                ->get();

            $namaSenderBranchWiseList = DB::table($tableName)
                ->select('sender', DB::raw('COUNT(id) as sendercount'))
                ->whereRaw("date LIKE '$particular_date%' AND type = 'namu'")
                ->groupBy('sender')
                ->get();

            $namaReceiverBranchWiseList = DB::table($tableName)
                ->select('receiver', DB::raw('COUNT(id) as receivercount'))
                ->whereRaw("date LIKE '$particular_date%' AND type = 'namu'")
                ->groupBy('receiver')
                ->get();
        } else {
            $typeWiseList = DB::table($tableName)
                ->select('type', DB::raw('COUNT(id) as typecount'))
                ->whereRaw("date LIKE '$particular_date%' AND type != 'namu' AND (sender_party_id IN ($accessible_party) OR receiver_party_id IN ($accessible_party))")
                ->groupBy('type')
                ->get();

            $senderBranchWiseList = DB::table($tableName)
                ->select('sender', DB::raw('COUNT(id) as sendercount'))
                ->whereRaw("date LIKE '$particular_date%' AND type != 'namu' AND (sender_party_id IN ($accessible_party))")
                ->groupBy('sender')
                ->get();

            $receiverBranchWiseList = DB::table($tableName)
                ->select('receiver', DB::raw('COUNT(id) as receivercount'))
                ->whereRaw("date LIKE '$particular_date%' AND type != 'namu' AND (receiver_party_id IN ($accessible_party))")
                ->groupBy('receiver')
                ->get();

            $namaSenderBranchWiseList = DB::table($tableName)
                ->select('sender', DB::raw('COUNT(id) as sendercount'))
                ->whereRaw("date LIKE '$particular_date%' AND type = 'namu' AND (sender_party_id IN ($accessible_party))")
                ->groupBy('sender')
                ->get();

            $namaReceiverBranchWiseList = DB::table($tableName)
                ->select('receiver', DB::raw('COUNT(id) as receivercount'))
                ->whereRaw("date LIKE '$particular_date%' AND type = 'namu' AND (receiver_party_id IN ($accessible_party))")
                ->groupBy('receiver')
                ->get();
        }

        foreach ($receiverBranchWiseList as $rKey => $rVal) {
            $finalCenterWiseArr[$rVal->receiver]['receive'] = $rVal->receivercount;
        }
        foreach ($senderBranchWiseList as $sKey => $sVal) {
            $finalCenterWiseArr[$sVal->sender]['send'] = $sVal->sendercount;
        }

        foreach ($namaReceiverBranchWiseList as $rKey => $rVal) {
            $finalCenterWiseArr[$rVal->receiver]['namareceive'] = $rVal->receivercount;
        }
        foreach ($namaSenderBranchWiseList as $sKey => $sVal) {
            $finalCenterWiseArr[$sVal->sender]['namasend'] = $sVal->sendercount;
        }

        $data['branchWiseList'] = $finalCenterWiseArr;
        $data['typeWiseList'] = $typeWiseList;
        $data['total_count'] = $total_count;
        $data['dayWiseList'] = $dayWiseList;
        // dd($data);
        if ($print == '1') {
            $datas = view('pages.party.labelcount', compact("data"))->render();
            echo $datas;
            exit;
        } else {
            $input_parmeter = array(
                'date' => $particular_date,
                'downloaded_by' => session('user_name')
            );
            $data_arr['inputs'] = $input_parmeter;
            $pdfFilePath = 'Label_Count_report_' . date('Y_m_d_H_i_s') . '.pdf';
            ob_end_clean();
            $document = new PDF([
                'mode' => 'utf-8',
                'format' => 'A4',
                'margin_header' => '3',
                'margin_top' => '20',
                'margin_bottom' => '20',
                'margin_footer' => '2',
            ]);

            $header = [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' => 'inline; filename="' . $pdfFilePath . '"'
            ];
            $data_arr['branchWiseList'] = $finalCenterWiseArr;
            $data_arr['typeWiseList'] = $typeWiseList;
            $data_arr['total_count'] = $total_count;
            $data_arr['dayWiseList'] = $dayWiseList;
            $html = view('pages.reportdownload.labelcountreport', compact("data_arr"))->render();
            $html = mb_convert_encoding($html, 'UTF-8', 'UTF-8');
            $document->WriteHTML($html);
            $document->Output($pdfFilePath, "D");
        }
    }

    public function get_party_to_party_report(Request $request, $print)
    {
        $sender = $request->input('sender');
        $receiver = $request->input('receiver');
        $amount = $request->input('amount');
        $is_deleted = $request->input('is_deleted');
        $date = $request->input('date_range');
        $date_range = explode(" to ", $date);

        $start_date = date('Y-m-d', strtotime($date_range[0]));
        $end_date = date('Y-m-d', strtotime($date_range[1]));


        $query = Vatav::with('sender_party', 'receiver_party')
            ->where('date', '>=', $start_date . ' 00:00:00')
            ->where('date', '<=', $end_date . ' 23:59:59')
            ->where('sender_party_id', $sender)
            ->where('receiver_party_id', $receiver);

        if (!is_null($amount)) {
            $query = $query->where('amount', $amount);
        }

        if ($is_deleted == 1) {
            $query = $query->where('is_deleted', 1);
            $partyList = $query->onlyTrashed()->get()->toArray();
        } else {
            $query = $query->where('is_deleted', 0)
                ->where('ho_approve', 1);
            // $partyList = $query->get()->toArray();
        }

        $queryBackup = VatavBackUp::with('sender_party', 'receiver_party')
            ->where('date', '>=', $start_date . ' 00:00:00')
            ->where('date', '<=', $end_date . ' 23:59:59')
            ->where('sender_party_id', $sender)
            ->where('receiver_party_id', $receiver);

        if (!is_null($amount)) {
            $queryBackup = $queryBackup->where('amount', $amount);
        }

        if ($is_deleted == 1) {
            $queryBackup = $queryBackup->where('is_deleted', 1);
            $partyList = $queryBackup->onlyTrashed()->get()->toArray();
        } else {
            $queryBackup = $queryBackup->where('is_deleted', 0)
                ->where('ho_approve', 1);
            // $partyList = $queryBackup->get()->toArray();
        }
        $partyList = $query->union($queryBackup)->get()->toArray();
        if (count($partyList) > 0) {
            foreach ($partyList as $key => $item) {
                $partyList[$key]['new_date'] = Carbon::parse($item['date'])->format('d-m-Y');
                $partyList[$key]['new_time'] = Carbon::parse($item['date'])->format('h:i A');
                $partyList[$key]['sender_party_name'] = !empty($item['sender_party'][0]['name']) ? $item['sender_party'][0]['name'] : '';
                $partyList[$key]['sender_party_code'] = !empty($item['sender_party'][0]['code']) ? $item['sender_party'][0]['code'] : '';
                $partyList[$key]['receiver_party_name'] = !empty($item['receiver_party'][0]['name']) ? $item['receiver_party'][0]['name'] : '';
                $partyList[$key]['receiver_party_code'] = !empty($item['receiver_party'][0]['code']) ? $item['receiver_party'][0]['code'] : '';
                $partyList[$key]['new_remarks'] = $item['remarks'];
                $partyList[$key]['srno'] = $item['serial_no'];
                $partyList[$key]['new_amount'] = number_format($item['amount'], 2, '.', '');
                $partyList[$key]['vatav'] = number_format($item['o_vatav'], 2, '.', '');
                $partyList[$key]['user'] = $item['entered_by'];
                $partyList[$key]['adhar_number'] = $item['s_adhar_number'];
            }
        }
        $mergeRecords = array_merge($partyList);
        // dd($mergeRecords);
        $SenderpartyDetails = $this->getPartyDetailsFromId($sender);
        $senderName = $SenderpartyDetails->code .= " (" . $SenderpartyDetails->name . ")";

        $ReceiverpartyDetails = $this->getPartyDetailsFromId($receiver);
        $receiverName = $ReceiverpartyDetails->code .= " (" . $ReceiverpartyDetails->name . ")";

        $demo_start_date = date('d-m-Y', strtotime($date_range[0]));
        $demo_end_date = date('d-m-Y', strtotime($date_range[1]));

        $new_date = $demo_start_date .= "to" . $demo_end_date;

        if (isset($mergeRecords)) {
            $data['allParties'] = $mergeRecords;
            $data['senderName'] = $senderName;
            $data['receiverName'] = $receiverName;
            $data['new_date'] = $new_date;
        } else {
            $data['allParties'] = array();
        }

        if ($print == '1') {
            $data = view('pages.party.partytoparty', compact("data"))->render();
            echo $data;
            exit;
        } else {
            $input_parmeter = array(
                'date' => $date,
                'downloaded_by' => session('user_name'),
                'is_deleted' => $is_deleted,
                'amount' => $amount
            );
            $data_arr['inputs'] = $input_parmeter;
            $data_arr['allParties'] = $data['allParties'];
            $pdfFilePath = 'Party_to_party_report_' . date('Y_m_d_H_i_s') . '.pdf';
            ob_end_clean();
            $document = new PDF([
                'mode' => 'utf-8',
                'format' => 'A4',
                'margin_header' => '3',
                'margin_top' => '20',
                'margin_bottom' => '20',
                'margin_footer' => '2',
            ]);

            // Set some header information for output
            $header = [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' => 'inline; filename="' . $pdfFilePath . '"'
            ];
            $data_arr['allParties'] = $data['allParties'];
            $data_arr['senderName'] = $data['senderName'];
            $data_arr['receiverName'] = $data['receiverName'];
            $data_arr['new_date'] = $data['new_date'];
            $html = view('pages.reportdownload.partytoparty', compact("data_arr"))->render();
            $html = mb_convert_encoding($html, 'UTF-8', 'UTF-8');
            $document->WriteHTML($html);
            $document->Output($pdfFilePath, "D");
        }
    }

    public function getSearchEntryList($print = '0')
    {
        $page = request()->input('startRow');
        $searchName = request()->input('search_name');
        $date = request()->input('date_range');
        $dateRange = explode(" to ", $date);
        $condition = '';
        $query = Vatav::query();

        if ($date != "") {
            $startDate = date('Y-m-d', strtotime($dateRange[0]));
            $endDate = date('Y-m-d', strtotime($dateRange[1]));

            if ($startDate == $endDate) {
                $query->where('date', 'LIKE', "$startDate%");
            } else {
                $query->whereBetween(DB::raw('DATE(date)'), [$startDate, $endDate]);
            }
        }

        $query->where('is_deleted', 0)
            ->where('ho_approve', 1);

        $query->where(function ($q) use ($searchName) {
            $q->where('sender', 'LIKE', "%$searchName%")
                ->orWhere('receiver', 'LIKE', "%$searchName%")
                ->orWhere('sender_name', 'LIKE', "%$searchName%")
                ->orWhere('sender_contact', 'LIKE', "%$searchName%")
                ->orWhere('receiver_name', 'LIKE', "%$searchName%")
                ->orWhere('receiver_contact', 'LIKE', "%$searchName%")
                ->orWhere('amount', 'LIKE', "%$searchName%")
                ->orWhere('entered_by', 'LIKE', "%$searchName%")
                ->orWhere('serial_no', 'LIKE', "%$searchName%")
                ->orWhere('remarks', 'LIKE', "%$searchName%");
        });

        $query->orderBy('date')
            ->orderBy('serial_no');

        $queryBackup = VatavBackUp::query();

        if ($date != "") {
            $startDate = date('Y-m-d', strtotime($dateRange[0]));
            $endDate = date('Y-m-d', strtotime($dateRange[1]));

            if ($startDate == $endDate) {
                $queryBackup->where('date', 'LIKE', "$startDate%");
            } else {
                $queryBackup->whereBetween(DB::raw('DATE(date)'), [$startDate, $endDate]);
            }
        }

        $queryBackup->where('is_deleted', 0)
            ->where('ho_approve', 1);

        $queryBackup->where(function ($q) use ($searchName) {
            $q->where('sender', 'LIKE', "%$searchName%")
                ->orWhere('receiver', 'LIKE', "%$searchName%")
                ->orWhere('sender_name', 'LIKE', "%$searchName%")
                ->orWhere('sender_contact', 'LIKE', "%$searchName%")
                ->orWhere('receiver_name', 'LIKE', "%$searchName%")
                ->orWhere('receiver_contact', 'LIKE', "%$searchName%")
                ->orWhere('amount', 'LIKE', "%$searchName%")
                ->orWhere('entered_by', 'LIKE', "%$searchName%")
                ->orWhere('serial_no', 'LIKE', "%$searchName%")
                ->orWhere('remarks', 'LIKE', "%$searchName%");
        });

        $queryBackup->orderBy('date')
            ->orderBy('serial_no');

        $allParties = $query->union($queryBackup)->get();


        $data['allParties'] = $allParties->toArray();
        // dd($data);
        if ($print == '1') {
            return view('pages.party.partysearchreport', $data)->render();
        } else {
            $inputParameter = [
                'date' => $date,
                'downloaded_by' => auth()->user()->username
            ];

            $dataArr['inputs'] = $inputParameter;
            $pdfFilePath = "Search_report_" . date("Y-m-d H:i:s") . ".pdf";
            $mpdf = new \Mpdf\Mpdf();
            ob_end_clean();
            $dataArr['allParties'] = $allParties;
            $html = view('pages.reportdownload.partysearchreport', compact('dataArr'))->render();
            $mpdf->WriteHTML($html);
            $mpdf->Output($pdfFilePath, "D");
        }
    }

    public function get_deleted_entry_report($print = '0')
    {
        ini_set('memory_limit', '-1');
        ini_set('max_execution_time', 0);

        $page = request()->input('startRow', 0);
        $date = request()->input('date_range');
        $date_range = explode(" to ", $date);

        $start_date = date('Y-m-d', strtotime($date_range[0]));
        $end_date = date('Y-m-d', strtotime($date_range[1]));

        $vatav_entries = Vatav::where('is_deleted', 1)
            ->where('deleted_time', '>=', $start_date . ' 00:00:00')
            ->where('deleted_time', '<=', $end_date . ' 23:59:59')
            ->withTrashed()
            ->get();

        $vatav_backup_entries = VatavBackUp::where('is_deleted', 1)
            ->where('deleted_time', '>=', $start_date . ' 00:00:00')
            ->where('deleted_time', '<=', $end_date . ' 23:59:59')
            ->withTrashed()
            ->get();

        $vatavData = $vatav_entries->merge($vatav_backup_entries);

        foreach ($vatavData as $row) {
            $sender = explode('(', $row->sender);
            $row->sender_code_name = rtrim($sender[1], ")");
            $row->sender_code = trim($sender[0]);

            $receiver = explode('(', $row->receiver);
            $row->receiver_code_name = rtrim($receiver[1], ")");
            $row->receiver_code = trim($receiver[0]);
        }

        if ($print == '1') {
            return view('pages.party.deletedentry', compact('vatavData'));
        } else {
            $input_parameter = [
                'date' => $date,
                'downloaded_by' => auth()->user()->company_name
            ];
            $data_arr['vatavData'] = $vatavData;
            $data_arr['inputs'] = $input_parameter;
            $pdfFilePath = "Deleted_entry_report_" . date('Y-m-d H:i:s') . ".pdf";
            $mpdf = new \Mpdf\Mpdf();
            ob_end_clean();
            $html = view('pages.reportdownload.deletedentry', compact('data_arr'))->render();
            $mpdf->WriteHTML($html);
            $mpdf->Output($pdfFilePath, "D");
        }
    }

    public function get_day_wise_commission_report(Request $request, $print = '0')
    {
        ini_set('memory_limit', '-1');
        ini_set('max_execution_time', 0);

        $page = request()->input('startRow', 0);
        $date = request()->input('date_range');
        $date_range = explode(" to ", $date);
        $sender = $request->input('sender');

        $start_date = date('Y-m-d', strtotime($date_range[0]));
        $end_date = date('Y-m-d', strtotime($date_range[1]));

        $query = Vatav::where('date', '>=', $start_date . ' 00:00:00')
            ->where('date', '<=', $end_date . ' 23:59:59')
            ->where('is_deleted', 0);

        $queryBackup = VatavBackUp::where('date', '>=', $start_date . ' 00:00:00')
            ->where('date', '<=', $end_date . ' 23:59:59')
            ->where('is_deleted', 0);

        if ($sender) {
            $query->where(function ($q) use ($sender) {
                $q->whereRaw('sender_party_id = ?', [$sender])
                    ->orWhereRaw('receiver_party_id = ?', [$sender]);
            });
            $queryBackup->where(function ($q) use ($sender) {
                $q->whereRaw('sender_party_id = ?', [$sender])
                    ->orWhereRaw('receiver_party_id = ?', [$sender]);
            });
        }

        $data['allEntryList'] = $query->union($queryBackup)->get();
        $final_arr = [];

        foreach ($data['allEntryList'] as $amountData) {
            if ($amountData->total_vatav_label == 0) {
                $amountData->party_commission = 0.00;
                $amountData->commission_party_amount = 0.00;
            } else {
                if ($amountData->sender_party_id == $sender) {
                    $amountData->party_commission = number_format($amountData->total_vatav_label - $amountData->s_vatav, "2", ".", "");
                    $amountData->commission_party_amount = number_format($amountData->sender_commission_party_amount, "2", ".", "");
                }
                if ($amountData->receiver_party_id == $sender) {
                    $amountData->party_commission = number_format($amountData->r_vatav, "2", ".", "");
                    $amountData->commission_party_amount = number_format($amountData->receiver_commission_party_amount, "2", ".", "");
                }
            }
            $final_arr[] = $amountData;
            // dd($amountData->total_vatav_label);
        }

        $data['allPartyList'] = $final_arr;

        if ($print == '1') {
            return view('pages.party.partywisecommission', $data)->render();
        } else {
            $input_parameter = [
                'sender' => $sender,
                'date' => $request->input('date_range'),
                'downloaded_by' => auth()->user()->company_name
            ];

            $data_arr['inputs'] = $input_parameter;
            $pdfFilePath = "party_wise_commission_report_" . date("Y-m-d H:i:s") . ".pdf";
            $mpdf = new \Mpdf\Mpdf();
            ob_end_clean();
            $data_arr['allPartyList'] = $data['allPartyList'];
            $html = view('pages.reportdownload.partywisecommission', $data_arr)->render();
            $mpdf->WriteHTML($html);
            $mpdf->Output($pdfFilePath, "D");
        }
    }

    public function get_month_wise_commission_report(Request $request, $print = '0')
    {
        ini_set('memory_limit', '-1');
        ini_set('max_execution_time', 0);

        $date = request()->input('date_range');
        $sender = request()->input('month_sender');
        $date = request()->input('month');
        $sender_new = $this->getPartyDetailsFromId($sender);
        $party_name = $sender_new->name;

        $exploded_date = explode("-", $date);
        $month = $exploded_date[0];
        $year = $exploded_date[1];

        $query = Vatav::whereYear('date', $year)
            ->whereMonth('date', $month)
            ->where('is_deleted', 0);

        $queryBackup = VatavBackUp::whereYear('date', $year)
            ->whereMonth('date', $month)
            ->where('is_deleted', 0);

        if ($sender) {
            $query->where(function ($q) use ($sender) {
                $q->whereRaw('sender_party_id = ?', [$sender])
                    ->orWhereRaw('receiver_party_id = ?', [$sender]);
            });

            $queryBackup->where(function ($q) use ($sender) {
                $q->whereRaw('sender_party_id = ?', [$sender])
                    ->orWhereRaw('receiver_party_id = ?', [$sender]);
            });
        }

        $data['allEntryList'] = $query->union($queryBackup)->get();

        foreach ($data['allEntryList'] as $_amountData) {
            if ($_amountData->total_vatav_label == 0) {
                $_amountData->party_commission = 0.00;
            } else {
                if ($_amountData->sender_party_id == $sender) {
                    $_amountData->party_commission = number_format($_amountData->total_vatav_label - $_amountData->s_vatav, "2", ".", "");
                    $_amountData->commission = number_format($_amountData->sender_commission_party_amount, "2", ".", "");
                }
                if ($_amountData->receiver_party_id == $sender) {
                    $_amountData->party_commission = number_format($_amountData->r_vatav, "2", ".", "");
                    $_amountData->commission = number_format($_amountData->receiver_commission_party_amount, "2", ".", "");
                }
            }
            $final_arr[] = $_amountData;
        }

        $tmp_arr = [];
        foreach ($final_arr as $value) {
            $da = date("Y-m-d", strtotime($value->date));
            if (!isset($tmp_arr[$da])) {
                $tmp_arr[$da] = ['party_commission' => 0, 'commission' => 0];
            }
            $tmp_arr[$da]['party_commission'] += $value->party_commission;
            $tmp_arr[$da]['commission'] += $value->commission;
        }

        $data['amountData'] = $tmp_arr;

        if ($print == '1') {
            return view('pages.party.day_wise_party_commission', $data)->render();
        } else {
            $input_parameter = [
                'sender' => $party_name,
                'date' => $date,
                'downloaded_by' => auth()->user()->company_name
            ];

            $data_arr['day_wise'] = 1;
            $data_arr['inputs'] = $input_parameter;
            $pdfFilePath = "party_wise_commission_report_" . date("Y-m-d H:i:s") . ".pdf";
            $mpdf = new \Mpdf\Mpdf();
            ob_end_clean();

            $data_arr['amountData'] = $data['amountData'];
            $html = view('pages.reportdownload.daywise_party_commission', $data_arr)->render();
            $mpdf->WriteHTML($html);
            $mpdf->Output($pdfFilePath, "D");
        }
    }

    public function get_year_wise_commission_report($print = '0')
    {
        $sender = request()->input('year_sender');
        $date = request()->input('year');

        $query = Vatav::whereYear('date', $date)->where('is_deleted', 0);
        $queryBackup = VatavBackUp::whereYear('date', $date)->where('is_deleted', 0);
        $sender_new = $this->getPartyDetailsFromId($sender);
        $party_name = $sender_new->name;

        if ($sender) {
            $query->where(function ($q) use ($sender) {
                $q->whereRaw('sender_party_id = ?', [$sender])
                    ->orWhereRaw('receiver_party_id = ?', [$sender]);
            });
            $queryBackup->where(function ($q) use ($sender) {
                $q->whereRaw('sender_party_id = ?', [$sender])
                    ->orWhereRaw('receiver_party_id = ?', [$sender]);
            });
        }

        $data['amountData'] = $query->union($queryBackup)->get();

        foreach ($data['amountData'] as $_amountData) {
            if ($_amountData->total_vatav_label == 0) {
                $_amountData->party_commission = 0.00;
            } else {
                if ($_amountData->sender_party_id == $sender) {
                    $_amountData->party_commission = number_format($_amountData->total_vatav_label - $_amountData->s_vatav, "2", ".", "");
                    $_amountData->commission = number_format($_amountData->sender_commission_party_amount, "2", ".", "");
                }
                if ($_amountData->receiver_party_id == $sender) {
                    $_amountData->party_commission = number_format($_amountData->r_vatav, "2", ".", "");
                    $_amountData->commission = number_format($_amountData->receiver_commission_party_amount, "2", ".", "");
                }
            }
            $final_arr[] = $_amountData;
        }
        $tmp_arr = [];
        foreach ($final_arr as $value) {
            $da = date("m", strtotime($value->date));
            if (!isset($tmp_arr[$da])) {
                $tmp_arr[$da]['party_commission'] = 0;
                $tmp_arr[$da]['commission'] = 0;
            }
            $tmp_arr[$da]['party_commission'] += $value->party_commission;
            $tmp_arr[$da]['commission'] += $value->commission;
        }

        $data['amountData'] = $tmp_arr;

        if ($print == '1') {
            return view('pages.party.month_wise_party_commission', $data);
        } else {
            $input_parameter = [
                'sender' => $party_name,
                'date' => $date,
                'downloaded_by' => auth()->user()->company_name
            ];

            $data_arr['inputs'] = $input_parameter;
            $pdfFilePath = "party_wise_commission_report_" . now()->format("Y-m-d H:i:s") . ".pdf";
            $mpdf = new \Mpdf\Mpdf();
            ob_end_clean();
            $data_arr['amountData'] = $data['amountData'];
            $html = view('pages.reportdownload.month_wise_party_commission', $data_arr)->render();
            $mpdf->WriteHTML($html);
            $mpdf->Output($pdfFilePath, "D");
        }
    }

    public function get_commission_list(Request $request, $print = '0')
    {
        ini_set('memory_limit', '-1');
        ini_set('max_execution_time', 0);

        $page = request()->input('startRow', 0);
        $date = request()->input('date_range');
        $date_range = explode(" to ", $date);
        $sender = $request->input('sender');

        $start_date = date('Y-m-d', strtotime($date_range[0]));
        $end_date = date('Y-m-d', strtotime($date_range[1]));

        $query = Vatav::where('date', '>=', $start_date . ' 00:00:00')
            ->where('date', '<=', $end_date . ' 23:59:59')
            ->where('is_deleted', 0);

        $queryBackup = VatavBackUp::where('date', '>=', $start_date . ' 00:00:00')
            ->where('date', '<=', $end_date . ' 23:59:59')
            ->where('is_deleted', 0);

        if ($sender) {
            $query->where(function ($q) use ($sender) {
                $q->whereRaw('sender_party_id = ?', [$sender])
                    ->orWhereRaw('receiver_party_id = ?', [$sender]);
            });
            $queryBackup->where(function ($q) use ($sender) {
                $q->whereRaw('sender_party_id = ?', [$sender])
                    ->orWhereRaw('receiver_party_id = ?', [$sender]);
            });
        }

        $data['allEntryList'] = $query->union($queryBackup)->get();
        $final_arr = [];

        foreach ($data['allEntryList'] as $amountData) {
            if ($amountData->total_vatav_label == 0) {
                $amountData->party_commission = 0.00;
                $amountData->commission_party_amount = 0.00;
            } else {
                if ($amountData->sender_party_id == $sender) {
                    $amountData->party_commission = number_format($amountData->total_vatav_label - $amountData->s_vatav, "2", ".", "");
                    $amountData->commission_party_amount = number_format($amountData->sender_commission_party_amount, "2", ".", "");
                }
                if ($amountData->receiver_party_id == $sender) {
                    $amountData->party_commission = number_format($amountData->r_vatav, "2", ".", "");
                    $amountData->commission_party_amount = number_format($amountData->receiver_commission_party_amount, "2", ".", "");
                }
            }
            $final_arr[] = $amountData;
            // dd($amountData->total_vatav_label);
        }

        $data['allPartyList'] = $final_arr;

        if ($print == '1') {
            return view('pages.party.commissionreport', $data)->render();
        } else {
            $input_parameter = [
                'sender' => $sender,
                'date' => $request->input('date_range'),
                'downloaded_by' => auth()->user()->company_name
            ];

            $data_arr['inputs'] = $input_parameter;
            $pdfFilePath = "party_wise_commission_report_" . date("Y-m-d H:i:s") . ".pdf";
            $mpdf = new \Mpdf\Mpdf();
            ob_end_clean();
            $data_arr['allPartyList'] = $data['allPartyList'];
            $html = view('pages.reportdownload.commissionreport', $data_arr)->render();
            $mpdf->WriteHTML($html);
            $mpdf->Output($pdfFilePath, "D");
        }
    }

    public function get_daywise_commission_list(Request $request, $print = '0')
    {
        ini_set('memory_limit', '-1');
        ini_set('max_execution_time', 0);

        $date = request()->input('date_range');
        $sender = request()->input('month_sender');
        $date = request()->input('month');
        $sender_new = $this->getPartyDetailsFromId($sender);
        $party_name = $sender_new->name;

        $exploded_date = explode("-", $date);
        $month = $exploded_date[0];
        $year = $exploded_date[1];

        $query = Vatav::whereYear('date', $year)
            ->whereMonth('date', $month)
            ->where('is_deleted', 0);

        $queryBackup = VatavBackUp::whereYear('date', $year)
            ->whereMonth('date', $month)
            ->where('is_deleted', 0);

        if ($sender) {
            $query->where(function ($q) use ($sender) {
                $q->whereRaw('sender_party_id = ?', [$sender])
                    ->orWhereRaw('receiver_party_id = ?', [$sender]);
            });

            $queryBackup->where(function ($q) use ($sender) {
                $q->whereRaw('sender_party_id = ?', [$sender])
                    ->orWhereRaw('receiver_party_id = ?', [$sender]);
            });
        }

        $data['allEntryList'] = $query->union($queryBackup)->get();

        foreach ($data['allEntryList'] as $_amountData) {
            if ($_amountData->total_vatav_label == 0) {
                $_amountData->party_commission = 0.00;
            } else {
                if ($_amountData->sender_party_id == $sender) {
                    $_amountData->party_commission = number_format($_amountData->total_vatav_label - $_amountData->s_vatav, "2", ".", "");
                    $_amountData->commission = number_format($_amountData->sender_commission_party_amount, "2", ".", "");
                }
                if ($_amountData->receiver_party_id == $sender) {
                    $_amountData->party_commission = number_format($_amountData->r_vatav, "2", ".", "");
                    $_amountData->commission = number_format($_amountData->receiver_commission_party_amount, "2", ".", "");
                }
            }
            $final_arr[] = $_amountData;
        }

        $tmp_arr = [];
        foreach ($final_arr as $value) {
            $da = date("Y-m-d", strtotime($value->date));
            if (!isset($tmp_arr[$da])) {
                $tmp_arr[$da] = ['party_commission' => 0, 'commission' => 0];
            }
            $tmp_arr[$da]['party_commission'] += $value->party_commission;
            $tmp_arr[$da]['commission'] += $value->commission;
        }

        $data['amountData'] = $tmp_arr;

        if ($print == '1') {
            return view('pages.party.daywise_party_commission', $data)->render();
        } else {
            $input_parameter = [
                'sender' => $party_name,
                'date' => $date,
                'downloaded_by' => auth()->user()->company_name
            ];

            $data_arr['day_wise'] = 1;
            $data_arr['inputs'] = $input_parameter;
            $pdfFilePath = "party_wise_commission_report_" . date("Y-m-d H:i:s") . ".pdf";
            $mpdf = new \Mpdf\Mpdf();
            ob_end_clean();

            $data_arr['amountData'] = $data['amountData'];
            $html = view('pages.reportdownload.daywise_party_commission', $data_arr)->render();
            $mpdf->WriteHTML($html);
            $mpdf->Output($pdfFilePath, "D");
        }
    }

    public function get_vatav_range_report(Request $request, $print = '0')
    {
        ini_set('memory_limit', '-1');
        ini_set('max_execution_time', 0);

        $userId = auth()->user()->id;
        $role = auth()->user()->role;
        $accessibleParty = $this->getAccessParty($userId);

        $date = $request->input('date');
        $date_range = explode(" to ", $date);

        $start_date = date('Y-m-d', strtotime($date_range[0]));
        $end_date = date('Y-m-d', strtotime($date_range[1]));

        if ($role === 'superadmin') {
            $vatavQuery = Vatav::where('is_deleted', 0)
                ->where('date', '>=', $start_date . " 00:00:00")
                ->where('date', '<=', $end_date . " 23:59:59")
                ->where(function ($query) {
                    $query->where('o_vatav', '!=', 0)
                        ->orWhere('sender', 'like', 'VAT %')
                        ->orWhere('receiver', 'like', 'VAT %');
                });
            $vatavBackUpQuery = VatavBackUp::where('is_deleted', 0)
                ->where('date', '>=', $start_date . " 00:00:00")
                ->where('date', '<=', $end_date . " 23:59:59")
                ->where(function ($query) {
                    $query->where('o_vatav', '!=', 0)
                        ->orWhere('sender', 'like', 'VAT %')
                        ->orWhere('receiver', 'like', 'VAT %');
                });
        } else {
            $vatavQuery = Vatav::where('is_deleted', 0)
                ->where('date', '>=', $start_date . " 00:00:00")
                ->where('date', '<=', $end_date . " 23:59:59")
                ->where(function ($query) use ($accessibleParty) {
                    $query->where(function ($q) {
                        $q->where('type', 'namu')
                            ->orWhere('sender', 'like', 'VAT %');
                    })
                        ->whereIn('receiver_party_id', $accessibleParty)
                        ->orWhere(function ($q) use ($accessibleParty) {
                            $q->where('receiver', 'like', 'VAT %');
                        })
                        ->whereIn('sender_party_id', $accessibleParty);
                });
            $vatavBackUpQuery = VatavBackUp::where('is_deleted', 0)
                ->where('date', '>=', $start_date . " 00:00:00")
                ->where('date', '<=', $end_date . " 23:59:59")
                ->where(function ($query) use ($accessibleParty) {
                    $query->where(function ($q) {
                        $q->where('type', 'namu')
                            ->orWhere('sender', 'like', 'VAT %');
                    })
                        ->whereIn('receiver_party_id', $accessibleParty)
                        ->orWhere(function ($q) use ($accessibleParty) {
                            $q->where('receiver', 'like', 'VAT %');
                        })
                        ->whereIn('sender_party_id', $accessibleParty);
                });
        }

        $data['amount'] = $vatavQuery->union($vatavBackUpQuery)->get();
        // dd($data);
        $data['vatavList'] = [];
        $data['sumData'] = 0;

        foreach ($data['amount'] as $amountVal) {
//            if (Str::contains($amountVal->sender, "VAT ")) {
            if ($amountVal->sender_party_id == 1) {
                if ($amountVal->type == 'namu') {
                    $amountVal->o_vatav = abs($amountVal->sender_total_amount);
                } else {
                    $amountVal->o_vatav = -1 * abs($amountVal->sender_total_amount);
                }
//            } elseif (Str::contains($amountVal->receiver, "VAT ")) {
            } elseif ($amountVal->receiver_party_id == 1) {
                if ($amountVal->type == 'namu') {
                    $amountVal->o_vatav = -1 * abs($amountVal->receiver_total_amount);
                } else {
                    $amountVal->o_vatav = abs($amountVal->receiver_total_amount);
                }
            }

            $data['sumData'] += $amountVal->o_vatav;
            $data['vatavList'][] = $amountVal;

        }
        // dd($data);

        if ($print == '1') {
            // dd($data);
            return view('pages.party.vatavrangereport', $data)->render();
        } else {
            $input_parameter = [
                'date' => $date,
                'downloaded_by' => auth()->user()->company_name,
            ];

            $data_arr['day_wise'] = 1;
            $data_arr['inputs'] = $input_parameter;
            $pdfFilePath = "vatav_rangewise_report_" . date("Y-m-d H:i:s") . ".pdf";
            $mpdf = new \Mpdf\Mpdf();
            ob_end_clean();

            $data_arr['sumData'] = $data['sumData'];
            $data_arr['vatavList'] = $data['vatavList'];
            $html = view('pages.reportdownload.vatavrangereport', $data_arr)->render();
            $mpdf->WriteHTML($html);
            $mpdf->Output($pdfFilePath, "D");
        }

    }

    public function get_vatav_report(Request $request, $print = 0)
    {
        ini_set('memory_limit', '-1');
        ini_set('max_execution_time', 0);

        $userId = auth()->user()->id;
        $role = auth()->user()->role;
        $accessibleParty = $this->getAccessParty($userId);

        $page = request()->input('startRow');
        $dateVal = request()->input('date');

        if ($role === 'superadmin') {
            $condition = "is_deleted = 0 AND `date` LIKE '{$dateVal}%' AND ((o_vatav != 0) OR (sender LIKE 'VAT %') OR (receiver LIKE 'VAT %'))";
        } else {
            $condition = "is_deleted = 0 AND `date` LIKE '{$dateVal}%' AND ((o_vatav != 0 OR type='namu') AND ((sender LIKE 'VAT %' AND receiver_party_id IN({$accessibleParty})) OR (receiver LIKE 'VAT %' AND sender_party_id IN({$accessibleParty}))))";
        }

        if (strtotime(date('Y-m-d', strtotime($dateVal))) < strtotime(date('Y-m-d'))) {
            $vatavData = VatavBackUp::whereRaw($condition)->get();
        } else {
            $vatavData = Vatav::whereRaw($condition)->get();
        }

        $data['vatavList'] = $vatavData;
        $data['sumData'] = 0;

        foreach ($data['vatavList'] as $amountVal) {
            if (Str::contains($amountVal->sender, "VAT ")) {
                if ($amountVal->type == 'namu') {
                    $amountVal->o_vatav = abs($amountVal->sender_total_amount);
                } else {
                    $amountVal->o_vatav = -1 * abs($amountVal->sender_total_amount);
                }
            } elseif (Str::contains($amountVal->receiver, "VAT ")) {
                if ($amountVal->type == 'namu') {
                    $amountVal->o_vatav = -1 * abs($amountVal->receiver_total_amount);
                } else {
                    $amountVal->o_vatav = abs($amountVal->receiver_total_amount);
                }
            } else {
                // Do nothing
            }

            $data['sumData'] += $amountVal->o_vatav;
        }
        // dd($data);

        if ($print == '1') {
            return view('pages.party.vatavrangereport', $data);
        } else {
            $inputParameter = [
                'date' => $dateVal,
                'downloaded_by' => auth()->user()->name,
            ];

            $data['inputs'] = $inputParameter;
            $data['sumData'] = $data['sumData'];

            $mpdf = new \Mpdf\Mpdf();
            $html = view('pages.reportdownload.vatavreport', compact('data'))->render();
            $mpdf->WriteHTML($html);
            $pdfFilePath = "Vatav_report_" . date("Y-m-d H:i:s") . ".pdf";
            $mpdf->Output($pdfFilePath, "D");
        }
    }

    public function get_daywise_vatav_report(Request $request, $print = 0)
    {
        $dateVal = request()->input('month');
        $explodedDate = explode("-", $dateVal);
        $month = $explodedDate[0];
        $year = $explodedDate[1];

        $vatavData = Vatav::whereRaw("is_deleted = 0 AND MONTH(date) = {$month} AND YEAR(date) = {$year} AND ((o_vatav != 0) OR (sender LIKE 'VAT %' OR receiver LIKE 'VAT %'))")
            ->union(
                VatavBackUp::whereRaw("is_deleted = 0 AND MONTH(date) = {$month} AND YEAR(date) = {$year} AND ((o_vatav != 0) OR (sender LIKE 'VAT %' OR receiver LIKE 'VAT %'))")
            )
            ->get();

        $data['vatavList'] = [];
        $data['sumData'] = 0;

        foreach ($vatavData as $amountVal) {
            if (Str::contains($amountVal->sender, "VAT ")) {
                if ($amountVal->type == 'namu') {
                    $amountVal->o_vatav = abs($amountVal->sender_total_amount);
                } else {
                    $amountVal->o_vatav = 1 * abs($amountVal->sender_total_amount);
                }
            } elseif (Str::contains($amountVal->receiver, "VAT ")) {
                if ($amountVal->type == 'namu') {
                    $amountVal->o_vatav = -1 * abs($amountVal->receiver_total_amount);
                } else {
                    $amountVal->o_vatav = abs($amountVal->receiver_total_amount);
                }
            }

            $tmpArr['date'] = $amountVal->date;
            $tmpArr['vatav'] = $amountVal->o_vatav;
            $data['tmparrayarray'][] = $tmpArr;
            $data['sumData'] += $amountVal->o_vatav;
        }

        $finalArr = [];
        foreach ($data['tmparrayarray'] as $k => $v) {
            $date = date('Y-m-d', strtotime($v['date']));
            $finalArr[$date] = ($finalArr[$date] ?? 0) + $v['vatav'];
        }

        $data['vatavList'] = $finalArr;

        if ($print == '1') {
            return view('pages.party.daywise_vatav_report', $data);
        } else {
            $input_parameter = [
                'date' => $date,
                'downloaded_by' => auth()->user()->company_name,
            ];

            $data_arr['inputs'] = $input_parameter;
            $pdfFilePath = "vatav_daywise_vatav_report_" . date("Y-m-d H:i:s") . ".pdf";
            $mpdf = new \Mpdf\Mpdf();
            ob_end_clean();

            $data_arr['vatavList'] = $data['vatavList'];
            $html = view('pages.reportdownload.daywise_vatav_report', $data_arr)->render();
            $mpdf->WriteHTML($html);
            $mpdf->Output($pdfFilePath, "D");
        }
    }

    public function get_monthwise_vatav_report(Request $request, $print = 0)
    {
        $year = request()->input('year');

        $vatavData = Vatav::whereRaw("is_deleted = 0 AND YEAR(date) = {$year} AND ((o_vatav != 0) OR (sender LIKE 'VAT %' OR receiver LIKE 'VAT %'))")
            ->union(
                VatavBackUp::whereRaw("is_deleted = 0 AND YEAR(date) = {$year} AND ((o_vatav != 0) OR (sender LIKE 'VAT %' OR receiver LIKE 'VAT %'))")
            )
            ->get();

        $data['vatavList'] = [];
        $data['sumData'] = 0;

        foreach ($vatavData as $amountVal) {
            if (Str::contains($amountVal->sender, "VAT ")) {
                if ($amountVal->type == 'namu') {
                    $amountVal->o_vatav = abs($amountVal->sender_total_amount);
                } else {
                    $amountVal->o_vatav = 1 * abs($amountVal->sender_total_amount);
                }
            } elseif (Str::contains($amountVal->receiver, "VAT ")) {
                if ($amountVal->type == 'namu') {
                    $amountVal->o_vatav = -1 * abs($amountVal->receiver_total_amount);
                } else {
                    $amountVal->o_vatav = abs($amountVal->receiver_total_amount);
                }
            }

            $tmpArr['date'] = $amountVal->date;
            $tmpArr['vatav'] = $amountVal->o_vatav;
            $data['tmparrayarray'][] = $tmpArr;
            $data['sumData'] += $amountVal->o_vatav;
        }

        $finalArr = [];
        foreach ($data['tmparrayarray'] as $k => $v) {
            $date = date('m', strtotime($v['date']));
            $finalArr[$date] = ($finalArr[$date] ?? 0) + $v['vatav'];
        }

        $data['vatavList'] = $finalArr;

        if ($print == '1') {
            return view('pages.party.month_wise_vatav_report', $data);
        } else {
            $input_parameter = [
                'date' => $year,
                'downloaded_by' => auth()->user()->company_name,
            ];

            $data_arr['inputs'] = $input_parameter;
            $pdfFilePath = "vatav_month_vatav_report_" . date("Y-m-d H:i:s") . ".pdf";
            $mpdf = new \Mpdf\Mpdf();
            ob_end_clean();

            $data_arr['vatavList'] = $data['vatavList'];
            $html = view('pages.reportdownload.month_wise_vatav_report', $data_arr)->render();
            $mpdf->WriteHTML($html);
            $mpdf->Output($pdfFilePath, "D");
        }
    }

    public function partydetails($id)
    {
        $entryDetails = Vatav::where('id', $id)->first();

        if (!$entryDetails) {
            $entryDetails = VatavBackUp::where('id', $id)->first();
        }

        $senderDetails = $this->getPartyDetailsFromId($entryDetails->sender_party_id);

        if (!empty($senderDetails->commission_party)) {
            $senderCommissionDetails = Party::select('code', 'name', 'vatav')
                ->where('id', $senderDetails->commission_party)
                ->first();

            $entryDetails->sender_commission_party_name = $senderCommissionDetails->name ?? '';
        } else {
            $entryDetails->sender_commission_party_name = '';
        }

        $receiverDetails = $this->getPartyDetailsFromId($entryDetails->receiver_party_id);

        if (!empty($receiverDetails->commission_party)) {
            $receiverCommissionDetails = Party::select('code', 'name', 'vatav')
                ->where('id', $receiverDetails->commission_party)
                ->first();

            $entryDetails->receiver_commission_party_name = $receiverCommissionDetails->name ?? '';
        } else {
            $entryDetails->receiver_commission_party_name = '';
        }

        $entryDetails->sender_code = $senderDetails->code;
        $entryDetails->receiver_code = $receiverDetails->code;
        $entryDetails->final_sender_name = $senderDetails->name;
        $entryDetails->final_receiver_name = $receiverDetails->name;

        return view('pages.party.partydetails', compact('entryDetails'));
    }

    public function vatavDelete(Request $request){
        $idArray = $request->input('id');
        
        foreach($idArray as $id){
            $vatav = Vatav::find($id);

            if(!empty($vatav)){
                $vatav->is_deleted = 1;
                $vatav->deleted_by = auth()->user()->company_name;
                $vatav->deleted_time = date('Y-m-d H:i:s');

                $vatav->update();
                $vatav->delete();
            }
        }
        
        $data['msg'] = 'Data deleted successfully';
        $data['status'] = 1;
        echo json_encode($data);
        exit;
    }

    public function accountClosed(Request $request)
    {
        $value = $request->input('value');
        $vatav_id = $request->input('vatav_id');
        $party_id = $request->input('party_id');
        $created_date = now();
        $updated_date = now();

        $existingRecord = AccountClose::where('party_id', $party_id)
            ->where('vatav_id', $vatav_id)
            ->first();

        if (!$existingRecord) {
            $accountClose = new AccountClose();
            $accountClose->party_id = $party_id;
            $accountClose->vatav_id = $vatav_id;
            $accountClose->is_close = $value;
            $accountClose->created_date = $created_date;
            $accountClose->updated_date = $updated_date;
            $result = $accountClose->save();
        } else {
            $existingRecord->is_close = $value;
            $existingRecord->updated_date = $updated_date;
            $result = $existingRecord->save();
        }

        if ($result) {
            $returnData = "success##Account closed successfully";
        } else {
            $returnData = "error##Account not close";
        }

        return $returnData;
    }
}