<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Traits\PartyBalanceTrait;
use Illuminate\Http\Request;
use App\Http\Traits\PartyTrait;
use App\Http\Traits\PriceTrait;
use App\Http\Traits\VatavTrait;
use App\Models\Vatav;
use App\Models\VatavBackUp;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\DB;

class AutovatavController extends Controller
{
    use PartyTrait;
    use PriceTrait;
    use VatavTrait;
    use PartyBalanceTrait;

    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware(function ($request, $next) {
            $permissionArray = session('permissionArray');
            if (!in_array('Autovatav', $permissionArray)) {
                return redirect('Admin/dashboard');
            } else{
                return $next($request);
            }
        });
    }


    public function index()
    {

        $allVatav = Vatav::where('type', 'auto')->get();
        $role = session('role');
        if ($role == 'superadmin') {
            $data['is_access'] = 1;
        } else {
            $data['is_access'] = 0;
        }
        return view('pages.autovatav.index', compact('allVatav'), $data);
    }

    public function add()
    {
        $party_ids = session('access_party');
        $role = session('role');
        if ($role == 'superadmin') {
            $data['is_access'] = 1;
        } else {
            $data['is_access'] = 0;
        }
        $partList = $this->getSenderParty($party_ids, $role);
        $receiverList = $this->getReceiverParty();
        $token = Str::random(7);
        $data['token'] = $token;
        return view('pages.autovatav.add', compact('partList', 'receiverList'), $data);
    }

    #[NoReturn] public function get_price_range(Request $request): void
    {
        // dd($request);
        $amount = $request->input('amount');
        $minimum = $request->input('minimum');
        $is_receiver_based = $request->input('is_receiver_based');
        $receiver_based_commission = $request->input('receiver_based_commission');
        // dd($is_receiver_based);

        if ($is_receiver_based == 1) {
            if ($amount <= 55000) {
                $calculate_vatav_on = $this->generate_price_range($amount);
                if ($minimum == 0 || $minimum == '') {
                    $calculate_vatav_on = $calculate_vatav_on;
                } else {
                    $calculate_vatav_on = ($calculate_vatav_on * $minimum) / 100;
                }
            } else {
                if ($receiver_based_commission <= 0 || $receiver_based_commission == '') {
                    $calculate_vatav_on = $this->generate_price_range($amount);
                    if ($minimum == 0 || $minimum == '') {
                        $calculate_vatav_on = $calculate_vatav_on;
                    } else {
                        $calculate_vatav_on = ($calculate_vatav_on * $minimum) / 100;
                    }
                } else {
                    $calculate_vatav_on = ($amount * $receiver_based_commission) / 100000;
                    
                    if ($minimum == 0 || $minimum == '') {
                        $calculate_vatav_on = $calculate_vatav_on;
                    } else {
                        $calculate_vatav_on = ($calculate_vatav_on * $minimum) / 100;
                    }
                    $calculate_vatav_on = round($calculate_vatav_on / 10) * 10;
                    
                }
            }
        } else {
            $calculate_vatav_on = $this->generate_price_range($amount);
            if ($minimum == 0 || $minimum == '') {
                $calculate_vatav_on = $calculate_vatav_on;
            } else {
                $calculate_vatav_on = ($calculate_vatav_on * $minimum) / 100;
            }
        }

        echo json_encode($calculate_vatav_on);
        exit;
    }

    #[NoReturn] public function calculate_vatav(Request $request): void
    {
        $calculate_vatav_on = $request->input('calculate_vatav_on');
        $flag = $request->input('flag');
        $amount = $request->input('amount');
        $sender_party = $request->input('sendercode');
        $receiver_party = $request->input('receivercode');

        if ($amount < 0) {
            $newamt = 0 - $amount;
        } else {
            $newamt = $amount;
        }
        if ($flag == 1) {
            $calculate_vatav_on_new = $this->calculateAutoVatav($calculate_vatav_on, $sender_party, $receiver_party, $amount, $flag);
        }
        // dd($calculate_vatav_on_new);
        // $sender_percentage                = $request->input('sender_percentage');
        // $receiver_percentage              = $request->input('receiver_percentage');
        // $sender_balance                   = ($request->input('sender_balance')) * 100;
        // $receiver_balance                 = ($request->input('receiver_balance')) * 100;

        $data['sender_vatav_amount'] = $calculate_vatav_on_new['sender_vatav_amount'];
        $data['receiver_vatav_amount'] = $calculate_vatav_on_new['receiver_vatav_amount'];

        $data['our_vatav'] = $calculate_vatav_on_new['our_vatav'];
        $data['sender_total_amount'] = $calculate_vatav_on_new['sender_total_amount'];
        $data['receiver_total_amount'] = $calculate_vatav_on_new['receiver_total_amount'];
        $data['sender_closing_balance'] = $calculate_vatav_on_new['sender_closing_balance'];
        $data['receiver_closing_balance'] = $calculate_vatav_on_new['receiver_closing_balance'];
        $data['calculate_vatav_on'] = $calculate_vatav_on;
        $data['sender_commission_amount'] = $calculate_vatav_on_new['sender_commission_amount'];
        $data['receiver_commission_amount'] = $calculate_vatav_on_new['receiver_commission_amount'];
        //}
        echo json_encode($data);
        exit;
    }

    public function store(Request $request)
    {
        $user = auth()->user();
        $reqData = $request->all();
//         dd($reqData);

        $o_vatav = 0;

        $date = $reqData['date'];
        $date = date('Y-m-d H:i:s', strtotime($date));
        $receive_time = date('Y-m-d H:i:s');

        $sender_id = $reqData['sender'];
        $receiver_id = $reqData['receiver'];

        $senderDetail = $this->getPartyDetailsFromId($sender_id);
        $receiverDetail = $this->getPartyDetailsFromId($receiver_id);

        $sender_name = $reqData['sender_name'];
        $sender_contact = $reqData['sender_contact'];
        $receiver_name = $reqData['receiver_name'];
        $our_vatav = $reqData['our_vatav'];
        $receiver_contact = $reqData['receiver_contact'];
        $amount = $reqData['amount'];
        $remarks = $reqData['remarks'];
        $sender_vatav_amount = $reqData['sender_vatav_amount'];
        $receiver_vatav_amount = $reqData['receiver_vatav_amount'];
        $sender_opening_balance = $reqData['sender_opening_balance'];
        $sender_closing_balance = $reqData['sender_closing_balance'];
        $receiver_opening_balance = $reqData['receiver_opening_balance'];
        $receiver_closing_balance = $reqData['receiver_closing_balance'];
        $sender_partnership_firm = $reqData['sender_partnership_firm'];
        $receiver_partnership_firm = $reqData['receiver_partnership_firm'];
        $sender_total_amount = $reqData['sender_total_amount'];
        $receiver_total_amount = $reqData['receiver_total_amount'];
        $s_adhar_number = $reqData['s_adhar_number'];
        $total_vatav_label = $reqData['vatav_on_receiver'];
        $sender_commission = $reqData['sender_commission'];
        $receiver_commission = $reqData['receiver_commission'];
        $token = $reqData['token'];

        $sender = $senderDetail->code . ' (' . $senderDetail->name . ')';
        $receiver = $receiverDetail->code . ' (' . $receiverDetail->name . ')';

        $sender_commission_party_id = $senderDetail->commission_party;
        $receiver_commission_party_id = $receiverDetail->commission_party;

        // if($sender_partnership_firm == 1 && $receiver_partnership_firm == 1)
        // {
        //     echo 'condition 1'.'<br>';
        //     echo $total_vatav_label.'<br>';
        //     echo $sender_commission.'<br>';
        //     echo $receiver_commission.'<br>';
        //     $o_vatav = $sender_commission - $receiver_commission;
        //     echo $o_vatav.'<br>';
        //     exit;
        // }
        // else if($sender_partnership_firm == 1 && $receiver_partnership_firm != 1)
        // {
        //     echo 'condition 2'.'<br>';
        //     echo $total_vatav_label.'<br>';
        //     echo $sender_commission.'<br>';
        //     echo $receiver_vatav_amount.'<br>';
        //     $o_vatav = $sender_commission - $receiver_vatav_amount;
        //     echo $o_vatav.'<br>';
        //     exit;
        // }
        // else if($sender_partnership_firm != 1 && $receiver_partnership_firm == 1)
        // {
        //     echo 'condition 3'.'<br>';
        //     echo $total_vatav_label.'<br>';
        //     echo $sender_vatav_amount.'<br>';
        //     echo $receiver_commission.'<br>';
        //     $o_vatav = $sender_vatav_amount - $receiver_commission;
        //     echo $o_vatav.'<br>';
        //     exit;
        // }
        // else if($sender_partnership_firm != 1 && $receiver_partnership_firm != 1)
        // {
        //     echo 'condition 4'.'<br>';
        //     echo $total_vatav_label.'<br>';
        //     echo $sender_vatav_amount.'<br>';
        //     echo $receiver_vatav_amount.'<br>';
        //     $o_vatav = $sender_vatav_amount - $receiver_vatav_amount;
        //     echo $o_vatav.'<br>';
        //     exit;
        // }
        // dd($o_vatav);


        $insertArray = array(
            'date' => $date,
            'sender' => $sender,
            'receiver' => $receiver,
            'sender_party_id' => $sender_id,
            'receiver_party_id' => $receiver_id,
            'amount' => $amount,
            'sender_total_amount' => $sender_total_amount,
            'receiver_total_amount' => $receiver_total_amount,
            'total_vatav_label' => $total_vatav_label,
            's_vatav' => $sender_vatav_amount,
            'r_vatav' => $receiver_vatav_amount,
            'o_vatav' => $our_vatav,
            'sender_commission_party_id' => $sender_commission_party_id,
            'receiver_commission_party_id' => $receiver_commission_party_id,
            'sender_commission_party_amount' => $sender_commission,
            'receiver_commission_party_amount' => $receiver_commission,
            'sender_is_partnership_firm' => $sender_partnership_firm,
            'receiver_partnership_firm' => $receiver_partnership_firm,
            'type' => 'auto',
            'sender_name' => $sender_name,
            'sender_contact' => $sender_contact,
            'receiver_name' => $receiver_name,
            'receiver_contact' => $receiver_contact,
            'remarks' => $remarks,
            'token' => $token,
            'entered_by' => $user->company_name,
            'status' => 'Pending',
            'is_deleted' => 0,
            'receive_time' => $receive_time,
            's_adhar_number' => $s_adhar_number,

        );
        $autoEntry = Vatav::create($insertArray);
        $id = $autoEntry->id;

        $updateArray = array(
            'serial_no' => $id,
            'sender_token' => 's' . $id,
            'receiver_token' => 'r' . $id,
        );
        $updateEntry = Vatav::where('id', $id)->update($updateArray);

        return redirect()->route('autovatav.add');
    }

    public function ajax_pagination(Request $request)
    {
        $reqData = $request->all();
        $data = array();

        $role = session('role');
        $is_access = 0;
        if ($role == 'superadmin') {
            $is_access = 1;
        }
        $currentDate = date('Y-m-d');
        $date = date('Y-m-d', strtotime($reqData['date']));
        // $search = $reqData['search'];

        if (strtotime($currentDate) > strtotime($date)) {
            $query = VatavBackUp::where('type', 'auto')
                ->where('is_deleted', 0)
                ->where('ho_approve', 0)
                ->where('date', 'LIKE', $date . '%');
        } else {
            $query = Vatav::where('type', 'auto')
                ->where('is_deleted', 0)
                ->where('ho_approve', 0)
                ->where('date', 'LIKE', $date . '%');
        }


        if (array_key_exists('search', $reqData) && !empty(trim($reqData['search'])) && (trim($reqData['search']) != '')) {
            $search = trim($reqData['search']);
            $query = $query->where(function ($query) use ($search) {
                $query->where('sender', 'LIKE', '%' . $search . '%')
                    ->orWhere('receiver', 'LIKE', '%' . $search . '%')
                    ->orWhere('amount', 'LIKE', '%' . $search . '%')
                    ->orWhere('token', 'LIKE', '%' . $search . '%')
                    ->orWhere('serial_no', 'LIKE', '%' . $search . '%')
                    ->orWhere('id', 'LIKE', '%' . $search . '%')
                    ->orWhere('sender_contact', 'LIKE', '%' . $search . '%')
                    ->orWhere('receiver_contact', 'LIKE', '%' . $search . '%');
            });
            $data['search'] = $search;
        } else {
            $data['search'] = '';
        }
        if (array_key_exists('sorting', $reqData) && !empty(trim($reqData['sorting'])) && (trim($reqData['sorting']) != '')) {
            $sortingArray = explode('#', $reqData['sorting']);

            $query = $query->orderBy($sortingArray[0], $sortingArray[1]);
            $data['sorting'] = $sortingArray[0] . '#' . $sortingArray[1];
        } else {
            $query = $query->orderBy('id', 'desc');
            $data['sorting'] = 'id#desc';
        }
        $data['startRow'] = 1;

        $autovatavLists = $query->get();
        $accessible_party = $this->getAccessParty();
        $accessibleParty = explode(",", $accessible_party);

        $autovatavList = array();
        foreach ($autovatavLists as $_autovatavLists) {
            if (in_array($_autovatavLists->sender_party_id, $accessibleParty)) {
                $autovatavList[] = $_autovatavLists;
            }
        }

        return view('pages.autovatav.pending_entry', compact('autovatavList', 'data', 'is_access'));
    }

    public function approved_entry(Request $request)
    {
        $reqData = $request->all();
        $data = array();

        $role = session('role');
        $is_access = 0;
        if ($role == 'superadmin') {
            $is_access = 1;
        }
        $currentDate = date('Y-m-d');
        $date = date('Y-m-d', strtotime($reqData['date']));

        if (strtotime($currentDate) > strtotime($date)) {
            $query = VatavBackUp::where('type', 'auto')
                ->where('is_deleted', 0)
                ->where('ho_approve', 1)
                ->where('date', 'LIKE', $date . '%');
        } else {
            $query = Vatav::where('type', 'auto')
                ->where('is_deleted', 0)
                ->where('ho_approve', 1)
                ->where('date', 'LIKE', $date . '%');
        }


        if (array_key_exists('search', $reqData) && !empty(trim($reqData['search'])) && (trim($reqData['search']) != '')) {
            $search = trim($reqData['search']);
            $query = $query->where(function ($query) use ($search) {
                $query->where('sender', 'LIKE', '%' . $search . '%')
                    ->orWhere('receiver', 'LIKE', '%' . $search . '%')
                    ->orWhere('amount', 'LIKE', '%' . $search . '%')
                    ->orWhere('serial_no', 'LIKE', '%' . $search . '%')
                    ->orWhere('id', 'LIKE', '%' . $search . '%')
                    ->orWhere('sender_contact', 'LIKE', '%' . $search . '%')
                    ->orWhere('receiver_contact', 'LIKE', '%' . $search . '%')
                    ->orWhere('token', 'LIKE', '%' . $search . '%');
            });
            $data['search'] = $search;
        } else {
            $data['search'] = '';
        }
        if (array_key_exists('sorting', $reqData) && !empty(trim($reqData['sorting'])) && (trim($reqData['sorting']) != '')) {
            $sortingArray = explode('#', $reqData['sorting']);

            $query = $query->orderBy($sortingArray[0], $sortingArray[1]);
            $data['sorting'] = $sortingArray[0] . '#' . $sortingArray[1];
        } else {
            $query = $query->orderBy('id', 'desc');
            $data['sorting'] = 'id#desc';
        }
        $data['startRow'] = 1;

        $autovatavLists = $query->get();
        $accessible_party = $this->getAccessParty();
        $accessibleParty = explode(",", $accessible_party);

        $autovatavList = array();
        foreach ($autovatavLists as $_autovatavLists) {
            if (in_array($_autovatavLists->sender_party_id, $accessibleParty)) {
                $autovatavList[] = $_autovatavLists;
            }
        }


        return view('pages.autovatav.approve_entry', compact('autovatavList', 'data', 'is_access'));
    }

    public function approve_vatav_entry($vatav_id)
    {
        date_default_timezone_set('Asia/Kolkata');
        $update_date = now();

        $vatav = Vatav::find($vatav_id);

        if ($vatav) {
            $vatav->ho_approve = 1;
            $vatav->receive_time = $update_date;
            $vatav->save();
        }

        $data['msg'] = "Data updated successfully";
        echo json_encode($data);
    }


    public function delete($id)
    {
        $user = auth()->user();
        $vatav = Vatav::find($id);

        if (!empty($vatav)) {
            $vatav->is_deleted = 1;
            $vatav->status = 'Deleted';
            $vatav->deleted_by = $user->company_name;
            $vatav->deleted_time = date('Y-m-d H:i:s');
            $vatav->update();
            $vatav->delete();

            $response['IsSuccess'] = true;
            $response['Message'] = 'Vatav delete successfully';
        }
        echo json_encode($response);
        exit;
    }

    public function entryDetails(Request $request, $id)
    {
        $entryDetails = Vatav::find($id);

        if (!empty($entryDetails)) {
            $senderPartyBalance = $this->getPartyClosingBalanceFromId($entryDetails->sender_party_id);
            $senderDetails = $this->getPartyDetailsFromId($entryDetails->sender_party_id);

            $receiverPartyBalance = $this->getPartyClosingBalanceFromId($entryDetails->receiver_party_id);
            $receiverDetails = $this->getPartyDetailsFromId($entryDetails->receiver_party_id);

            $entryDetails->sender_code = $senderDetails->code;
            $entryDetails->receiver_code = $receiverDetails->code;
            $entryDetails->final_sender_name = $senderDetails->name;
            $entryDetails->final_receiver_name = $receiverDetails->name;
            $entryDetails->sender_vatav_percentage = $senderDetails->sender_vatav;
            $entryDetails->receiver_vatav_percentage = $receiverDetails->receiver_vatav;
            if (!empty($senderPartyBalance)) {
                $entryDetails->sender_opening_balance = $senderPartyBalance->closing_balance;
                $entryDetails->sender_closing_balance = $senderPartyBalance->closing_balance + $entryDetails->sender_total_amount;
            } else {
                $entryDetails->sender_opening_balance = 0.00;
                $entryDetails->sender_closing_balance = 0.00;
            }
            if (!empty($receiverPartyBalance)) {
                $entryDetails->receiver_opening_balance = $receiverPartyBalance->closing_balance;
                $entryDetails->receiver_closing_balance = $receiverPartyBalance->closing_balance + $entryDetails->receiver_total_amount;
            } else {
                $entryDetails->receiver_opening_balance = 0.00;
                $entryDetails->receiver_closing_balance = 0.00;
            }

            $entryDetails->sender_closing_balance_normal = $entryDetails->sender_opening_balance + $entryDetails->amount;
            $entryDetails->receiver_closing_balance_normal = $entryDetails->receiver_opening_balance - $entryDetails->amount;
            $entryDetails->sender_total_amount_normal = $entryDetails->amount;
            $entryDetails->date = date("d-m-Y H:i:s", strtotime($entryDetails->date));
            $dataResult = view('pages.autovatav.add_edit', compact('entryDetails'))->render();
            echo $dataResult;
            exit;
        }
    }

    public function edit_autovatav(Request $request)
    {
        $amount = $request->input('amount');
        $total_vatav_label = $request->input('total_vatav_label');
        $sender_vatav = $request->input('sender_vatav');
        $receiver_vatav = $request->input('receiver_vatav');
        $our_vatav = $request->input('our_vatav');
        $our_vatav = $request->input('our_vatav');
        $sender_commission = $request->input('sender_commission');
        $receiver_commission = $request->input('receiver_commission');
        $receiver_total_amount = $request->input('receiver_total_amount');
        $sender_total_amount = $request->input('sender_total_amount');
        $sender_name = $request->input('sender_name');
        $sender_contact = $request->input('sender_contact');
        $receiver_name = $request->input('receiver_name');
        $receiver_contact = $request->input('receiver_contact');
        $remarks = $request->input('remarks');
        $id = $request->input('id');
        $type = $request->input('type');
        $upadte_date = date('Y-m-d H:i:s');

        $updateArray = array(
            'remarks' => $remarks,
            'sender_name' => $sender_name,
            'sender_contact' => $sender_contact,
            'receiver_name' => $receiver_name,
            'receiver_contact' => $receiver_contact,
            'type' => $type,
            'total_vatav_label' => $total_vatav_label,
            's_vatav' => $sender_vatav,
            'sender_total_amount' => $sender_total_amount,
            'receiver_total_amount' => $receiver_total_amount,
            'r_vatav' => $receiver_vatav,
            'o_vatav' => $our_vatav,
            'sender_commission_party_amount' => $sender_commission,
            'receiver_commission_party_amount' => $receiver_commission,
            'ho_approve' => 1,
            'receive_time' => $upadte_date,
        );

        Vatav::where('id', $id)->update($updateArray);
        return redirect('Admin/autovatav');
    }
}
