<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Company;
use App\Models\Modules;
use App\Models\Party;
use App\Models\User;
use Illuminate\Contracts\View\Factory;
use Illuminate\Contracts\View\View;
use Illuminate\Foundation\Application;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Routing\Redirector;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use JetBrains\PhpStorm\NoReturn;

class CompanyController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware(function ($request, $next) {
            $permissionArray = session('permissionArray');
            if (!in_array('Users', $permissionArray)) {
                return redirect('Admin/dashboard');
            } else{
                return $next($request);
            }
        });
    }

    public function index(): View|Application|Factory|\Illuminate\Contracts\Foundation\Application
    {
        // $user = auth()->user();
        $role = session('role');

        $query = Company::orderBy('id', 'asc');
        $query = $query->whereNotIn('id', ['1']);
        if ($role != "superadmin") {
            $query = $query->whereNull('role');
        }
        $allCompanies = $query->get();
        return view('pages.company.index', compact('allCompanies'));
    }

    public function add(): View|Application|Factory|\Illuminate\Contracts\Foundation\Application
    {
        $modulesList = Modules::get();
        $party = Party::orderBy('code', 'asc')->get();
        $array = array();
        foreach ($party as $_senderData) {
            // if($_senderData->code != "VAT"){
            $array[] = array(
                'id' => $_senderData->id,
                'value' => $_senderData->code . ' (' . $_senderData->name . ')',
            );
            // }
        }
        $partyList = $array;
        return view('pages.company.add', compact("modulesList", "partyList"));
    }

    public function store(Request $request): Application|Redirector|RedirectResponse|\Illuminate\Contracts\Foundation\Application
    {
        $validator = Validator::make($request->all(), [
            'company_name' => 'required',
            'username' => 'required',
            'password' => 'required',
            'permissions' => 'required'
        ]);
        if ($validator->fails()) {
            return redirect()->back()->withErrors($validator)->withInput();
        }

        $user_name = $request->session()->get('user_name');
        $password = Hash::make($request->input('password'));
        $company = new Company();

        $permissions = implode(",", $request->input('permissions'));
        $parties = implode(",", $request->input('parties'));

        $company->company_name = $request->input('company_name');
        $company->company_email = $request->input('company_email');
        $company->company_contact = $request->input('company_contact');
        $company->username = $request->input('username');
        $company->password = $password;
        $company->permissions = $permissions;
        $company->created_by = $user_name;
        $company->parties = $parties;

        $company->save();

//        $userDetails = new User();
//        $userDetails->name = $request->input('company_name');
//        $userDetails->email = $request->input('username');
//        $userDetails->password = $password;
//        $userDetails->user_type = "company";
//        $userDetails->user_id = $company->id;
//        $userDetails->save();

        return redirect('Admin/users')->with('status', 'Company add Successfully');
    }

    public function edit($id): View|Application|Factory|\Illuminate\Contracts\Foundation\Application
    {
        $modulesList = Modules::get();
        $companyDetails = Company::find($id);
        $party = Party::orderBy('code', 'asc')->get();
        $array = array();
        foreach ($party as $_senderData) {
            // if($_senderData->code != "VAT"){
            $array[] = array(
                'id' => $_senderData->id,
                'value' => $_senderData->code . ' (' . $_senderData->name . ')',
            );
            // }
        }
        $partyList = $array;
        $partyCheckboxList = explode(",", $companyDetails->parties);
        return view('pages.company.edit', compact('partyCheckboxList', 'partyList', 'modulesList', 'companyDetails'));
    }

    public function update(Request $request, $id): Application|Redirector|RedirectResponse|\Illuminate\Contracts\Foundation\Application
    {
        $validator = Validator::make($request->all(), [
            'company_name' => 'required',
            'username' => 'required',
            'permissions' => 'required'
        ]);

        if ($validator->fails()) {
            return redirect()->back()->withErrors($validator)->withInput();
        }

        $password = $request->input('password');

        $company = Company::find($id);

        $permissions = implode(",", $request->input('permissions'));
        $parties = implode(",", $request->input('parties'));

        $company->company_name = $request->input('company_name');
        $company->company_email = $request->input('company_email');
        $company->company_contact = $request->input('company_contact');
        $company->username = $request->input('username');
        $new_password = Hash::make($password);
        if (isset($password) && $password != "") {
            $company->password = $new_password;
        }
        $company->permissions = $permissions;
        $company->parties = $parties;

        $company->update();

//        $userDetails = User::where("user_id", $id)->where('user_type', 'company')->first();
//        $userDetails->name = $request->input('company_name');
//        $userDetails->email = $request->input('username');
//        if (isset($password) && $password != "") {
//            $userDetails->password = $new_password;
//        }
//        $userDetails->update();

        return redirect('Admin/users')->with('status', 'Company add Successfully');
    }

    #[NoReturn] public function delete($id): void
    {
        $response = array();
        $company = Company::find($id);

        if (!empty($company)) {
            $company->delete();
            $response['IsSuccess'] = true;
            $response['Message'] = "Company delete successfully";
        }
        echo json_encode($response);
        exit;
    }

    public function logout_user($id): Application|Redirector|RedirectResponse|\Illuminate\Contracts\Foundation\Application
    {

        $updateArray['is_loggedin'] = 0;
        Company::where('id', $id)->update($updateArray);
        return redirect('Admin/users');
    }


}