<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Traits\DeviceTrait;
use App\Http\Traits\NotificationTrait;
use App\Http\Traits\PartyTrait;
use App\Models\DeviceToken;
use App\Models\Party;
use App\Models\Vatav;
use Illuminate\Http\Request;
use Illuminate\Routing\Redirector;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\RedirectResponse;
use Illuminate\Contracts\View\View;
use Illuminate\Foundation\Application;
use Illuminate\Contracts\View\Factory;
use JetBrains\PhpStorm\NoReturn;
use App\Models\PartyBalance;

class DashboardController extends Controller
{

    public function __construct()
    {
        $this->middleware('auth');
    }

    public function index()
    {

        $parties = Party::where('is_deleted', 0)->get();
        $final_arr = [];
        $closing_bal = [];
        foreach ($parties as $party) {
            $balance_data = PartyBalance::where('party_id', $party->id)->orderBy('date', 'DESC')->orderBy('id', 'DESC')->first();

            $cityarr = ["PARTY", "ACC", "PERSIONAL PARTY", "COMMITION"];

            if (!in_array($party->city, $cityarr)) {
                if ($balance_data) {
                    $closing_bal[$party->id] = $balance_data->closing_balance;
                    $temp_array['closing_balance'] = $balance_data->closing_balance;
                } else {
                    $closing_bal[$party->id] = 0;
                    $temp_array['closing_balance'] = 0;
                }
                $temp_array['id'] = $party->id;
                $temp_array['name'] = $party->name;
                $final_arr[] = $temp_array;
            }

            // dd($data);
        }
        // Sorting arrays
        $volume = array_column($final_arr, 'closing_balance');
        array_multisort($volume, SORT_DESC, $final_arr);

        arsort($closing_bal);
        $max_array = array_slice($closing_bal, 0, 15, true);

        asort($closing_bal);
        $min_array = array_slice($closing_bal, 0, 15, true);


        $data = array(
            'final_arr' => $final_arr,
            'max_array' => $max_array,
            'min_array' => $min_array,
            'role' => session('role'),
        );
        return view('pages.dashboard.dashboard', $data);
    }

    public function recalculate(): View|Application|Factory|\Illuminate\Contracts\Foundation\Application
    {
        return view('pages.dashboard.recalculate');
    }

    public function calculate(Request $request): View|Application|Factory|\Illuminate\Contracts\Foundation\Application
    {
        // dd($request);
        $date = $request->input('date');
        $reportDate = date('Y-m-d', strtotime($date));

        DB::select("call recalculate_balance('$reportDate')");

        $generalReportData = $this->getTmpGeneralReport();
        $generalReportData['date'] = $reportDate;
        // dd($generalReportData);

        return view('pages.party.tmp_general_report', $generalReportData);
    }

    public function getTmpGeneralReport(): array
    {

        $partyData = Party::get();
        $partyByIds = $partyData->keyBy('id');

        $partyBalanceData = DB::table('tmp_party_balance AS pb')
            ->join(DB::raw('(SELECT MAX(date) AS dt, party_id FROM tmp_party_balance GROUP BY party_id) AS pbb'), function ($join) {
                $join->on('pb.date', '=', 'pbb.dt')
                    ->on('pb.party_id', '=', 'pbb.party_id');
            })
            ->orderBy('pb.id', 'DESC')
            ->get();

        $partyArr = [];
        $resultSet = [];
        foreach ($partyBalanceData as $_partyBalance) {
            if (!in_array($_partyBalance->party_id, $partyArr)) {
                $_partyBalance->name = $partyByIds[$_partyBalance->party_id]->name;
                $_partyBalance->code = $partyByIds[$_partyBalance->party_id]->code;
                $partyArr[] = $_partyBalance->party_id;
                $resultSet[] = $_partyBalance;
            }
        }

        // Sort by closing balance
        usort($resultSet, function ($a, $b) {
            return $b->closing_balance <=> $a->closing_balance;
        });

        $data['allParties'] = $resultSet;
        return $data;
    }

    public function confirmReport($date): RedirectResponse
    {
        $reportDate = date('Y-m-d', strtotime($date));


        $pbData = DB::table('tmp_party_balance')
            ->orderBy('date')
            ->orderBy('id')
            ->get();

        $partyIdArray = [];

        $resTmpParty = DB::table('tmp_party_balance')
            ->select('party_id')
            ->distinct('party_id')
            ->orderBy('party_id')
            ->orderBy('date')
            ->orderBy('id')
            ->get();

        foreach ($resTmpParty as $_rec) {
            $partyId = $_rec->party_id;

            $opBalanceQuery = DB::table('party_balance')
                ->select('closing_balance')
                ->where('date', '<=', $reportDate)
                ->where('party_id', $partyId)
                ->orderByDesc('date')
                ->orderByDesc('id')
                ->limit(1)
                ->get();

            if ($opBalanceQuery->isEmpty()) {
                $opBalance = 0;
            } else {
                $opBalance = $opBalanceQuery[0]->closing_balance;

                // Update tmp_party_balance table with opening_balance and closing_balance
                DB::table('tmp_party_balance')
                    ->where('party_id', $partyId)
                    ->update([
                        'opening_balance' => DB::raw("opening_balance + $opBalance"),
                        'closing_balance' => DB::raw("closing_balance + $opBalance")
                ]);
            }
        }

        // Delete and insert data into party_balance table
        DB::table('party_balance')
            ->where('date', 'LIKE', "$reportDate%")
            ->delete();

        DB::table('party_balance')->insertUsing(
            ['opening_balance', 'closing_balance', 'party_id', 'vatav_id', 'status', 'date'],
            DB::table('tmp_party_balance')->select('opening_balance', 'closing_balance', 'party_id', 'vatav_id', 'status', 'date')
        );

        return redirect()->route('report.general');
    }

    public function update_closing_balance()
    {
        return view('pages.dashboard.update_closing_balance');
    }

    public function updateClosingBalance(Request $request)
    {
        $date = $request->input('date');
        $date = date('Y-m-d', strtotime($date));
        DB::statement("truncate party_closing_balance");
        DB::statement("call update_party_closing_balance('$date')");
        return redirect()->route('dashboard.updateClosingBalance');
    }
}
