<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Vatav;
use App\Http\Traits\PartyTrait;
use App\Http\Traits\VatavTrait;
use Illuminate\Support\Str;

class ManualvatavController extends Controller
{

    use PartyTrait;
    use VatavTrait;

    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware(function ($request, $next) {
            $permissionArray = session('permissionArray');
            if (!in_array('Manualvatav', $permissionArray)) {
                return redirect('Admin/dashboard');
            } else{
                return $next($request);
            }
        });
    }

    public function index()
    {

        $allVatav = Vatav::get();
        $role = session('role');
        if ($role == 'superadmin') {
            $data['is_access'] = 1;
        } else {
            $data['is_access'] = 0;
        }
        return view('pages.manual.index', compact('allVatav'), $data);
    }

    public function add()
    {
        $party_ids = session('access_party');
        $role = session('role');
        if ($role == 'superadmin') {
            $data['is_access'] = 1;
        } else {
            $data['is_access'] = 0;
        }
        $partList = $this->getSenderParty($party_ids, $role);
        $receiverList = $this->getReceiverParty();
        $token = Str::random(7);
        $data['token'] = $token;
        return view('pages.manual.add', $data, compact('partList', 'receiverList'),$data);
    }

    public function calculate_vatav(Request $request)
    {
        $calculate_vatav_on = $request->input('total_vatav');
        // dd($calculate_vatav_on);
        $flag = $request->input('flag');
        $amount = $request->input('amount');
        $sender_party = $request->input('sendercode');
        $receiver_party = $request->input('receivercode');
        $sender_balance = $request->input('sender_balance');
        $receiver_balance = $request->input('receiver_balance');

        $sender_vatav = $request->input('sender_vatav');
        $receiver_vatav = $request->input('receiver_vatav');
        $total_vatav = $request->input('total_vatav');

        $senderDetails = $this->getPartyDetailsFromId($sender_party);
        $receiverDetails = $this->getPartyDetailsFromId($receiver_party);

        $sender_percentage = $sender_vatav;
        $receiver_percentage = $receiver_vatav;

        if ($amount < 0) {
            $newamt = 0 - $amount;
        } else {
            $newamt = $amount;
        }
        
        $calculate_vatav_on_new = $this->calculate_manual_vatav($sender_party,$receiver_party,$amount,$calculate_vatav_on,$sender_percentage,$receiver_percentage,$sender_balance,$receiver_balance,$flag);
        // dd($calculate_vatav_on_new);

        $data['sender_vatav_amount']      = $calculate_vatav_on_new['sender_vatav_amount'];
        $data['receiver_vatav_amount']    = $calculate_vatav_on_new['receiver_vatav_amount'];

        $data['our_vatav']                = $calculate_vatav_on_new['our_vatav'];
        $data['sender_total_amount']      = $calculate_vatav_on_new['sender_total_amount'];
        $data['receiver_total_amount']    = $calculate_vatav_on_new['receiver_total_amount'];
        $data['sender_closing_balance']   = $calculate_vatav_on_new['sender_closing_balance'];
        $data['receiver_closing_balance'] = $calculate_vatav_on_new['receiver_closing_balance'];
        $data['calculate_vatav_on']       = $calculate_vatav_on;
        $data['sender_commission_amount']  = $calculate_vatav_on_new['sender_commission_amount'];
        $data['receiver_commission_amount'] = $calculate_vatav_on_new['receiver_commission_amount'];
        //}
        return json_encode($data);
        exit;
    }

    public function store(Request $request)
    {
        $user = auth()->user();
        $reqData = $request->all();
        // dd($reqData);

        $date = $reqData['date'];
        $date = date('Y-m-d H:i:s', strtotime($date));
        $receive_time = date('Y-m-d H:i:s');
        
        $sender_id = $reqData['sender'];
        $receiver_id = $reqData['receiver'];

        $senderDetail = $this->getPartyDetailsFromId($sender_id);
        $receiverDetail = $this->getPartyDetailsFromId($receiver_id);

        $sender_name = $reqData['sender_name'];
        $sender_contact = $reqData['sender_contact'];
        $receiver_name = $reqData['receiver_name'];
        $our_vatav = $reqData['our_vatav'];
        $receiver_contact = $reqData['receiver_contact'];
        $amount = $reqData['amount'];
        $remarks = $reqData['remarks'];
        $sender_vatav_amount = $reqData['sender_vatav_amount'];
        $receiver_vatav_amount = $reqData['receiver_vatav_amount'];
        $sender_opening_balance = $reqData['sender_opening_balance'];
        $sender_closing_balance = $reqData['sender_closing_balance'];
        $receiver_opening_balance = $reqData['receiver_opening_balance'];
        $receiver_closing_balance = $reqData['receiver_closing_balance'];
        $sender_partnership_firm   = $reqData['sender_partnership_firm'];
        $receiver_partnership_firm = $reqData['receiver_partnership_firm'];
        $sender_total_amount = $reqData['sender_total_amount'];
        $receiver_total_amount = $reqData['receiver_total_amount'];
        $total_vatav_label = $reqData['total_vatav'];
        $sender_commission = $reqData['sender_commission'];
        $receiver_commission = $reqData['receiver_commission'];
        $s_adhar_number = $reqData['s_adhar_number'];
        $token = $reqData['token'];

        $sender = $senderDetail->code .' ('. $senderDetail->name .')' ;
        $receiver = $receiverDetail->code .' ('. $receiverDetail->name .')' ;

        $sender_commission_party_id = $senderDetail->commission_party;
        $receiver_commission_party_id = $receiverDetail->commission_party;


        $created_at = date('Y-m-d H:i:s');

        $insertArray = array(
          'date' => $date,
          'sender' => $sender,
          'receiver' => $receiver,
          'sender_party_id' => $sender_id,
          'receiver_party_id' => $receiver_id,
          'amount' => $amount,
          'sender_total_amount' => $sender_total_amount,
          'receiver_total_amount' => $receiver_total_amount,
          'total_vatav_label' => $total_vatav_label,
          's_vatav' => $sender_vatav_amount,
          'r_vatav' => $receiver_vatav_amount,
          'o_vatav' => $our_vatav,
          'sender_commission_party_id' => $sender_commission_party_id,
          'receiver_commission_party_id' => $receiver_commission_party_id,
          'sender_commission_party_amount' => $sender_commission,
          'receiver_commission_party_amount' => $receiver_commission,
          'sender_is_partnership_firm' => $sender_partnership_firm,
          'receiver_partnership_firm' => $receiver_partnership_firm,
          'type' => 'manual',
          'sender_name' => $sender_name,
          'sender_contact' => $sender_contact,
          'receiver_name' => $receiver_name,
          'receiver_contact' => $receiver_contact,
          'remarks' => $remarks,
          'token' => $token,
          'entered_by' => $user->company_name,
          'status' => 'Pending',
           'is_deleted' => 0,
           'receive_time' => $receive_time,
           's_adhar_number' => $s_adhar_number,
        );
        // dd($insertArray);
        $autoEntry = Vatav::create($insertArray);
        $id = $autoEntry->id;

        $updateArray = array(
            'serial_no' => $id,
            'sender_token' => 's'.$id,
            'receiver_token' => 'r'.$id,
        );
        $updateEntry = Vatav::where('id',$id)->update($updateArray);

        return redirect()->route('manualvatav.add');
    }

    public function ajax_pagination(Request $request)
    {
        $reqData = $request->all();
        $data = array();

        $date = date('Y-m-d', strtotime($reqData['date']));
        // $search = $reqData['search'];

        $query = Vatav::where('type', 'manual')
            ->where('is_deleted', 0)
            ->where('date', 'LIKE', $date . '%');

        if (array_key_exists('search', $reqData) && !empty(trim($reqData['search'])) && (trim($reqData['search']) != '')) {
            $search = trim($reqData['search']);
            $query = $query->where(function ($q) use ($search) {
                $q = $q->where('sender', 'iLIKE', '%' . $search . '%')
                    ->orWhere('receiver', 'iLIKE', '%' . $search . '%')
                    ->orWhere('amount', 'LIKE', '%' . $search . '%');
            });
            $data['search'] = $search;
        } else {
            $data['search'] = '';
        }
        if (array_key_exists('sorting', $reqData) && !empty(trim($reqData['sorting'])) && (trim($reqData['sorting']) != '')) {
            $sortingArray = explode('#', $reqData['sorting']);

            $query = $query->orderBy($sortingArray[0], $sortingArray[1]);
            $data['sorting'] = $sortingArray[0] . '#' . $sortingArray[1];
        } else {
            $query = $query->orderBy('id', 'desc');
            $data['sorting'] = 'id#desc';
        }
        $data['startRow']      = 1;

        $startRow = $reqData['start'] ?? 1;
        $endRow = $reqData['length'] ?? 10;
        $tableTotalRows = $query->count();

        $autovatavList = $query->skip($startRow - 1)->take($endRow)->get();

        $data['startRow'] = $startRow;
        $data['endRow'] = $endRow;
        $data['tableTotalRows'] = $tableTotalRows;
        // dd($autovatavList);

        return view('pages.manual.ajax_pagination', compact('autovatavList', 'data'));
    }

    public function delete($id)
    {
        $user= auth()->user();
        $vatav = Vatav::find($id);

        if(!empty($vatav))
        {
            $vatav->is_deleted = 1;
            $vatav->status = 'Deleted';
            $vatav->deleted_by = $user->company_name;
            $vatav->deleted_time = date('Y-m-d H:i:s');
            $vatav->update();
            $vatav->delete();

            $response['IsSuccess'] = true;
            $response['Message'] = 'Vatav delete successfully';
        }
        echo json_encode($response);
        exit;
    }
}
