<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\Vatav;
use App\Http\Traits\PartyTrait;
use App\Http\Traits\VatavTrait;
use Illuminate\Support\Str;

class NamuController extends Controller
{
    use PartyTrait;
    use VatavTrait;

    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware(function ($request, $next) {
            $permissionArray = session('permissionArray');
            if (!in_array('Namu', $permissionArray)) {
                return redirect('Admin/dashboard');
            } else{
                return $next($request);
            }
        });

    }

    public function index()
    {

        $allVatav = Vatav::where('type', 'namu')->get();
        $role = session('role');
        if ($role == 'superadmin') {
            $data['is_access'] = 1;
        } else {
            $data['is_access'] = 0;
        }
        return view('pages.namu.index', compact('allVatav'), $data);
    }

    public function add()
    {
        $party_ids = session('access_party');
        $role = session('role');
        if ($role == 'superadmin') {
            $data['is_access'] = 1;
        } else {
            $data['is_access'] = 0;
        }
        $partList = $this->getNormalSenderParty($party_ids, $role);

        $receiverList = $this->getNormalReceiverParty();
        return view('pages.namu.add', $data, compact('partList', 'receiverList'));
    }

    public function calculate_vatav(Request $request)
    {

        $amount = $request->input('amount');
        $sender_balance = $request->input('sender_balance');
        $receiver_balance = $request->input('receiver_balance');
        $sendercode = $request->input('sendercode');
        $receivercode = $request->input('receivercode');

        $normalVatav = $this->calculateNamuVatav($amount, $sender_balance, $receiver_balance);

        $data['our_vatav'] = $normalVatav['our_vatav'];
        $data['sender_total_amount'] = $normalVatav['sender_total_amount'] ?? 0;
        $data['receiver_total_amount'] = $normalVatav['receiver_total_amount'] ?? 0;
        $data['sender_vatav_amount'] = $normalVatav['sender_vatav_amount'];
        $data['receiver_vatav_amount'] = $normalVatav['receiver_vatav_amount'];
        $data['sender_closing_balance'] = $normalVatav['sender_closing_balance'];
        $data['receiver_closing_balance'] = $normalVatav['receiver_closing_balance'];

        echo json_encode($data);
        exit;
    }

    public function store(Request $request)
    {
        $user = auth()->user();
        $reqData = $request->all();
        // dd($reqData);

        $date = $reqData['date'];
        $date = date('Y-m-d H:i:s', strtotime($date));
        $receive_time = date('Y-m-d H:i:s');

        $sender_id = $reqData['sender'];
        $receiver_id = $reqData['receiver'];

        $senderDetail = $this->getPartyDetailsFromId($sender_id);
        $receiverDetail = $this->getPartyDetailsFromId($receiver_id);

        $sender_name = $reqData['sender_name'];
        $sender_contact = $reqData['sender_contact'];
        $receiver_name = $reqData['receiver_name'];
        $our_vatav = $reqData['our_vatav'];
        $receiver_contact = $reqData['receiver_contact'];
        $amount = $reqData['amount'];
        $remarks = $reqData['remarks'];
        $sender_vatav_amount = $reqData['sender_vatav_amount'];
        $receiver_vatav_amount = $reqData['receiver_vatav_amount'];
        $sender_opening_balance = $reqData['sender_opening_balance'];
        $sender_closing_balance = $reqData['sender_closing_balance'];
        $receiver_opening_balance = $reqData['receiver_opening_balance'];
        $receiver_closing_balance = $reqData['receiver_closing_balance'];
        $sender_total_amount = $reqData['sender_total_amount'];
        $receiver_total_amount = $reqData['receiver_total_amount'];
        $s_adhar_number = $reqData['s_adhar_number'];
        $token = Str::random(7);

        $sender = $senderDetail->code . ' (' . $senderDetail->name . ')';
        $receiver = $receiverDetail->code . ' (' . $receiverDetail->name . ')';

        $sender_commission_party_id = $senderDetail->commission_party;
        $receiver_commission_party_id = $receiverDetail->commission_party;

        $created_at = date('Y-m-d H:i:s');

        $insertArray = array(
            'date' => $date,
            'sender' => $sender,
            'receiver' => $receiver,
            'sender_party_id' => $sender_id,
            'receiver_party_id' => $receiver_id,
            'amount' => $amount,
            'remarks' => $remarks,
            'sender_name' => $sender_name,
            'sender_contact' => $sender_contact,
            'receiver_name' => $receiver_name,
            'receiver_contact' => $receiver_contact,
            'type' => 'namu',
            's_vatav' => $sender_vatav_amount,
            'r_vatav' => $receiver_vatav_amount,
            'entered_by' => $user->company_name,
            'is_deleted' => 0,
            'o_vatav' => $our_vatav,
            'sender_commission_party_id' => $sender_commission_party_id,
            'receiver_commission_party_id' => $receiver_commission_party_id,
            'sender_total_amount' => $sender_total_amount,
            'receiver_total_amount' => $receiver_total_amount,
            'status' => 'Pending',
            'token' => $token,
            'receive_time' => $receive_time,
            's_adhar_number' => $s_adhar_number
        );
        $autoEntry = Vatav::create($insertArray);
        $id = $autoEntry->id;

        $updateArray = array(
            'serial_no' => $id,
            'sender_token' => 's' . $id,
            'receiver_token' => 'r' . $id,
        );
        $updateEntry = Vatav::where('id', $id)->update($updateArray);

        return redirect()->route('namu.add');
    }

    public function ajax_pagination(Request $request)
    {
        $reqData = $request->all();
        $data = array();

        $date = date('Y-m-d', strtotime($reqData['date']));
        // $search = $reqData['search'];

        $query = Vatav::where('type', 'namu')
            ->where('is_deleted', 0)
            ->where('date', 'LIKE', $date . '%');

        if (array_key_exists('search', $reqData) && !empty(trim($reqData['search'])) && (trim($reqData['search']) != '')) {
            $search = trim($reqData['search']);
            $query = $query->where(function ($q) use ($search) {
                $q = $q->where('sender', 'iLIKE', '%' . $search . '%')
                    ->orWhere('receiver', 'iLIKE', '%' . $search . '%')
                    ->orWhere('amount', 'LIKE', '%' . $search . '%');
            });
            $data['search'] = $search;
        } else {
            $data['search'] = '';
        }
        if (array_key_exists('sorting', $reqData) && !empty(trim($reqData['sorting'])) && (trim($reqData['sorting']) != '')) {
            $sortingArray = explode('#', $reqData['sorting']);

            $query = $query->orderBy($sortingArray[0], $sortingArray[1]);
            $data['sorting'] = $sortingArray[0] . '#' . $sortingArray[1];
        } else {
            $query = $query->orderBy('id', 'desc');
            $data['sorting'] = 'id#desc';
        }
        $startRow = $reqData['start'] ?? 1;
        $endRow = $reqData['length'] ?? 10;
        $tableTotalRows = $query->count();

        $autovatavList = $query->skip($startRow - 1)->take($endRow)->get();

        $data['startRow'] = $startRow;
        $data['endRow'] = $endRow;
        $data['tableTotalRows'] = $tableTotalRows;
        // dd($data);

        return view('pages.namu.ajax_pagination', compact('autovatavList', 'data'));
    }

    public function delete($id)
    {
        $user = auth()->user();
        $vatav = Vatav::find($id);

        if (!empty($vatav)) {
            $vatav->is_deleted = 1;
            $vatav->status = 'Deleted';
            $vatav->deleted_by = $user->company_name;
            $vatav->deleted_time = date('Y-m-d H:i:s');
            $vatav->update();
            $vatav->delete();

            $response['IsSuccess'] = true;
            $response['Message'] = 'Vatav delete successfully';
        }
        echo json_encode($response);
        exit;
    }
}
