<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Notices;
use Illuminate\Contracts\View\View;
use Illuminate\Foundation\Application;
use Illuminate\Contracts\View\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Routing\Redirector;
use Illuminate\Http\RedirectResponse;
use JetBrains\PhpStorm\NoReturn;

class NoticesController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware(function ($request, $next) {
            $permissionArray = session('permissionArray');
            if (!in_array('Notices', $permissionArray)) {
                return redirect('Admin/dashboard');
            } else{
                return $next($request);
            }
        });
    }

    public function index(): View|Application|Factory|\Illuminate\Contracts\Foundation\Application
    {
        $noticeList = Notices::orderBy('id', 'ASC')->get();
        return view('pages.notice.index', compact("noticeList"));
    }

    public function add(): View|Application|Factory|\Illuminate\Contracts\Foundation\Application
    {
        return view('pages.notice.add');
    }

    public function store(Request $request): Application|Redirector|RedirectResponse|\Illuminate\Contracts\Foundation\Application
    {
        $validator = Validator::make($request->all(), [
            'notice_value' => 'required',
        ]);
        if ($validator->fails()) {
            return redirect()->back()->withErrors($validator)->withInput();
        }

        if (isset($_POST['show_mobile']) && $_POST['show_mobile'] != '') {
            $show_mobile = $request->input('show_mobile');;
        } else {
            $show_mobile = 0;
        }

        $notice = new Notices();

        $notice->notice_value = $request->input('notice_value');
        $notice->show_mobile = $show_mobile;

        $notice->save();
        $this->_writeNoticeFile();
        return redirect('Admin/notices')->with('status', 'Notice add Successfully');

    }

    public function edit($id): View|Application|Factory|\Illuminate\Contracts\Foundation\Application
    {
        $noticeDetails = Notices::find($id);
        return view('pages.notice.edit', compact("noticeDetails"));
    }

    public function update(Request $request, $id): Application|Redirector|RedirectResponse|\Illuminate\Contracts\Foundation\Application
    {
        $validator = Validator::make($request->all(), [
            'notice_value' => 'required',
        ]);
        if ($validator->fails()) {
            return redirect()->back()->withErrors($validator)->withInput();
        }

        if (isset($_POST['show_mobile']) && $_POST['show_mobile'] != '') {
            $show_mobile = $request->input('show_mobile');;
        } else {
            $show_mobile = 0;
        }

        $notice = Notices::find($id);

        $notice->notice_value = $request->input('notice_value');
        $notice->show_mobile = $show_mobile;

        $notice->update();
        $this->_writeNoticeFile();
        return redirect('Admin/notices')->with('status', 'Notice update Successfully');

    }

    #[NoReturn] public function delete($id): void
    {
        $response = array();
        $notice = Notices::find($id);

        if (!empty($notice)) {
            $notice->delete();
            $response['IsSuccess'] = true;
            $response['Message'] = "Notice delete successfully";
        }
        $this->_writeNoticeFile();
        echo json_encode($response);
        exit;
    }

    private function _writeNoticeFile(): void
    {
        $noticeData = Notices::select('notice_value')
            ->where('is_deleted', 0)
            ->where("show_mobile", 1)->get();

        $filename = storage_path('app/public/uploads') . '/notice.json';
        $handle = fopen($filename, "w");
        $contents = fwrite($handle, json_encode($noticeData));
        fclose($handle);
    }
}
