<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Party;
use App\Models\PartyUser;
use App\Models\Company;
use App\Models\PartyBalance;
use App\Models\User;
use Illuminate\Http\Request;
use App\Http\Traits\PartyTrait;
use App\Http\Traits\DeviceTrait;

class PartyController extends Controller
{
    use PartyTrait;
    use DeviceTrait;

    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware(function ($request, $next) {
            $permissionArray = session('permissionArray');
            if (!in_array('Party', $permissionArray)) {
                return redirect('Admin/dashboard');
            } else{
                return $next($request);
            }
        });
    }

    public function index()
    {

        $allParties = Party::where('is_deleted', 0)
//                            ->whereNot('code','VAT')
            ->orderBy('id', 'DESC')->get();

        foreach ($allParties as $parties) {
            $partyUserListCount = PartyUser::where('parent_party', $parties->id)
                ->where('is_logged_in', 1)
                ->count();

            $partyLoginCount = 0;
            if ($parties->is_logged_in == 1) {
                $partyLoginCount = $partyUserListCount + 1;
            } else {
                $partyLoginCount = $partyUserListCount;
            }
            $parties->partyLoginCount = $partyLoginCount;
        }
        return view('pages.party.index', compact('allParties'));
    }

    // public function ajaxPagination()
    // {
    //     $page = request()->input('page', 1);

    //     $sorting = request()->input('sorting');
    //     if (!empty($sorting)) {
    //         $sortingArray = explode("#", $sorting);
    //         if ($sortingArray[0] == 'closing_balance') {
    //             $order = [
    //                 'pb.closing_balance' => $sortingArray[1]
    //             ];
    //         } else {
    //             $order = [
    //                 $sortingArray[0] => $sortingArray[1]
    //             ];
    //         }
    //     } else {
    //         $order = [
    //             'p.id' => 'desc'
    //         ];
    //     }

    //     $select = ['*'];
    //     $condition = ['is_deleted' => 0];

    //     $search = request()->input('search');
    //     if (!empty($search)) {
    //         $searchCondition = [
    //             ['p.name', 'like', '%' . $search . '%'],
    //             ['p.code', 'like', '%' . $search . '%'],
    //             ['p.address', 'like', '%' . $search . '%'],
    //             ['p.city', 'like', '%' . $search . '%'],
    //             'is_deleted' => 0
    //         ];
    //         $condition = $searchCondition;
    //     }

    //     $parties = Party::select($select)
    //         ->where($condition)
    //         ->orderBy(array_keys($order)[0], array_values($order)[0])
    //         ->paginate($recordPerPage);

    //     $totalRows = $parties->total();
    //     $startRow = $parties->firstItem();
    //     $endRow = $parties->lastItem();
    //     $currentPage = $parties->currentPage();
    //     $totalPages = $parties->lastPage();

    //     $serial = ($currentPage - 1) * $recordPerPage + 1;

    //     $data = [
    //         'allParties' => $parties,
    //         'tableTotalRows' => $totalRows,
    //         'startRow' => $startRow,
    //         'endRow' => $endRow,
    //         'currentPage' => $currentPage,
    //         'totalPages' => $totalPages,
    //         'search' => $search,
    //         'serial_start' => $serial
    //     ];

    //     foreach ($parties as $party) {
    //         $partyBalance = PartyBalance::where('party_id', $party->id)
    //             ->orderBy('date', 'desc')
    //             ->orderBy('id', 'desc')
    //             ->first();
    //         if ($partyBalance) {
    //             $party->closing_balance = $partyBalance->closing_balance;
    //         } else {
    //             $party->closing_balance = 0;
    //         }

    //         $partyUserListCount = PartyUser::where('parent_party', $party->id)
    //             ->where('is_logged_in', 1)
    //             ->count();

    //         $partyLoginCount = 0;
    //         if ($party->is_logged_in == 1) {
    //             $partyLoginCount = $partyUserListCount + 1;
    //         } else {
    //             $partyLoginCount = $partyUserListCount;
    //         }
    //         $party->partyLoginCount = $partyLoginCount;
    //     }

    //     return view('party.ajax_pagination', $data);
    // }


    public function add()
    {
        $allParties = Party::whereNot('code', 'VAT')->get();;
        return view('pages.party.add', compact('allParties'));
    }

    public function store(Request $request)
    {

        $reqData = $request->all();
        // dd($reqData);
        // Validate the form data
        $validatedData = $request->validate([
            'name' => 'required|string|max:255',
            'code' => 'required|string|max:255|unique:party',
            'username' => 'required|string|max:255',
            'password' => 'required|string|max:255',
        ]);
        $password = bcrypt($reqData['password']);
        $party = new Party();
        $party->name = $reqData['name'];
        $party->code = $reqData['code'];
        $party->address = $reqData['address'];
        $party->city = $reqData['city'];
        $party->partnership_firm = $reqData['is_partnership_firm'] ?? 0;
        $party->minimum = $reqData['minimum_value'] ?? 0;
        $party->commission_party = $reqData['commission_party'];
        $party->phone_no = $reqData['phone_no'];
        $party->transfer_limit = $reqData['transfer_limit'];
        $party->per_label_limit = $reqData['per_label_limit'] ?? NULL;
        $party->sender_vatav = $reqData['sender_vatav'];
        $party->receiver_vatav = $reqData['receiver_vatav'];
        $party->username = $reqData['username'];
        $party->password = $password;
        $party->is_close = $reqData['is_close'] ?? 0;
        $party->reference_name = $reqData['reference_name'];
        $party->reference_contact = $reqData['reference_contact'];
        $party->is_receiver_based = $reqData['is_receiver_based'] ?? 0;
        $party->receiver_based_commission = $reqData['receiver_based_commission'] ?? NULL;
        // $party->vatav = $validatedData['vatav'];
        // $party->parent_party = $validatedData['parent_party'];
        // dd($party);
        $party->save();

//        $userDetails = new User();
//        $userDetails->name = $reqData['name'];
//        $userDetails->email = $reqData['username'];
//        $userDetails->password = $password;
//        $userDetails->user_type = "party";
//        $userDetails->user_id = $party->id;
//        $userDetails->save();


        $companyData = Company::find(1);
        $partyIds = $companyData->parties . "," . $party->id;

        $companyData->parties = $partyIds;
        $companyData->update();

        return redirect()->route('party.index');
    }

    public function check_code_exist_or_not(Request $request)
    {
        $code = $request->input('code');
        $party_id = $request->input('party_id');
        if($party_id != ''){
            $party = Party::where('code', $code)->whereNot('id',$party_id)->withTrashed()->get();
        } else {
            $party = Party::where('code', $code)->withTrashed()->get();
        }
        

        if (!empty($party) && count($party) > 0) {
            $response['IsSuccess'] = false;
            $response['Message'] = "Code is already Exist";
        } else {
            $response['IsSuccess'] = true;
            $response['Message'] = "";
        }
        echo json_encode($response);
        exit;
    }

    public function edit($id)
    {
        $allParty = Party::find($id);
        $partyList = Party::whereNot('id', $id)->whereNot('code', 'VAT')->get();
        $partyUser = PartyUser::where('parent_party', $id)->get();
        return view('pages.party.edit', compact('allParty', 'partyList', 'partyUser'));
    }

    public function update(Request $request)
    {

        $reqData = $request->all();
        $id = $reqData['id'];
        $party = Party::find($id);
        $party->name = $reqData['name'];
        $party->code = $reqData['code'];
        $party->address = $reqData['address'];
        $party->city = $reqData['city'];
        $party->partnership_firm = $reqData['is_partnership_firm'] ?? 0;
        $party->minimum = $reqData['minimum_value'] ?? 0;
        $party->commission_party = $reqData['commission_party'];
        $party->phone_no = $reqData['phone_no'];
        $party->transfer_limit = $reqData['transfer_limit'];
        $party->per_label_limit = $reqData['per_label_limit'];
        $party->sender_vatav = $reqData['sender_vatav'];
        $party->receiver_vatav = $reqData['receiver_vatav'];
        $party->username = $reqData['username'];
        $party->password = isset($reqData['password']) ? bcrypt($reqData['password']) : $party->password;
        $party->is_close = $reqData['is_close'] ?? 0;
        $party->reference_name = $reqData['reference_name'];
        $party->reference_contact = $reqData['reference_contact'];
        $party->status = $reqData['status'];
        $party->is_receiver_based = $reqData['is_receiver_based'] ?? 0;
        $party->receiver_based_commission = $reqData['receiver_based_commission'] ?? NULL;
        $party->update();
        // dd($reqData);

//        $userDetails = User::where("user_id", $id)->where("user_type", "party")->first();
//
//        $userDetails->name = $reqData['name'];
//        $userDetails->email = $reqData['username'];
//        $userDetails->password = bcrypt($reqData['password']) ?? $userDetails->password;

//        $userDetails->update();

        return redirect()->route('party.index');
    }

    public function delete($id)
    {
        $response = array();
        $party = Party::find($id);
        if (!empty($party)) {
            $party->is_deleted = 1;
            $party->update();
            $party->delete();
            $response['IsSuccess'] = true;
            $response['Message'] = "Party delete Successfully";
        }
        echo json_encode($response);
        exit;
    }

    public function get_party_users_list(Request $request)
    {
        $reqData = $request->all();
        $parent_party = $reqData['parent_party'];

        $data = PartyUser::where('parent_party', $parent_party)->get();

        $html = "<table class='table table-striped table-bordered dataTable'>";
        $html .= "<caption><h4>Party Users List</h4></caption><thead>
                  <th>username</th>
                  <th>Permission</th>
                  <th>Action</th></thead>";

        foreach ($data as $values) {
            $html .= "<tr>";
            $html .= "<td>" . $values->username . "</td>";
            $html .= "<td>" . ($values->permissions == 0 ? 'View' : 'Add') . "</td>";
            // $html  .= "<td>".$values->city."</td>";
            // $html  .= "<td>".$values->phone_no."</td>";
            // $html  .= "<td><a class='edit_p btn btn-primary' onclick='get($values->id)' data-attr='$values->id' ><i class='fa fa-edit'></i> Edit</a></td>";
            $html .= "<td><a class='delete_p btn btn-danger' onclick=deleteRow('$values->id') ><i class='fa fa-trash'></i> Delete</a>";
            if ($values->is_logged_in == 1) {
                $html .= " <a class='logout_p btn btn-default' data-attr='$values->id' ><i class='fa fa-sign-out' aria-hidden='true'></i>Logout</a></td>";
            } else {
                $html .= "</td>";
            }
            $html .= "</tr>";
        }
        $html .= "</table>";
        // dd($html);
        return response()->json($html);
    }

    #[NoReturn] public function get_sender_balance(Request $request): void
    {
        $code = urldecode($request->input('code'));

        $partyDetails = $this->getPartyDetailsFromId($code);

        $partyClosingBalance = PartyBalance::where('party_id', $code)
            ->orderBy('date', 'DESC')
            ->orderBy('id', 'DESC')
            ->first();
        // dd($partyDetails);
        $data = [];

        if (isset($partyDetails) && !empty($partyDetails)) {
            $data['sender_vatav'] = $partyDetails->sender_vatav;
            $data['receiver_vatav'] = $partyDetails->receiver_vatav;
            $data['limit'] = $partyDetails->transfer_limit;
            $data['is_receiver_based'] = $partyDetails->is_receiver_based;
            $data['receiver_based_commission'] = $partyDetails->receiver_based_commission;
            $data['partnership_firm'] = $partyDetails->partnership_firm;
            $data['commission_party'] = $partyDetails->commission_party;
        }

        if (isset($partyClosingBalance) && !empty($partyClosingBalance)) {
            $data['opening_balance'] = number_format($partyClosingBalance->closing_balance, 2, ".", "");
        } else {
            $data['opening_balance'] = 0.00;
        }

        $response = [
            'IsSuccess' => !empty($data),
            'data' => $data,
        ];

        echo json_encode($response);
        exit;
    }

    public function logout_party(Request $request, $id)
    {
        $partyData = Party::find($id);
        $partyData->is_logged_in = 0;
        $partyData->update();

        $deviceTokenData = $this->get_party_device_token($id, 0);

        if (!empty($deviceTokenData)) {
            $deviceTokenData->delete();
        }
        return redirect('Admin/party')->with('status', 'Party update Successfully');
    }
}