<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Traits\PartyTrait;
use App\Http\Traits\UserTrait;
use Illuminate\Contracts\View\Factory;
use Illuminate\Contracts\View\View;
use Illuminate\Foundation\Application;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Routing\Redirector;
use Illuminate\Support\Facades\DB;

class ReportController extends Controller
{
    use PartyTrait;
    use UserTrait;

    public function __construct()
    {
        $this->middleware('auth');
    }

    public function general(): Factory|Application|View|Redirector|\Illuminate\Contracts\Foundation\Application|RedirectResponse
    {
        $permissionArray = session('permissionArray');
        if (!in_array('Generalreport', $permissionArray)) {
            return redirect('Admin/dashboard');
        }
        return view('pages.report.general');
    }

    public function party(): Factory|Application|View|Redirector|\Illuminate\Contracts\Foundation\Application|RedirectResponse
    {
        $permissionArray = session('permissionArray');
        if (!in_array('Partyreport', $permissionArray)) {
            return redirect('Admin/dashboard');
        }

        $partyList = $this->getPartyForReport();
        return view('pages.report.party', compact("partyList"));
    }

    public function day(): Factory|Application|View|Redirector|\Illuminate\Contracts\Foundation\Application|RedirectResponse
    {
        $permissionArray = session('permissionArray');
        if (!in_array('Dayreport', $permissionArray)) {
            return redirect('Admin/dashboard');
        }
        return view('pages.report.day');
    }

    public function dayentry(): Factory|Application|View|Redirector|\Illuminate\Contracts\Foundation\Application|RedirectResponse
    {
        $permissionArray = session('permissionArray');
        if (!in_array('Dayentryreport', $permissionArray)) {
            return redirect('Admin/dashboard');
        }
        return view('pages.report.dayentry');
    }

    public function city(): Factory|Application|View|Redirector|\Illuminate\Contracts\Foundation\Application|RedirectResponse
    {
        $permissionArray = session('permissionArray');
        if (!in_array('Cityreport', $permissionArray)) {
            return redirect('Admin/dashboard');
        }
        return view('pages.report.city');
    }

    public function vatav(): Factory|Application|View|Redirector|\Illuminate\Contracts\Foundation\Application|RedirectResponse
    {
        $permissionArray = session('permissionArray');
        if (!in_array('Vatavreport', $permissionArray)) {
            return redirect('Admin/dashboard');
        }
        return view('pages.report.vatav');
    }

    public function vatavrange(): Factory|Application|View|Redirector|\Illuminate\Contracts\Foundation\Application|RedirectResponse
    {
        $permissionArray = session('permissionArray');
        if (!in_array('Vatavreport', $permissionArray)) {
            return redirect('Admin/dashboard');
        }
        return view('pages.report.vatavrange');
    }

    public function labelcount(): Factory|Application|View|Redirector|\Illuminate\Contracts\Foundation\Application|RedirectResponse
    {
        $permissionArray = session('permissionArray');
        if (!in_array('Labelreport', $permissionArray)) {
            return redirect('Admin/dashboard');
        }
        return view('pages.report.labelcount');
    }

    public function partytoparty(): Factory|Application|View|Redirector|\Illuminate\Contracts\Foundation\Application|RedirectResponse
    {
        $permissionArray = session('permissionArray');
        if (!in_array('Partytopartyreport', $permissionArray)) {
            return redirect('Admin/dashboard');
        }

        $partyList = $this->getPartyForReport();
        return view('pages.report.partytoparty', compact("partyList"));
    }


    public function searchreport()
    {
        $permissionArray = session('permissionArray');
        if (!in_array('SearchReport', $permissionArray)) {
            return redirect('Admin/dashboard');
        }

        return view('pages.report.searchreport');
    }

    public function deletedentryreport()
    {
        $permissionArray = session('permissionArray');
        if (!in_array('DeletedEntryReport', $permissionArray)) {
            return redirect('Admin/dashboard');
        }

        return view('pages.report.deletedentry');
    }

    public function partywisecommissionreport()
    {
        $permissionArray = session('permissionArray');
        if (!in_array('PartywiseCommission', $permissionArray)) {
            return redirect('Admin/dashboard');
        }
        $partyList = $this->getPartyForReport();
        return view('pages.report.partywisecommission', compact('partyList'));
    }

    public function commissionreport()
    {
        $permissionArray = session('permissionArray');
        if (!in_array('CommissionReport', $permissionArray)) {
            return redirect('Admin/dashboard');
        }
        $partyList = $this->getPartyForReport();
        return view('pages.report.commissionreport', compact('partyList'));
    }

}
