<?php

namespace App\Http\Controllers\Admin;

use App\Models\Settings;
use Illuminate\Contracts\View\View;
use Illuminate\Foundation\Application;
use Illuminate\Contracts\View\Factory;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Routing\Redirector;
use Illuminate\Http\RedirectResponse;
use App\Http\Controllers\Controller;

class SettingsController extends Controller
{
    public function __construct()
    {
        $this->middleware('auth');
        $this->middleware(function ($request, $next) {
            $permissionArray = session('permissionArray');
            if (!in_array('Settings', $permissionArray)) {
                return redirect('Admin/dashboard');
            } else {
                return $next($request);
            }
        });
    }

    public function index(): View|Application|Factory|\Illuminate\Contracts\Foundation\Application
    {
        $settingList = Settings::orderBy('id', 'ASC')->get();
        return view('pages.setting.index', compact("settingList"));
    }

    public function edit($id): View|Application|Factory|\Illuminate\Contracts\Foundation\Application
    {
        $settingDetails = Settings::find($id);
        return view('pages.setting.edit', compact("settingDetails"));
    }

    public function update(Request $request, $id): Application|Redirector|RedirectResponse|\Illuminate\Contracts\Foundation\Application
    {
        $validator = Validator::make($request->all(), [
            'value' => 'required',
        ]);
        if ($validator->fails()) {
            return redirect()->back()->withErrors($validator)->withInput();
        }


        $setting = Settings::find($id);

        $setting->value = $request->input('value');
        $setting->update();

        return redirect('Admin/settings')->with('status', 'Setting update Successfully');

    }


}