<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Tymon\JWTAuth\Facades\JWTAuth;
use App\Http\Traits\PartyTrait;
use App\Http\Traits\VatavTrait;
use App\Http\Traits\PriceTrait;
use App\Models\LocalVatav;
use App\Models\User;
use Exception;
use Illuminate\Support\Facades\Config;

class LocalVatavController extends Controller
{
    //
    use PartyTrait;
    use VatavTrait;
    use PriceTrait;

    public function __construct()
    {
        Config::set('jwt.user', User::class);
        Config::set('auth.providers', ['users' => [
            'driver' => 'eloquent',
            'model' => User::class,
        ]]);
    }

    public function getAllLocalEntry(Request $request)
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => (object)[]], 200);
            }
            $current_date = date('Y-m-d');
            $date = $request->input('date');
            if (isset($date) && $date != "") {
                $current_date = date('Y-m-d', strtotime($request->input('date')));
            }
            $party = $request->input('party');
            $party_id = $request->input('party_id');
            $data = LocalVatav::where('sender_party_id', $party_id)
                                ->where('status', 'Pending')
                                ->where('is_deleted', 0)
//                                ->whereDate('date', $current_date)
                                ->get();

            foreach ($data as $row){
                $row->date = date('d-m-Y h:i A', strtotime($row->date));
                $row->amount = number_format($row->amount, 2, '.', '');
            }

            if (!empty($data)) {
                return response()->json(['status' => 1, 'message' => 'Data listed successfully', 'data' => $data], 200);
            } else {
                return response()->json(['status' => 0, 'message' => 'No data found', 'data' => []], 200);
            }
        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => []], 200);
        }

    }

    public function add_local_entry(Request $request)
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();

            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => (object)[]], 200);
            }

            $amount = $request->input('amount');
            $current_time = date('H:i:s');
            $start_time = "09:00:00";
            $end_time = "18:45:00";

            // if($amount < 1000){
            //     return response()->json(['status' => 0, 'message' => 'Amount should be more than 1000', 'data' => (object)[]], 200);
            // }

            if (strtotime($start_time) > strtotime($current_time)) {
                return response()->json(['status' => 0, 'message' => 'Office is not open so please add after some time', 'data' => (object)[]], 200);
            } elseif (strtotime($current_time) > strtotime($end_time)) {
                return response()->json(['status' => 0, 'message' => 'Office is closed', 'data' => (object)[]], 200);
            }

            $party_id = $request->input('party_id');

            $partyDetail = $this->getPartyDetailsFromId($party_id);

            if (!empty($partyDetail)) {
                if ($partyDetail->is_close == 1) {
                    return response()->json(['status' => 0, 'message' => 'You have not permission for Add', 'data' => (object)[]], 200);
                }
            }
            $sender = $request->input('sender');

             $entered_by = $request->input('entered_by');
            $sender_name = $request->input('sender_name');
            $sender_contact = $request->input('sender_contact');
            $receiver_name = $request->input('receiver_name');
            $receiver_contact = $request->input('receiver_contact');
            $remarks = $request->input('remarks');
            $address = $request->input('address');
            $s_adhar_number = $request->input('adhar_number');

            // return response()->json(['status' => 0, 's_adhar_number' => $s_adhar_number], 200);
            // exit;
            $token = $request->input('token');

            $sendercode = explode(' (', $sender);

            $sender_party_details = $this->getPartyDetailsFromCode($sendercode[0]);

            $sender_party_id = $sender_party_details->id;

            $finalSender = $sender_party_details->code . ' (' . $sender_party_details->name . ')';

            $serial_no = 1;
            $sender_token = "s" . $serial_no;
            $receiver_token = "r" . $serial_no;
            $receive_time = date('Y-m-d H:i:s');

            // $sender_party_closing_balance = $this->getPartyClosingBalanceFromId($sender_party_id);
            // $sender_transfer_limit = $sender_party_details->limit;
            // $sender_limit = 0;

            // if (!empty($sender_party_closing_balance)) {
            //     $sender_limit = $sender_party_closing_balance->closing_balance + $amount;
            // } else {
            //     $sender_limit = 0 + $amount;
            // }
            // $is_limit = 1;

            // $senderTransferLimit = str_replace(".","",$sender_transfer_limit);
            // $senderLimit = str_replace(".","",$sender_limit);
            // if ($senderTransferLimit != "" && $senderTransferLimit != 0) {
            //     if ((float)$senderLimit > (float)$senderTransferLimit) {
            //         $is_limit = 0;
            //     }
            // }
            $amount = number_format($amount / 100, "2", ".", "");
            // if ($is_limit == 1) {
            $insertArray = array(
                'date' => date('Y-m-d H:i:s'),
                'serial_no' => $serial_no,
                'sender' => $finalSender,
                'sender_party_id' => $sender_party_id,
                'amount' => $amount,
                'type' => 'local',
                'sender_name' => $sender_name,
                'sender_contact' => $sender_contact,
                'receiver_name' => $receiver_name,
                'receiver_contact' => $receiver_contact,
                'remarks' => $remarks,
                'token' => $token,
                'sender_token' => $sender_token,
                'receiver_token' => $receiver_token,
                'entered_by' => $sender_party_details->code,
                'status' => 'Pending',
                'is_deleted' => 0,
                'address' => $address,
                'receive_time' => $receive_time,
                'created_at' => $receive_time,
                'updated_at' => $receive_time,
                's_adhar_number' => $s_adhar_number,
            );

            $vatavId = LocalVatav::insertGetId($insertArray);

            $updateArray = array(
                'serial_no' => $vatavId,
                'sender_token' => 's' . $vatavId,
                'receiver_token' => 'r' . $vatavId,
            );
            $updateEntry = LocalVatav::where('id', $vatavId)->update($updateArray);

            return response()->json(['status' => 1, 'message' => 'Data inserted successfully', 'data' => (object)[]], 200);

            // } else{
            //     return response()->json(['status' => 0, 'message' => 'Limit exceeded', 'data' => (object)[]], 200);
            // }
        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => (object)[]], 200);
        }
    }

    public function getLocalEntry(Request $request)
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();

            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => (object)[]], 200);
            }

            $id = $request->input('id');
            $entry = LocalVatav::find($id);

            $amount = number_format($entry->amount * 100, "0", ".", "");
            $calculate_vatav_on = $this->generate_price_range($amount);

            $data['amount'] = $amount;
            $data['sender'] = $entry->sender;
            $data['sender_party_id'] = $entry->sender_party_id;
            $data['sender_name'] = $entry->sender_name;
            $data['sender_contact'] = $entry->sender_contact;
            $data['receiver_name'] = $entry->receiver_name;
            $data['receiver_contact'] = $entry->receiver_contact;
            $data['remarks'] = $entry->remarks;
            $data['token'] = $entry->token;
            $data['address'] = $entry->address;
            $data['s_adhar_number'] = $entry->s_adhar_number;
            $data['calculate_vatav_on'] = $calculate_vatav_on;

            if ($entry) {
                return response()->json(['status' => 1, 'message' => 'Data Listed successfully', 'data' => $data], 200);

            } else {
                return response()->json(['status' => 0, 'message' => 'No Data Found', 'data' => (object)[]], 200);
            }
        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => (object)[]], 200);
        }
    }

    public function deleteLocalEntry($id)
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();

            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => (object)[]], 200);
            }

            $entry = LocalVatav::find($id);

            if ($entry) {
                $entry->delete();
                return response()->json(['status' => 1, 'message' => 'Entry Deleted successfully'], 200);

            } else {
                return response()->json(['status' => 0, 'message' => 'No Data Found', 'data' => (object)[]], 200);
            }
        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => (object)[]], 200);
        }
    }
}
