<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Traits\DeviceTrait;
use App\Models\DeviceToken;
use App\Models\Party;
use App\Models\User;
use App\Models\PartyUser;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Validator;
use Tymon\JWTAuth\Exceptions\JWTException;
use Tymon\JWTAuth\Exceptions\TokenExpiredException;
use Tymon\JWTAuth\Exceptions\TokenInvalidException;
use Tymon\JWTAuth\Facades\JWTAuth;

class LoginController extends Controller
{
    use DeviceTrait;

    public function __construct()
    {
        Config::set('jwt.user', User::class);
        Config::set('auth.providers', ['users' => [
            'driver' => 'eloquent',
            'model' => User::class,
        ]]);
    }

    public function getAuthenticatedUser(): JsonResponse
    {
        try {
            if (!$user = JWTAuth::parseToken()->authenticate()) {
                return response()->json(['user_not_found'], 404);
            }
        } catch (TokenExpiredException $e) {
            return response()->json(['token_expired'], $e->getStatusCode());
        } catch (TokenInvalidException $e) {
            return response()->json(['token_invalid'], $e->getStatusCode());
        } catch (JWTException $e) {
            return response()->json(['token_absent'], $e->getStatusCode());
        }
        return response()->json(compact('user'));
    }

    public function check_user_exit(Request $request)
    {
        $validation_array = [
            'username' => 'required',
//            'device_token' => 'required',
            'password' => 'required'
        ];
        $validation = Validator::make($request->all(), $validation_array);
        if ($validation->fails()) {
            return response()->json(['status' => 0, 'message' => $validation->messages()->first(), 'data' => (object)[]], 200);
        }
        try {
            $userName = $request->input('username');
            $password = $request->input('password');
            $device_token = $request->input('device_token');
            $credentials = ['email' => $userName, 'password' => $password, 'user_type' => 'party'];
            $data = array();

            if (!$token = JWTAuth::attempt($credentials)) {
                $userCredentials = ['email' => $userName, 'password' => $password, 'user_type' => 'user'];

                if (!$userToken = JWTAuth::attempt($userCredentials)) {
                    return response()->json(['status' => 0, 'message' => 'Invalid Credentials, Please try again', 'data' => (object)[]], 200);
                }
                $user = auth()->user();
                $userDetail = PartyUser::find($user->user_id);
                if (!empty($userDetail)) {
                    if ($userDetail->is_logged_in == 1) {
                        return response()->json(['status' => 0, 'message' => "You can't login because yor are logged in another device !!", 'data' => (object)[]], 200);
                    }

                    $deviceToken = $this->get_party_device_token($userDetail->parent_party, $userDetail->id);

                    if (empty($deviceToken)) {
                        $deviceTokenData = new DeviceToken();
                        $deviceTokenData->party_id = $userDetail->parent_party;
                        $deviceTokenData->device_token = $device_token;
                        $deviceTokenData->user_id = $userDetail->id;
                        $deviceTokenData->save();
                    } else {
                        $deviceToken->device_token = $device_token;
                        $deviceToken->update();
                    }

                    $partyDetails = Party::find($userDetail->parent_party);

                    $data['token'] = $userToken;
                    $data['party_id'] = $partyDetails->id;
                    $data['companyNameCode'] = $partyDetails->code . ' (' . $partyDetails->name . ')';
                    $data['companyName'] = $partyDetails->name;
                    $data['permission'] = $userDetail->permissions;
                    $data['companyCode'] = $partyDetails->code;
                    $data['username'] = $partyDetails->username;
                    $data['login_username'] = $userDetail->username;
                    $data['is_logged_in'] = 0;
                    $data['is_sub_user'] = true;
                    $data['sub_party_id'] = $userDetail->id;

                    $userDetail->is_logged_in = 1;
                    $userDetail->update();
                    return response()->json(['status' => 1, 'message' => "You are logged in successfully..!!", 'data' => $data], 200);

                } else {
                    return response()->json(['status' => 0, 'message' => 'Invalid Credentials, Please try again', 'data' => (object)[]], 200);
                }
            }
            $partyDetail = Party::find(auth()->user()->user_id);

            if (!empty($partyDetail)) {
                if ($partyDetail->status == 'inactive') {
                    return response()->json(['status' => 0, 'message' => 'You can not login because you are Inactive..!!', 'data' => (object)[]], 200);
                }
                if ($partyDetail->is_logged_in == 1) {
                    return response()->json(['status' => 0, 'message' => "You can't login because yor are logged in another device !!", 'data' => (object)[]], 200);
                }

                $deviceToken = $this->get_party_device_token($partyDetail->id, 0);

                if (empty($deviceToken)) {
                    $deviceTokenData = new DeviceToken();
                    $deviceTokenData->party_id = $partyDetail->id;
                    $deviceTokenData->device_token = $device_token;
                    $deviceTokenData->user_id = 0;
                    $deviceTokenData->save();
                } else {
                    $deviceToken->device_token = $device_token;
                    $deviceToken->update();
                }

                $data['token'] = $token;
                $data['party_id'] = $partyDetail->id;
                $data['companyNameCode'] = $partyDetail->code . ' (' . $partyDetail->name . ')';
                $data['companyName'] = $partyDetail->name;
                $data['permission'] = '';
                $data['companyCode'] = $partyDetail->code;
                $data['username'] = $partyDetail->username;
                $data['login_username'] = $partyDetail->username;
                $data['is_logged_in'] = 0;
                $data['is_sub_user'] = false;
                $data['sub_party_id'] = 0;

                $partyDetail->is_logged_in = 1;
                $partyDetail->update();

                return response()->json(['status' => 1, 'message' => "You are logged in successfully..!!", 'data' => $data], 200);

            } else {
                return response()->json(['status' => 0, 'message' => 'Invalid Credentials, Please try again', 'data' => (object)[]], 200);
            }


        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => (object)[]], 200);
        }
    }


    public function party_logout(Request $request)
    {
        $party_id = $request->input('party_id');
//        $device_token = $request->input('device_token');
        $sub_party_id = $request->input('sub_party_id');

        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...'], 200);
            }
            $deviceToken = $this->get_party_device_token($party_id, $sub_party_id);
            if (!empty($deviceToken)) {
                $deviceToken->delete();
            }
            if ($sub_party_id == 0) {
                $partyData = Party::find($party_id);
                $partyData->is_logged_in = 0;
                $partyData->update();
                JWTAuth::invalidate($request->token);
            } else {
                $partyUserData = PartyUser::find($sub_party_id);
                $partyUserData->is_logged_in = 0;
                $partyUserData->update();
                JWTAuth::invalidate($request->token);
            }
            return response()->json(['status' => 1, 'message' => 'User logged out Successfully..!', 'data' => (object)[]]);

        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => (object)[]], 200);
        }
    }

    public function check_active_inactive_user(Request $request)
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...'], 200);
            }
            $party_id = $request->input('party_id');
            $sub_party_id = $request->input('sub_party_id');
            $partyData = Party::find($party_id);

            if ($partyData->status == 'inactive') {
                $deviceToken = $this->get_party_device_token($party_id, $sub_party_id);
                if (!empty($deviceToken)) {
                    $deviceToken->delete();
                }
            }
            if (!empty($partyData)) {
                return response()->json(['status' => 1, 'message' => 'Data Listed successfully', 'data' => $partyData], 200);
            } else {
                return response()->json(['status' => 0, 'message' => 'No Data available', 'data' => (object)[]]);
            }
        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => (object)[]], 200);
        }
    }

    public function checkUserLoginOrNotLogin(Request $request)
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...'], 200);
            }
            $party_id = $request->input('party_id');
            $sub_party_id = $request->input('sub_party_id');

            $partyData = array();

            if ($sub_party_id != 0){
                $partyData = PartyUser::find($sub_party_id);
            } else {
                $partyData = Party::find($party_id);
            }
            if (!empty($partyData)) {
                return response()->json(['status' => 1, 'message' => 'Data Listed Successfully', 'data' => $partyData], 200);
            } else{
                return response()->json(['status' => 0, 'message' => 'No Data available', 'data' => (object)[]]);
            }

        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => (object)[]], 200);
        }
    }


}