<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Models\Party;
use App\Models\User;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Config;
use Tymon\JWTAuth\Facades\JWTAuth;

class PartyController extends Controller
{
    public function __construct()
    {
        Config::set('jwt.user', User::class);
        Config::set('auth.providers', ['users' => [
            'driver' => 'eloquent',
            'model' => User::class,
        ]]);
    }

    public function get_party(Request $request): JsonResponse
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => (object)[]], 200);
            }
            $party_id = $request->input('party_id');
            $filename = storage_path('app/public/uploads') . '/party.json';
            $handle = fopen($filename, "rb");
            $contents = fread($handle, filesize($filename));
            fclose($handle);
            $partyData = json_decode($contents);

            $data = array();
            foreach ($partyData as $row) {
                if ($row->party_id != $party_id) {
                    $data[] = array(
                        'id' => $row->id,
                        'name' => $row->name,
                        'city' => $row->city,
                        'party_id' => $row->party_id,
                    );
                }
            }
            if (!empty($data)) {
                return response()->json(['status' => 1, 'message' => 'Data listed successfully', 'data' => $data], 200);
            } else {
                return response()->json(['status' => 0, 'message' => 'No data found', 'data' => []], 200);
            }

        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => (object)[]], 200);
        }
    }

    public function get_party_profile(Request $request): JsonResponse
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => (object)[]], 200);
            }
            $party_id = $request->input('party_id');
            $party = Party::find($party_id);
            if (!empty($party)) {
                return response()->json(['status' => 1, 'message' => 'Data listed successfully', 'data' => $party], 200);
            } else {
                return response()->json(['status' => 0, 'message' => 'No data found', 'data' => []], 200);
            }
        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => (object)[]], 200);
        }
    }
}