<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Traits\DeviceTrait;
use App\Http\Traits\NotificationTrait;
use App\Http\Traits\PartyTrait;
use App\Http\Traits\VatavTrait;
use App\Models\LocalVatav;
use App\Models\User;
use App\Models\Vatav;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\DB;
use Tymon\JWTAuth\Facades\JWTAuth;
use Mpdf\Mpdf as PDF;

class ReportController extends Controller
{
    use DeviceTrait;
    use NotificationTrait;
    use PartyTrait;
    use VatavTrait;

    public function __construct()
    {
        Config::set('jwt.user', User::class);
        Config::set('auth.providers', ['users' => [
            'driver' => 'eloquent',
            'model' => User::class,
        ]]);
    }

    public function changeVatavStatus(Request $request): JsonResponse
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => (object)[]], 200);
            }
            $id = $request->input('vatav_id');
            $status = $request->input('status');

            $vatavData = Vatav::find($id);

            $vatavData->status = $status;
            $vatavData->update();

            if ($status == 'Paid') {
                $sender_party_id = $vatavData->sender_party_id;
                $receiver = $vatavData->receiver;
                // $amt = number_format($vatavData->amount / 100, "2", ".", "");
                $amt = number_format($vatavData->amount * 100);

                $device_token = $this->getAllPartyDevices($sender_party_id);


                if (!empty($device_token)) {
                    $notification_arr['title'] = 'Entry of amount ' . $amt . ' Paid By ' . $receiver;
                    $notification_arr['msg'] = 'Entry of amount ' . $amt . ' Paid By ' . $receiver;
                    $notification_arr['body'] = 'Entry of amount ' . $amt . ' Paid By ' . $receiver;
                    $notification_arr['type'] = "status_change";
                    $this->android_send_push_notification($notification_arr, $device_token);
                }
            }
            return response()->json(['status' => 1, 'message' => 'Data updated successfully', 'data' => (object)[]], 200);


        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => (object)[]], 200);
        }
    }

    public function receiverPendingVatavCount(Request $request): JsonResponse
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => (object)[]], 200);
            }
            $party_id = $request->input('party_id');

            $entryLists = Vatav::where('is_deleted', 0)
                ->where('receiver_party_id', $party_id)
                ->where('status', 'Pending')
                ->where('ho_approve', 1)
                ->where('date', 'LIKE', date('Y-m-d') . '%')
                ->get();
            if (!empty($entryLists)) {
                return response()->json(['status' => 1, 'message' => 'Data listed successfully', 'data' => $entryLists], 200);
            } else {
                return response()->json(['status' => 0, 'message' => 'Data not found', 'data' => []], 200);
            }
        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => []], 200);
        }
    }

    public function getAllSentEntry(Request $request): JsonResponse
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => []], 200);
            }
            $current_date = date('Y-m-d');
            $start_date = date('Y-m-d', strtotime('-30 days'));
            $end_date = date('Y-m-d');
            $date = $request->input('date');
            if (isset($date) && $date != "") {
                $current_date = date('Y-m-d', strtotime($request->input('date')));

                $start_date = date('Y-m-d', strtotime('-30 days', strtotime($date)));
                $end_date = date('Y-m-d', strtotime($date));
            }

            $party = $request->input('party');
            $party_id = $request->input('party_id');

            $allPartyList = $this->getAllPartyWithCode();


            $allParties = DB::table('party as p')
                ->join('vatav as v', 'v.sender_party_id', '=', 'p.id')
                ->leftJoin('party_balance as pb', function ($join) {
                    $join->on('pb.party_id', '=', 'p.id')
                        ->on('pb.vatav_id', '=', 'v.id');
                })
                ->select('v.token', 'v.label', 'v.ho_approve', 'v.type', 'pb.opening_balance', 'v.sender_token', 'v.receiver_token', 'v.id', DB::raw("DATE_FORMAT(v.date, '%d-%m-%y %h:%i %p') AS date"), DB::raw("DATE_FORMAT(v.receive_time, '%d-%m-%y %h:%i %p') AS receive_time"), 'v.sender_name', 'v.sender_contact', 'v.receiver_name', 'v.receiver_contact', 'v.sender as sender', 'v.receiver as opp', 'v.r_vatav', 'v.s_vatav', 'v.amount', 'v.sender_total_amount', 'v.receiver_total_amount', 'v.remarks', 'v.serial_no', 'pb.closing_balance', 'v.status', 'v.entered_by', 'v.sender_party_id', 'v.receiver_party_id')
                ->where('p.id', $party_id)
                ->whereBetween(DB::raw('DATE(v.date)'), [$start_date, $end_date])
                ->where('v.is_deleted', 0)
                ->where('v.ho_approve', 1)
                ->where(function ($query) {
                    $query->where('v.status', 'Pending')
                        ->orWhere('v.status', 'Received')
                        ->orWhere('v.status', 'Confirm');
                })
                ->orWhere(function ($query) use ($party_id, $current_date, $start_date, $end_date) {
                    $query->where('v.sender_party_id', $party_id)
                        ->where('v.status', 'Paid')
                        ->whereBetween(DB::raw('DATE(v.date)'), [$start_date, $end_date]);
                })
                ->union(DB::table('party as pr')
                    ->join('vatavBackUp as vb', 'vb.sender_party_id', '=', 'pr.id')
                    ->leftJoin('party_balance as pbb', function ($join) {
                        $join->on('pbb.party_id', '=', 'pr.id')
                            ->on('pbb.vatav_id', '=', 'vb.id');
                    })
                    ->select('vb.token', 'vb.label', 'vb.ho_approve', 'vb.type', 'pbb.opening_balance', 'vb.sender_token', 'vb.receiver_token', 'vb.id', DB::raw("DATE_FORMAT(vb.date, '%d-%m-%y %h:%i %p') AS date"), DB::raw("DATE_FORMAT(vb.receive_time, '%d-%m-%y %h:%i %p') AS receive_time"), 'vb.sender_name', 'vb.sender_contact', 'vb.receiver_name', 'vb.receiver_contact', 'vb.sender as sender', 'vb.receiver as opp', 'vb.r_vatav', 'vb.s_vatav', 'vb.amount', 'vb.sender_total_amount', 'vb.receiver_total_amount', 'vb.remarks', 'vb.serial_no', 'pbb.closing_balance', 'vb.status', 'vb.entered_by', 'vb.sender_party_id', 'vb.receiver_party_id')
                    ->where('pr.id', $party_id)
                    ->whereBetween('vb.date', [$start_date, $end_date])
                    ->where('vb.is_deleted', 0)
                    ->where('vb.ho_approve', 1)
                    ->where(function ($query) {
                        $query->where('vb.status', 'Pending')
                            ->orWhere('vb.status', 'Received')
                            ->orWhere('vb.status', 'Confirm');
                    })
                    ->orWhere(function ($query) use ($party_id, $current_date, $start_date, $end_date) {
                        $query->where('vb.sender_party_id', $party_id)
                            ->where('vb.status', 'Paid')
                            ->whereBetween('vb.date', [$start_date, $end_date]);
                    }))
                ->orderBy('serial_no', 'DESC')
                ->get();

            foreach ($allParties as $_allParties) {
                $name = explode(" (", $_allParties->opp);
                $opp_name = explode(")", $name[1]);

                $sname = explode(" (", $_allParties->sender);
                $sopp_name = explode(")", $sname[1]);

                if ($allPartyList[$_allParties->receiver_party_id]->city == 'AH RPO') {
                    $opp_name[0] = 'RATANPOL VIA';
                }
                if ($allPartyList[$_allParties->receiver_party_id]->city == 'KLM') {
                    // $opp_name[0] = 'KALUPUR MESSAGE';
                    $opp_name[0] = 'RATANPOL VIA';
                }

                if ($allPartyList[$_allParties->sender_party_id]->city == 'AH RPO') {
                    $sopp_name[0] = 'KALUPUR MESSAGE';
                }
                if ($allPartyList[$_allParties->sender_party_id]->city == 'KLM') {
                    $sopp_name[0] = 'KALUPUR MESSAGE';
                }
                $_allParties->amount = number_format($_allParties->amount, 2, '.', '');
                $_allParties->opp = $opp_name[0];
                $_allParties->sender = $sopp_name[0];
                $_allParties->receiver_party_mobile_number = $allPartyList[$_allParties->receiver_party_id]->phone_no;
                $_allParties->sender_party_mobile_number = $allPartyList[$_allParties->sender_party_id]->phone_no;
            }

            if (!empty($allParties)) {
                return response()->json(['status' => 1, 'message' => 'Data listed successfully', 'data' => $allParties], 200);
            } else {
                return response()->json(['status' => 0, 'message' => 'No data found', 'data' => []], 200);
            }

        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => []], 200);
        }
    }

    public function getAllReceivedEntry(Request $request): JsonResponse
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => []], 200);
            }

            $current_date = date('Y-m-d');
            $date = $request->input('date');
            if (isset($date) && $date != "") {
                $current_date = date('Y-m-d', strtotime($request->input('date')));
            }
            $party = $request->input('party');
            $party_id = $request->input('party_id');
            $allPartyList = $this->getAllPartyWithCode();

            $vatavBackUpQuery = DB::table('party as p')->select('v.token', 'v.label', 'v.ho_approve', 'v.type', 'pb.opening_balance', 'v.sender_token', 'v.receiver_token', 'v.id', DB::raw("DATE_FORMAT(v.date, '%d-%m-%y %h:%i %p') AS send_date"), DB::raw("DATE_FORMAT(v.receive_time, '%d-%m-%y %h:%i %p') AS date"), 'v.sender_name', 'v.sender_contact', 'v.receiver_name', 'v.receiver_contact', 'v.sender as sender', 'v.receiver as opp', 'v.r_vatav', 'v.s_vatav', 'v.amount', 'v.sender_total_amount', 'v.receiver_total_amount', 'v.remarks', 'v.serial_no', 'pb.closing_balance', 'v.status', 'v.entered_by', 'v.receiver_party_id', 'v.sender_party_id')
                ->join('vatavBackUp as v', 'v.receiver_party_id', '=', 'p.id')
                ->leftJoin('party_balance as pb', function ($join) {
                    $join->on('pb.party_id', '=', 'p.id')->on('pb.vatav_id', '=', 'v.id');
                })
                ->where('p.id', '=', $party_id)
                ->where('v.date', 'LIKE', $current_date . '%')
                ->where('v.is_deleted', 0)
                ->where('v.ho_approve', 1)
                ->orderBy('v.serial_no', 'DESC');
            $vatavQuery = DB::table('party as p')->select('v.token', 'v.label', 'v.ho_approve', 'v.type', 'pb.opening_balance', 'v.sender_token', 'v.receiver_token', 'v.id', DB::raw("DATE_FORMAT(v.date, '%d-%m-%y %h:%i %p') AS send_date"), DB::raw("DATE_FORMAT(v.receive_time, '%d-%m-%y %h:%i %p') AS date"), 'v.sender_name', 'v.sender_contact', 'v.receiver_name', 'v.receiver_contact', 'v.sender as sender', 'v.receiver as opp', 'v.r_vatav', 'v.s_vatav', 'v.amount', 'v.sender_total_amount', 'v.receiver_total_amount', 'v.remarks', 'v.serial_no', 'pb.closing_balance', 'v.status', 'v.entered_by', 'v.receiver_party_id', 'v.sender_party_id')
                ->join('vatav as v', 'v.receiver_party_id', '=', 'p.id')
                ->leftJoin('party_balance as pb', function ($join) {
                    $join->on('pb.party_id', '=', 'p.id')
                        ->on('pb.vatav_id', '=', 'v.id');
                })
                ->where('p.id', '=', $party_id)
                ->where('v.date', 'LIKE', $current_date . '%')
                ->where('v.is_deleted', 0)
                ->where('v.ho_approve', 1)
                ->orderBy('v.serial_no', 'DESC');
            $allParties = $vatavBackUpQuery->union($vatavQuery)->get();
            foreach ($allParties as $_allParties) {
                $name = explode(" (", $_allParties->opp);
                $opp_name = explode(")", $name[1]);

                $sname = explode(" (", $_allParties->sender);
                $sopp_name = explode(")", $sname[1]);

                if ($allPartyList[$_allParties->receiver_party_id]->city == 'AH RPO') {
                    $opp_name[0] = 'RATANPOL VIA';
                }
                if ($allPartyList[$_allParties->receiver_party_id]->city == 'KLM') {
                    $opp_name[0] = 'RATANPOL VIA';
                }

                if ($allPartyList[$_allParties->sender_party_id]->city == 'AH RPO') {
                    $sopp_name[0] = 'KALUPUR MESSAGE';
                }
                if ($allPartyList[$_allParties->sender_party_id]->city == 'KLM') {
                    $sopp_name[0] = 'KALUPUR MESSAGE';
                }
                $_allParties->amount = number_format($_allParties->amount, 2, '.', '');
                $_allParties->opp = $opp_name[0];
                $_allParties->sender = $sopp_name[0];
                $_allParties->receiver_party_mobile_number = $allPartyList[$_allParties->receiver_party_id]->phone_no;
                $_allParties->sender_party_mobile_number = $allPartyList[$_allParties->sender_party_id]->phone_no;
            }

            if (!empty($allParties)) {
                return response()->json(['status' => 1, 'message' => 'Data listed successfully', 'data' => $allParties], 200);
            } else {
                return response()->json(['status' => 0, 'message' => 'No data found', 'data' => []], 200);
            }

        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => []], 200);
        }
    }

    public function generate_report(Request $request): JsonResponse
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => (object)[]], 200);
            }
            $party = $request->input('party');
            $partyId = $request->input('party_id');
            $date = $request->input('date');

            $newDate = date('Y-m-d', strtotime($date));
            if (isset($date) && $date != "") {

                $start_date = date('Y-m-d', strtotime('-30 days', strtotime($date)));
                $end_date = date('Y-m-d', strtotime($date));
            } else {
                $start_date = date('Y-m-d', strtotime('-30 days', strtotime(date('Y-m-d'))));
                $end_date = date('Y-m-d');
            }
            $data['pendingVatavList'] = $this->getSenderVatavListWithStatus($partyId, $start_date, $end_date, 'Pending');
            $data['paidVatavList'] = $this->getSenderVatavListWithStatus($partyId, $start_date, $end_date, 'Paid');
            $data['receivedVatavList'] = $this->getSenderVatavListWithStatus($partyId, $start_date, $end_date, 'Received');
            $data['confirmVatavList'] = $this->getSenderVatavListWithStatus($partyId, $start_date, $end_date, 'Confirm');

            $html = view('pages.app.sentpartyreport', compact('data'))->render();
            $pdfFilePath = 'sent_party_report_' . date('Y_m_d_H_i_s') . '.pdf';

            ob_end_clean();
            $document = new PDF([
                'mode' => 'utf-8',
                'format' => 'A4',
                'margin_header' => '3',
                'margin_top' => '20',
                'margin_bottom' => '20',
                'margin_footer' => '2',
            ]);

            // Set some header information for output
            $header = [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' => 'inline; filename="' . $pdfFilePath . '"'
            ];
            $uploadPath = storage_path('app/public/uploads/');
            $html = mb_convert_encoding($html, 'UTF-8', 'UTF-8');
            $document->WriteHTML($html);
            $document->Output($uploadPath . $pdfFilePath, 'F');
            $dataArray = array();
            $dataArray['filename'] = $pdfFilePath;
            return response()->json(['status' => 1, 'message' => 'Report Generated successfully', 'data' => $dataArray], 200);

        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => (object)[]], 200);
        }
    }

    public function generate_received_report(Request $request): JsonResponse
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => (object)[]], 200);
            }

        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => (object)[]], 200);
        }

        $party = $request->input('party');
        $partyId = $request->input('party_id');
        $date = $request->input('date');

        $newDate = date('Y-m-d', strtotime($date));
        $data['pendingVatavList'] = $this->getReceiverVatavListWithStatus($partyId, $newDate, 'Pending');
        $data['paidVatavList'] = $this->getReceiverVatavListWithStatus($partyId, $newDate, 'Paid');
        $data['receivedVatavList'] = $this->getReceiverVatavListWithStatus($partyId, $newDate, 'Received');
        $data['confirmVatavList'] = $this->getReceiverVatavListWithStatus($partyId, $newDate, 'Confirm');

        $html = view('pages.app.receivedpartyreport', compact('data'))->render();
        $pdfFilePath = 'received_party_report_' . date('Y_m_d_H_i_s') . '.pdf';

        ob_end_clean();
        $document = new PDF([
            'mode' => 'utf-8',
            'format' => 'A4',
            'margin_header' => '3',
            'margin_top' => '20',
            'margin_bottom' => '20',
            'margin_footer' => '2',
        ]);

        // Set some header information for output
        $header = [
            'Content-Type' => 'application/pdf',
            'Content-Disposition' => 'inline; filename="' . $pdfFilePath . '"'
        ];
        $uploadPath = storage_path('app/public/uploads/');
        $html = mb_convert_encoding($html, 'UTF-8', 'UTF-8');
        $document->WriteHTML($html);
        $document->Output($uploadPath . $pdfFilePath, 'F');
        $dataArray = array();
        $dataArray['filename'] = $pdfFilePath;
        return response()->json(['status' => 1, 'message' => 'Report Generated successfully', 'data' => $dataArray], 200);
    }

    public function downloadGeneralReport(Request $request): JsonResponse
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => (object)[]], 200);
            }
            $party_id = $request->input('party_id');
            $party = $request->input('partyNameCode');
            $date = $request->input('date_range');
            $print = $request->input('print');
            $downloaded_by = $request->input('downloaded_by');

            $allparyASSCArr = $this->getAllPartyWithCode();

            $date_range = explode(" to ", $date);

            $start_date = date('Y-m-d', strtotime($date_range[0]));
            $end_date = date('Y-m-d', strtotime($date_range[1]));


            $allParties = DB::table('vatav as v')->select('v.sender_is_partnership_firm', 'v.receiver_is_partnership_firm', 'v.sender_commission_party_amount', 'v.receiver_commission_party_amount', 'v.total_vatav_label', 'v.type', 'v.sender_token', 'v.receiver_token', 'v.entered_by', 'v.id', 'v.sender as sender', 'v.receiver as opp', 'v.r_vatav', 'v.s_vatav', 'v.amount', 'v.date', 'v.sender_total_amount', 'v.receiver_total_amount', 'v.remarks', 'v.serial_no', 'v.type', 'v.sender_party_id', 'v.receiver_party_id')
                ->where(function ($query) use ($party_id) {
                    $query->where('v.sender_party_id', $party_id)
                        ->orWhere('v.receiver_party_id', $party_id);
                })
                ->whereDate('v.date', '>=', $start_date)
                ->whereDate('v.date', '<=', $end_date)
                ->where('v.is_deleted', 0)->where('v.ho_approve', 1)
                ->union(DB::table('vatavBackUp as vb')
                    ->select('vb.sender_is_partnership_firm', 'vb.receiver_is_partnership_firm', 'vb.sender_commission_party_amount', 'vb.receiver_commission_party_amount', 'vb.total_vatav_label', 'vb.type', 'vb.sender_token', 'vb.receiver_token', 'vb.entered_by', 'vb.id', 'vb.sender as sender', 'vb.receiver as opp', 'vb.r_vatav', 'vb.s_vatav', 'vb.amount', 'vb.date', 'vb.sender_total_amount', 'vb.receiver_total_amount', 'vb.remarks', 'vb.serial_no', 'vb.type', 'vb.sender_party_id', 'vb.receiver_party_id')
                    ->where(function ($query) use ($party_id) {
                        $query->where('vb.sender_party_id', $party_id)
                            ->orWhere('vb.receiver_party_id', $party_id);
                    })
//                    ->orWhere('vb.receiver_party_id', $party_id)
                    ->whereDate('vb.date', '>=', $start_date)
                    ->whereDate('vb.date', '<=', $end_date)
                    ->where('vb.is_deleted', 0)
                    ->where('vb.ho_approve', 1))
                ->orderBy('date')
                ->orderBy('serial_no')
                ->get();
            $ceCount = 0;
            $deCount = 0;
            $credit_sum = 0;
            $debit_sum = 0;
            $data['opening_balance'] = $this->getPartyOpeningBalance($party_id, $start_date);
            $current_balance_vat = $data['opening_balance'];
            $total_commission = 0;
            $final_arr = array();

            foreach ($allParties as $_allParties) {
                $n_arr = array();
                $vat = false;
                $com = false;
                $s_code = explode(" (", $_allParties->sender);
                $sender_name = explode(")", $s_code[1]);
                $r_code = explode(" (", $_allParties->opp);
                $receiver_name = explode(")", $r_code[1]);

                if ($_allParties->sender_party_id == $party_id) {
                    $n_arr['sender_party'] = $sender_name[0];

                    if ($allparyASSCArr[$_allParties->receiver_party_id]->city == "AH RPO") {
                        $n_arr['opposite_party'] = "RATANPOL VIA";
                    } elseif ($allparyASSCArr[$_allParties->receiver_party_id]->city == "KLM") {
                        $n_arr['opposite_party'] = "RATANPOL VIA";
                    } else {
                        $n_arr['opposite_party'] = $receiver_name[0];
                    }

                    if ($_allParties->type == "namu") {
                        $n_arr['debit'] = $_allParties->sender_total_amount;
                        $n_arr['credit'] = 0.00;
                        $deCount++;
                    } else {
                        $n_arr['debit'] = 0.00;
                        $n_arr['credit'] = $_allParties->sender_total_amount;
                        $ceCount++;
                    }
                    if ($s_code[0] == "VAT") {
                        $vat = true;
                        $current_balance_vat += $n_arr['credit'];
                    }
                }

                if ($_allParties->receiver_party_id == $party_id) {
                    $n_arr['sender_party'] = $receiver_name[0];

                    if ($allparyASSCArr[$_allParties->sender_party_id]->city == "AH RPO") {
                        $n_arr['opposite_party'] = "RATANPOL VIA";
                    } elseif ($allparyASSCArr[$_allParties->sender_party_id]->city == "KLM") {
                        $n_arr['opposite_party'] = "RATANPOL VIA";
                    } else {
                        $n_arr['opposite_party'] = $sender_name[0];
                    }
                    $n_arr['credit'] = 0.00;
                    $n_arr['debit'] = $_allParties->receiver_total_amount;

                    if ($_allParties->type == "namu") {
                        $n_arr['credit'] = $_allParties->receiver_total_amount;
                        $n_arr['debit'] = 0.00;
                        $ceCount++;
                    } else {
                        $n_arr['credit'] = 0.00;
                        $n_arr['debit'] = $_allParties->receiver_total_amount;
                        $deCount++;
                    }

                    $r_code = explode(" (", $_allParties->opp);
                    if ($r_code[0] == "COM") {
                        $com = true;
                    }
                    if ($r_code[0] == "VAT") {
                        $vat = true;
                        $current_balance_vat += $n_arr['debit'];
                    }
                }

                if ($_allParties->total_vatav_label == 0) {
                    $n_arr['commission'] = 0.00;
                } else {
                    if ($_allParties->sender_party_id == $party_id) {
                        if ($_allParties->sender_is_partnership_firm == 1) {
                            $n_arr['commission'] = number_format($_allParties->sender_commission_party_amount, "2", ".", "");
                        } else {
                            $n_arr['commission'] = number_format($_allParties->total_vatav_label - $_allParties->s_vatav, "2", ".", "");
                        }
                    }
                    if ($_allParties->receiver_party_id == $party_id) {
                        if ($_allParties->receiver_is_partnership_firm == 1) {
                            $n_arr['commission'] = number_format($_allParties->receiver_commission_party_amount, "2", ".", "");
                        } else {
                            $n_arr['commission'] = number_format($_allParties->r_vatav, "2", ".", "");
                        }
                    }
                }
                $total_commission += $n_arr['commission'];
                $n_arr['entered_by'] = $_allParties->entered_by;
                $n_arr['remarks'] = $_allParties->remarks;
                $n_arr['sender_token'] = $_allParties->sender_token;
                $n_arr['receiver_token'] = $_allParties->receiver_token;

                $n_arr['date'] = date('d-m-Y', strtotime($_allParties->date));
                $n_arr['time'] = date('h:i:s', strtotime($_allParties->date));
                $n_arr['serial_no'] = $_allParties->serial_no;
                $n_arr['type'] = $_allParties->type;
//                $n_arr['current_balance'] = number_format($_allParties->closing_balance, "2", ".", "");
//                if ($vat == true) {
//                    $n_arr['current_balance'] = number_format($current_balance_vat, "2", ".", "");
//                }
//                if ($com == true) {
//                    $n_arr['current_balance'] = number_format($_allParties->closing_balance, "2", ".", "");
//                }
//                $n_arr['opening_balance'] = number_format($_allParties->opening_balance, "2", ".", "");
                $n_arr['id'] = $_allParties->id;

                $credit_sum += $n_arr['credit'];
                $debit_sum += $n_arr['debit'];

                $final_arr[] = $n_arr;
            }
            $total_de_sum = $debit_sum;
            $total_cr_sum = $credit_sum;
            if ($data['opening_balance'] > 0) {
                $total_cr_sum += $data['opening_balance'];
            } else {
                $total_de_sum += $data['opening_balance'];
            }
            $data['credit_sum'] = number_format($total_cr_sum, "2", ".", "");
            $data['debit_sum'] = number_format($total_de_sum, "2", ".", "");
            $data['allParties'] = $final_arr;
            $data['total_commission'] = number_format($total_commission, 2, ".", "");

            if ($print == 1) {
                $input_parmeter = array(
                    'downloaded_by' => $downloaded_by,
                    'party_name' => $party,
                    'date' => $date
                );
                $data['inputs'] = $input_parmeter;
                $html = view('pages.app.partyreport', compact('data'))->render();
                $pdfFilePath = "General_report_" . date('Y_m_d_H_i_s') . ".pdf";

                ob_end_clean();
                $document = new PDF([
                    'mode' => 'utf-8',
                    'format' => 'A4',
                    'margin_header' => '3',
                    'margin_top' => '20',
                    'margin_bottom' => '20',
                    'margin_footer' => '2',
                ]);

                // Set some header information for output
                $header = [
                    'Content-Type' => 'application/pdf',
                    'Content-Disposition' => 'inline; filename="' . $pdfFilePath . '"'
                ];
                $uploadPath = storage_path('app/public/uploads/');
                $html = mb_convert_encoding($html, 'UTF-8', 'UTF-8');
                $document->WriteHTML($html);
                $document->Output($uploadPath . $pdfFilePath, 'F');
                $dataArray = array();
                $dataArray['filename'] = $pdfFilePath;
                return response()->json(['status' => 1, 'message' => 'PDF downloaded successfully', 'data' => $dataArray], 200);
            } else {
                $api_data['message'] = 'Data Listed successfully';
                $api_data['status'] = 1;
                $api_data['data'] = $data['allParties'];
                $api_data['opening_balance'] = $data['opening_balance'];
                $api_data['total_commission'] = $data['total_commission'];
                $api_data['credit_sum'] = $data['credit_sum'];
                $api_data['debit_sum'] = $data['debit_sum'];
                return response()->json($api_data);
            }

        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => []], 200);
        }
    }


}