<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Http\Traits\DeviceTrait;
use App\Http\Traits\NotificationTrait;
use App\Http\Traits\PartyBalanceTrait;
use App\Http\Traits\PartyTrait;
use App\Http\Traits\PriceTrait;
use App\Http\Traits\VatavTrait;
use App\Models\LocalVatav;
use App\Models\User;
use App\Models\Vatav;
use Exception;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Config;
use Tymon\JWTAuth\Facades\JWTAuth;

class VatavController extends Controller
{
    use DeviceTrait;
    use NotificationTrait;
    use PriceTrait;
    use PartyTrait;
    use VatavTrait;
    use PartyBalanceTrait;

    public function __construct()
    {
        Config::set('jwt.user', User::class);
        Config::set('auth.providers', ['users' => [
            'driver' => 'eloquent',
            'model' => User::class,
        ]]);
    }

    public function calculateVatav(Request $request): JsonResponse
    {
        // return response()->json(['status' => 1, 'message' => 'Data listed successfully'], 200);
        // exit;

        try {
            $user = JWTAuth::parseToken()->authenticate();
            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => (object)[]], 200);
            }

            $amount = $request->input('amount');
            $is_consider_calculated_vatav = $request->input('is_consider_calculated_vatav');
            $calculate_vatav_on = $request->input('calculate_vatav_on');
            $receiver_party_id = $request->input('receiver_party_id') ?? NULL;

            $amount = $request->input('amount');
            $minimum = 0;
            $is_receiver_based = 0;
        
            if($receiver_party_id)
            {
                $partyDetail = $this->getPartyDetailsFromId($receiver_party_id);
                $is_receiver_based = $partyDetail->is_receiver_based;
                $is_receiver_based_commission = $partyDetail->receiver_based_commission;

                if ($is_receiver_based == 1) {
                    if ($amount <= 55000) {
                        $calculate_vatav_on = $this->generate_price_range($amount);
                        // if ($minimum == 0 || $minimum == '') {
                            $calculate_vatav_on = $calculate_vatav_on;
                        // } else {
                        //     $calculate_vatav_on = ($calculate_vatav_on * $minimum) / 100;
                        // }
                    } else {
                        if ($is_receiver_based_commission <= 0 || $is_receiver_based_commission == '') {
                            $calculate_vatav_on = $this->generate_price_range($amount);
                            // if ($minimum == 0 || $minimum == '') {
                                $calculate_vatav_on = $calculate_vatav_on;
                            // } else {
                            //     $calculate_vatav_on = ($calculate_vatav_on * $minimum) / 100;
                            // }
                        } else {
                            $calculate_vatav_on = ($amount * $is_receiver_based_commission) / 100000;
                            // if ($minimum == 0 || $minimum == '') {
                                $calculate_vatav_on = $calculate_vatav_on;
                            // } else {
                            //     $calculate_vatav_on = ($calculate_vatav_on * $minimum) / 100;
                            // }
                            $calculate_vatav_on = round($calculate_vatav_on / 10) * 10;
                        }
                    }
                } else {
                    $calculate_vatav_on = $this->generate_price_range($amount);
                    if ($minimum == 0 || $minimum == '') {
                        $calculate_vatav_on = $calculate_vatav_on;
                    } else {
                        $calculate_vatav_on = ($calculate_vatav_on * $minimum) / 100;
                    }
                }
            }
            else
            {
                if ($is_consider_calculated_vatav == 0) {
                $calculate_vatav_on = $this->generate_price_range($amount);
                }
            }

            // if ($is_consider_calculated_vatav == 0) {
            //     $calculate_vatav_on = $this->generate_price_range($amount);
            // }
            // $data['calculate_vatav_on'] = $calculate_vatav_on;
            // $data['our_vatav'] = 0;

            
            $data['calculate_vatav_on'] = $calculate_vatav_on;
            $data['our_vatav'] = 0;
            
            return response()->json(['status' => 1, 'message' => 'Data listed successfully', 'data' => $data], 200);

        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => (object)[]], 200);
        }
    }

    public function autovatavAdd(Request $request): JsonResponse
    {
        try {
            $user = JWTAuth::parseToken()->authenticate();

            if (!$user) {
                return response()->json(['status' => 0, 'message' => 'You are not able login from this application...', 'data' => (object)[]], 200);
            }

            $amount = $request->input('amount');
            $current_time = date('H:i:s');
            // $start_time = "09:00:00";
            // $end_time = "18:45:00";

            $start_time = "00:00:00";
            $end_time = "23:59:59";
            if ($amount < 1000) {
                return response()->json(['status' => 0, 'message' => 'Amount should be more than 1000', 'data' => (object)[]], 200);
            }

            if (strtotime($start_time) > strtotime($current_time)) {
                return response()->json(['status' => 0, 'message' => 'Office is not open so please add after some time', 'data' => (object)[]], 200);
            } elseif (strtotime($current_time) > strtotime($end_time)) {
                return response()->json(['status' => 0, 'message' => 'Office is closed', 'data' => (object)[]], 200);
            }

            $party_id = $request->input('party_id');

            $partyDetail = $this->getPartyDetailsFromId($party_id);

            if (!empty($partyDetail)) {
                if ($partyDetail->is_close == 1) {
                    return response()->json(['status' => 0, 'message' => 'You have not permission for Add', 'data' => (object)[]], 200);
                }
            }
            $sender = $request->input('sender');
            $receiver = $request->input('receiver');

            if (trim($sender) == trim($receiver)) {
                return response()->json(['status' => 0, 'message' => 'Please select sender and receiver different', 'data' => (object)[]], 200);
            }

            $entered_by = $request->input('entered_by');
            $sender_name = $request->input('sender_name');
            $sender_contact = $request->input('sender_contact');
            $receiver_name = $request->input('receiver_name');
            $receiver_contact = $request->input('receiver_contact');
            $remarks = $request->input('remarks');
            $token = $request->input('token');
            $entry_label = $request->input('entry_label');
            $calculate_vatav_on = $request->input('calculate_vatav_on');
            $s_adhar_number = $request->input('adhar_number');
            $local_vatav_id = $request->input('local_vatav_id') ?? NULL;

            $sendercode = explode(' (', $sender);
            $receivercode = explode(' (', $receiver);


            $sender_party_details = $this->getPartyDetailsFromCode($sendercode[0]);
            $receiver_party_details = $this->getPartyDetailsFromCode($receivercode[0]);

            $sender_party_id = $sender_party_details->id;
            $receiver_party_id = $receiver_party_details->id;

            $finalSender = $sender_party_details->code . ' (' . $sender_party_details->name . ')';
            $finalReceiver = $receiver_party_details->code . ' (' . $receiver_party_details->name . ')';

            $sender_commission_party_id = $sender_party_details->commission_party;
            $receiver_commission_party_id = $receiver_party_details->commission_party;


            $sender_partnership_firm = $sender_party_details->partnership_firm;
            $receiver_partnership_firm = $receiver_party_details->partnership_firm;

            $calculateData = $this->calculateAppAutoVatav($sender_party_details, $receiver_party_details, $amount, $calculate_vatav_on, $sender_party_id,$receiver_party_id);

            $our_vatav = number_format($calculateData['our_vatav'] / 100, "2", ".", "");
            $calculate_vatav_on = number_format($calculate_vatav_on / 100, "2", ".", "");
            $amount = number_format($amount / 100, "2", ".", "");
            $sender_vatav_amount = number_format($calculateData['sender_vatav_amount'] / 100, "2", ".", "");
            $receiver_vatav_amount = number_format($calculateData['receiver_vatav_amount'] / 100, "2", ".", "");
            $sender_total_amount = number_format($calculateData['sender_total_amount'] / 100, "2", ".", "");
            $receiver_total_amount = number_format((-1 * $calculateData['receiver_total_amount']) / 100, "2", ".", "");
            $sender_commission = number_format($calculateData['sender_commission_amount'] / 100, "2", ".", "");
            $receiver_commission = number_format($calculateData['receiver_commission_amount'] / 100, "2", ".", "");

            $serial_no = 1;
            $sender_token = "s" . $serial_no;
            $receiver_token = "r" . $serial_no;
            $receive_time = date('Y-m-d H:i:s');

            $sender_party_closing_balance = $this->getPartyClosingBalanceFromId($sender_party_id);
            $sender_transfer_limit = $sender_party_details->transfer_limit;
            $sender_per_label_limit = $sender_party_details->per_label_limit;
            // $receiver_per_label_limit = $receiver_party_details->per_label_limit;
            $sender_limit = 0;
//            $newAmount = str_replace(".", "", $amount);
            if ($sender_per_label_limit != "" && $sender_per_label_limit !=0){
                if($amount > $sender_per_label_limit) {
                    return response()->json(['status' => 0, 'message' => 'Entry Limit exceeded', 'data' => (object)[]], 200);
                }
            }


            if (!empty($sender_party_closing_balance)) {
                $sender_limit = $sender_party_closing_balance->closing_balance + $amount;
            } else {
                $sender_limit = 0 + $amount;
            }
            $is_limit = 1;

            if ($sender_transfer_limit != "" && $sender_transfer_limit != 0) {
                if ((float)$sender_limit > (float)$sender_transfer_limit) {
                    $is_limit = 0;
                }
            }

            if ($is_limit == 1) {
                $insertArray = array(
                    'date' => date('Y-m-d H:i:s'),
                    'serial_no' => $serial_no,
                    'sender' => $finalSender,
                    'receiver' => $finalReceiver,
                    'sender_party_id' => $sender_party_id,
                    'receiver_party_id' => $receiver_party_id,
                    'amount' => $amount,
                    'sender_total_amount' => $sender_total_amount,
                    'receiver_total_amount' => $receiver_total_amount,
                    'total_vatav_label' => $calculate_vatav_on,
                    's_vatav' => $sender_vatav_amount,
                    'r_vatav' => $receiver_vatav_amount,
                    'o_vatav' => $our_vatav,
                    'sender_commission_party_id' => $sender_commission_party_id,
                    'receiver_commission_party_id' => $receiver_commission_party_id,
                    'sender_commission_party_amount' => $sender_commission,
                    'receiver_commission_party_amount' => $receiver_commission,
                    'sender_is_partnership_firm' => $sender_partnership_firm,
                    'receiver_is_partnership_firm' => $receiver_partnership_firm,
                    'type' => 'auto',
                    'sender_name' => $sender_name,
                    'sender_contact' => $sender_contact,
                    'receiver_name' => $receiver_name,
                    'receiver_contact' => $receiver_contact,
                    'remarks' => $remarks,
                    'token' => $token,
                    'sender_token' => $sender_token,
                    'receiver_token' => $receiver_token,
                    'entered_by' => $entered_by,
                    'status' => 'Pending',
                    'label' => $entry_label,
                    'ho_approve' => $amount <= 3000.00 ? 1 : 0,
                    'is_deleted' => 0,
                    'receive_time' => $receive_time,
                    's_adhar_number' => $s_adhar_number,
                    'local_vatav_id' => $local_vatav_id,
                    'created_at' => $receive_time,
                    'updated_at' => $receive_time,
                    'device_type' => 'mobile',
                );

                $vatavId = Vatav::insertGetId($insertArray);
                if ($vatavId){
                    $updateArray = array(
                        'serial_no' => $vatavId,
                        'sender_token' => 's' . $vatavId,
                        'receiver_token' => 'r' . $vatavId,
                    );
                    $updateEntry = Vatav::where('id', $vatavId)->update($updateArray);

                    if ($local_vatav_id) {
                        $findEntry = LocalVatav::find($local_vatav_id);
                        $findEntry->status = "Forwarded";
                        $findEntry->save();
                        $findEntry->delete();
                    }

                    // Get Party Device token
                    if ($insertArray['ho_approve'] == 1){
                        $device_token = $this->getAllPartyDevices($receiver_party_id);
                        if (!empty($device_token)) {
                            $notification_arr['title'] = 'Receive Entry from: ' . $sender;
                            $notification_arr['body'] = 'You have received an entry of ' . $amount . " from " . $sender;
                            $notification_arr['type'] = 'receive_entry';
                            $notification_arr['msg'] = 'You have received an entry of ' . $amount . " from " . $sender;
                            $this->android_send_push_notification($notification_arr, $device_token);
                        }
                    }

                    // Get Device token
                    $userTokenArray = $this->getUserDeviceToken($sender_party_id);

                    $deviceTokenArray = array_unique($userTokenArray);

                    $notification_arr['title'] = 'New Entry from: ' . $sender . " to " . $receiver;
                    $notification_arr['body'] = 'New entry of ' . $amount . " from " . $sender . " to " . $receiver;
                    $notification_arr['type'] = 'New Entry';
                    $notification_arr['msg'] = 'New entry of ' . $amount . " from " . $sender . " to " . $receiver;
                    $notification_arr['data'] = $vatavId;
                    $this->web_send_push_notification($notification_arr, $deviceTokenArray);

                }


                return response()->json(['status' => 1, 'message' => 'Data inserted successfully', 'data' => (object)[]], 200);

            } else {
                return response()->json(['status' => 0, 'message' => 'Limit exceeded', 'data' => (object)[]], 200);
            }


        } catch (Exception $e) {
            return response()->json(['status' => 0, 'message' => $e->getMessage(), 'data' => (object)[]], 200);
        }
    }
}