<?php

namespace App\Http\Controllers\Auth;

use App\Http\Controllers\Controller;
use Illuminate\Contracts\View\Factory;
use Illuminate\Contracts\View\View;
use Illuminate\Foundation\Application;
use Illuminate\Http\RedirectResponse;
use Illuminate\Http\Request;
use Illuminate\Routing\Redirector;
use Illuminate\Support\Facades\Auth;
use App\Models\Company;
use App\Models\Modules;
use Illuminate\Support\Facades\Session;

class LoginController extends Controller
{
    public function __construct()
    {
    }

    public function index(): View|Application|Factory|\Illuminate\Contracts\Foundation\Application
    {
        return view('auth.login');
    }

    public function userLogin(Request $request)
    {
        $request->validate([
            'username' => 'required',
            'password' => 'required',
        ]);
        $token = $request->token;
        $password = $request->password;
        $email = $request->username;

        $credentials = ['username' => $email, 'password' => $password];
        if (Auth::attempt($credentials)) {
            $user = auth()->user();

            $company_user = Company::where('id', $user->id)->first();
            if ($company_user->status == 'inactive') {
                $request->session()->put('login_error', 'You can not login because you are Inactive..!!');
                return redirect("Admin/login")->withError('Login details are not valid');
            }
            $updateArray['is_loggedin'] = 1;
            $updateArray['token'] = $token;
            Company::where('id', $company_user->id)->update($updateArray);
            $permissionArray = explode(",", $company_user->permissions);
            $modulesList = Modules::get();

            $arr = array();
            foreach ($modulesList as $_modulesList) {
                if (in_array($_modulesList->id, $permissionArray)) {
                    $arr[] = $_modulesList->module_name;
                }

            }
            $request->session()->put('permissionArray', $arr);

            // $roleData = Roles::where('id', $company_user->role_id)->first();
            $party_ids = explode(",", $company_user->parties);

            $request->session()->put('access_party', $party_ids);
            $request->session()->put('role', $company_user->role);
            $request->session()->put('user_name', $company_user->username);
            return redirect()->intended('Admin/dashboard')->withSuccess('Signed in');
        }
        $request->session()->put('login_error', 'Incorrect username or password.');
        return redirect("Admin/login")->withError('Login details are not valid');
    }


    public function logout(Request $request): Application|Redirector|RedirectResponse|\Illuminate\Contracts\Foundation\Application
    {
        $user = auth()->user();
        $company_user = Company::where('id', $user->id)->first();
        $updateArray['is_loggedin'] = 0;
//        $updateArray['token'] = '';
        Company::where('id', $company_user->id)->update($updateArray);

        Session::flush();

        Auth::logout();

        return redirect('Admin/login');
    }

    public function updateToken(Request $request): void
    {
        $user = auth()->user();
        $token = $request->token;
        $user_id = $user->id;
        $company_user = Company::where('id', $user_id)->first();
        $updateArray['token'] = $token;
        $updateArray['device_type'] = 'web';
        Company::where('id', $company_user->id)->update($updateArray);

    }

}
