<?php

namespace App\Http\Traits;

use App\Models\Party;
use App\Models\PartyBalance;
use Illuminate\Support\Facades\DB;

trait PartyTrait
{
    use UserTrait;

    public function getPartyOpeningBalance($partyId, $date)
    {
        $partyBalance = PartyBalance::where(DB::raw("DATE(date)"), "<", $date)
            ->where('party_id', $partyId)->orderBy('date', 'DESC')->orderBy('id', 'DESC')->first();
        if (empty($partyBalance)) {
            $bal = 0;
        } else {
            $bal = $partyBalance->closing_balance;
        }
        return $bal;
    }

    public function get_party_opening_balance($partyId)
    {
        $partyBalance = PartyBalance::where('party_id', $partyId)->orderBy('date', 'DESC')->orderBy('id', 'DESC')->first();
        if (empty($partyBalance)) {
            $bal = 0;
        } else {
            $bal = $partyBalance->closing_balance;
        }
        return $bal;
    }

    public function getPartyBalanceFormPartyId($partyId)
    {
        return PartyBalance::where('party_id', $partyId)
            ->orderBy('date', 'DESC')
            ->orderBy('id', 'DESC')
            ->first();
    }

    public function getPartyDetailsFromId($id)
    {
        return Party::find($id);
    }

    public function getPartyForReport(): array
    {
        $accessParty = $this->getAccessParty();

        $query = "select distinct * from party m1 where not exists (select commission_party from party m2 where m2.commission_party = m1.id) AND m1.id IN($accessParty)  AND m1.is_deleted = 0 ORDER by m1.code ASC";
        $senderData = DB::select($query);

        $array = array();
        foreach ($senderData as $_senderData) {
            if ($_senderData->id != 1) {
                $array[] = array(
                    'id' => $_senderData->id,
                    'value' => $_senderData->code . ' (' . $_senderData->name . ')'
                );
            }
        }
        return $array;
    }

    public function getAccessParty()
    {
        $user_id = auth()->user()->id;
        $userDetail = $this->getCompanyDetails($user_id);
        return $userDetail->parties;
    }

    public function getSenderParty($party_ids, $role): array
    {
        $accessParty = $this->getAccessParty();

        $query = "select distinct * from party m1 where not exists (select commission_party from party m2 where m2.commission_party = m1.id) AND m1.id IN($accessParty)  AND m1.is_deleted = 0 ORDER by m1.code ASC";
        $senderData = DB::select($query);

        $array = array();
        foreach ($senderData as $_senderData) {
            if ($_senderData->id != 1) {
                $array[] = array(
                    'id' => $_senderData->id,
                    'value' => $_senderData->code . ' (' . $_senderData->name . ')'
                );
            }
        }
        return $array;
    }

    public function getReceiverParty(): array
    {
        $query = "select distinct * from party m1 where not exists (select commission_party from party m2 where m2.commission_party = m1.id) AND m1.is_deleted = 0 ORDER by m1.code ASC";
        $receiverData = DB::select($query);
        $receiverArray = array();
        foreach ($receiverData as $_receiverData) {
            if ($_receiverData->id != 1) {
                $receiverArray[] = array(
                    'id' => $_receiverData->id,
                    'value' => $_receiverData->code . ' (' . $_receiverData->name . ')'
                );
            }
        }
        return $receiverArray;
    }

    public function getSenderPartyOld($party_ids, $role): array
    {
        $senderData = Party::where('status', 'active')
            ->where('is_deleted', 0)
            ->orderBy('code', 'ASC')
            ->get();
        $array = array();
        foreach ($senderData as $_senderData) {
            if ($role !== 'superadmin') {
                if (in_array($_senderData->id, $party_ids)) {
                    if ($_senderData->id != 1) {
                        $array[] = array(
                            'id' => $_senderData->id,
                            'value' => $_senderData->code . ' (' . $_senderData->name . ')'
                        );
                    }
                }
            } else {
                if ($_senderData->id != 1) {
                    $array[] = array(
                        'id' => $_senderData->id,
                        'value' => $_senderData->code . ' (' . $_senderData->name . ')'
                    );
                }
            }
        }
        return $array;
    }

    public function getReceiverPartyOld(): array
    {
        $receiverData = Party::where('status', 'active')
            ->where('is_deleted', 0)
            ->orderBy('code', 'ASC')->get();
        $receiverArray = array();
        foreach ($receiverData as $_receiverData) {
            if ($_receiverData->id != 1) {
                $receiverArray[] = array(
                    'id' => $_receiverData->id,
                    'value' => $_receiverData->code . ' (' . $_receiverData->name . ')'
                );
            }
        }
        return $receiverArray;
    }

    public function getNormalSenderParty($party_ids, $role): array
    {
        $senderData = Party::where('status', 'active')
            ->where('is_deleted', 0)
            ->orderBy('code', 'ASC')
            ->get();
        $array = array();
        foreach ($senderData as $_senderData) {
            if ($role !== 'superadmin') {
                if (in_array($_senderData->id, $party_ids)) {
                    $array[] = array(
                        'id' => $_senderData->id,
                        'value' => $_senderData->code . ' (' . $_senderData->name . ')'
                    );
                }
            } else {
                $array[] = array(
                    'id' => $_senderData->id,
                    'value' => $_senderData->code . ' (' . $_senderData->name . ')'
                );
            }
        }
        return $array;
    }

    public function getNormalReceiverParty(): array
    {
        $receiverData = Party::where('status', 'active')
            ->where('is_deleted', 0)
            ->orderBy('code', 'ASC')->get();
        $receiverArray = array();
        foreach ($receiverData as $_receiverData) {
            $receiverArray[] = array(
                'id' => $_receiverData->id,
                'value' => $_receiverData->code . ' (' . $_receiverData->name . ')'
            );
        }
        return $receiverArray;
    }

    public function getPartyDetailsFromCode($code)
    {
        return Party::where('code', $code)->first();
    }

    public function getAllPartyWithCode(): array
    {
        $partyData = array();

        $partiesData = Party::where('is_deleted', 0)->get();

        foreach ($partiesData as $_partiesData) {
            $partyData[$_partiesData->id] = $_partiesData;
        }

        return $partyData;
    }

}